;+
;
;                      Tropospheric Emission
;                          Spectrometer
;                             (TES)
;
;                         Copyright 2007
;              California Institute of Technology.
;                       ALL RIGHTS RESERVED.
;           U.S. Government Sponsorship acknowledged.
;
;           Any commercial use must be negotiated with the
;           Office of Technology Transfer at the California
;           Institute of Technology.
;
; 
;  This software may be subject to U.S. export control laws. 
;  By accepting this software, the user agrees to comply with all 
;  applicable U.S. export laws and regulations. User has the 
;  responsibility to obtain export licenses, or other export 
;  authority as may be required before exporting such information 
;  to foreign countries or providing access to foreign persons.
;
;
;
; get_data_attributes
;
; PURPOSE:
; 	Returns the attributes fields for data and geolocation
;	fields in TES L2 HDF product files (version F01_01).
;	Note that this a support subroutine and should not
;	normally be directly called by a user.
;
; CALL:
;	get_data_sttributes,dataid,datatype,attributes,special=special
;
; INPUTS:
;	dataid = HDF ID pointer associated with the field that
;		to which the attributes are attached.
;	datatype = the IDL data type number (from size(var,/type))
;
; KEYWORD INPUT:
;	special = when set, do not read _FillValue attribute
;
; OUTPUT:
;	attributes = a structure of the attrubutes (see calling
;		programs for details)
;
; 
; ORIGINAL AUTHOR:
;	Brendan Fisher, JPL, 08/2005
;
; UPDATE HISTORY:
;	2006/08/22: added utctime keyword for F04 data
;	2007/12/20: added version keyword for F05 data
;
;-


pro get_data_attributes,dataid,datatype,attrib,special=special, $
  utctime=utctime,version=version

compile_opt idl2, hidden

if n_elements(special) eq 0 then special = 0
if n_elements(utctime) eq 0 then utctime = 0
if n_elements(version) eq 0 then version = 5

if utctime then special = 1

case datatype of 
     1 : holder = byte(0)
     2 : holder = fix(0)
     3 : holder = long(0)
     4 : holder = float(0)
     5 : holder = double(0)
     6 : holder = complex(0.,0.)
     7 : holder = '' ; string
     8 : holder = 0 ; structure type--should not get!
     9 : holder = dcomplex(0.,0.)
    10 : holder = 0 ; pointer type--should not get!
    11 : holder = 0 ; object ref type--should not get!
    12 : holder = uint(0)
    13 : holder = ulong(0)
    14 : holder = long64(0)
    15 : holder = ulong64(0)
    else : holder = 0
endcase

attrib = { _FillValue : holder, $
           MissingValue : holder, $
           Title : '', $
           Units : '', $
           UniqueFieldDefinition : '' }

if not special then begin
    id = h5a_open_name(dataid,'_FillValue')
    attrib._FillValue = h5a_read(id)
    h5a_close,id
endif

id = h5a_open_name(dataid,'MissingValue')
attrib.MissingValue = h5a_read(id)
h5a_close,id

id = h5a_open_name(dataid,'Title')
attrib.Title = h5a_read(id)
h5a_close,id

id = h5a_open_name(dataid,'Units')
attrib.Units = h5a_read(id)
h5a_close,id

id = h5a_open_name(dataid,'UniqueFieldDefinition')
attrib.UniqueFieldDefinition = h5a_read(id)
h5a_close,id

if utctime then begin
    id = h5a_open_name(dataid,'ARRAYOFSTRINGS')
    arrayofstrings = h5a_read(id)
    h5a_close,id
    
    
    if version le 4 then begin 
        id= h5a_open_name(dataid,'StringLengthAttribute')
        StringLengthAttribute = h5a_read(id)
        h5a_close,id
    endif else StringLengthAttribute = 0

    attrib2 = { ARRAYOFSTRINGS : ARRAYOFSTRINGS, $
                MissingValue : attrib.missingvalue, $
                Title : attrib.title, $
                Units : attrib.units, $
                UniqueFieldDefinition : attrib.UniqueFieldDefinition, $
                StringLengthAttribute : StringLengthAttribute $
          }
    attrib = attrib2
endif

return

end
