;+
;
;                      Tropospheric Emission
;                          Spectrometer
;                             (TES)
;
;                         Copyright 2008,2009,2010,2011
;              California Institute of Technology.
;                       ALL RIGHTS RESERVED.
;           U.S. Government Sponsorship acknowledged.
;
;           Any commercial use must be negotiated with the
;           Office of Technology Transfer at the California
;           Institute of Technology.
;
; 
;  This software may be subject to U.S. export control laws. 
;  By accepting this software, the user agrees to comply with all 
;  applicable U.S. export laws and regulations. User has the 
;  responsibility to obtain export licenses, or other export 
;  authority as may be required before exporting such information 
;  to foreign countries or providing access to foreign persons.
;
;
; read_l2_hdf_ancillary
;
; PURPOSE:
;	Read a TES L2 ancillary HDF product file and
;	returns the ancillary data fields and geolocation data for data
;	versions F01, F02, F03, F04, F05, F06.  Data are returned in simple
;	structures of simple arrays with no processing, filtering,
;	etc.  
;       NOTE:  all data are returned in keywords since the nadir and
;       limb swaths only appear as dictated by the data content.
;
;
; CALL: 
; 	read_l2_hdf_ancillary,file,
; 	  nadir_ancillary_data = nadir_ancillary_data,
;	  limb_ancillary_data = limb_ancillary_data,
; 	  nadir_geo_data = nadir_geo_data, $
; 	  limb_geo_data = limb_geo_data, $
;	  nadir_ancillary_attributes = nadir_ancillary_attributes, $
;	  limb_ancillary_attributes = limb_ancillary_attributes, $
;	  nadir_geo_attributes = nadir_geo_attributes, $
;	  limb_geo_attributes = limb_geo_attributes, $
;	  file_attributes = file_attributes,
;	  hdfeos_data = hdfeos_data, $
;	  status = status, read_ph=read_ph
;
; INPUTS:
;	file = name of file to be read (string)
;
; KEYWORD INPUT:
;	read_ph = if set, read production history.  If not set, the
;		production history will be a null string
;
;
; OUTPUTS:
;	[nadir or limb]_ancillary_data = a structure containing all of the data fields
;		       found in the ancillary swath group.  NOTE: This list
;		       combines the nadir and limb into one list
;		       though they are returned separately.   These
;		       fields are:
;               
;               Filter_Position_1A
;               Filter_Position_1B
;               Filter_Position_2A
;               Filter_Position_2B
;		H2O_HDOAveragingKernel (F05+)
;		HDO_H2OAveragingKernel (F05+)
;		HDO_H2ORatioMeasurementErrorCovariance (F05+)
;		HDO_H2ORatioSystematicErrorCovariance (F05+)
;		HDO_H2ORatioTotalErrorCovariance (F05+)
;               H2O_TAveragingKernel (F01-F04)
;               LandSurfaceEmissErrors (F01,F02) (Nadir only)
;               LandSurfaceEmissivity (F01,F02) (Nadir only)
;               OrbitAscendingFlag
;		OzoneTroposphericColumn (F03-F05) (Nadir only)
;		OzoneTroposphericColumnError (F03-F05) (Nadir only)
;		OzoneTroposphericColumnInitial (F03-F05) (Nadir only)
;               PixelsUsedFlag
;		RetrievedPointingAngle (F03+) (Limb Only)
;		RetrievedPointingAngleError (F03+) (Limb Only)
;               SolarAzimuthAngle
;               SpacecraftAltitude
;               SpacecraftLatitude
;               SpacecraftLongitude
;		SurfaceEmissConstraint (F03+) (Nadir only)
;		SurfaceEmissErrors (F03+) (Nadir only)
;		SurfaceEmissInitial (F03+) (Nadir only)
;		SurfaceEmissivity (F03+) (Nadir only)
;		SurfaceTempInitialRefined (F05+) (nadir only) 
;               T_H2OAveragingKernel (F01-F04)
;               T_H2OCovariance (F01-F04)
;		TropopausePressure (F03+)
;	    	UTCTime (F04+) 	 
;               VerticalCoordinate    (group attribute)
;	        CloudFrequency (F02+)   (group attribute)
;               EmissivityWavenumber  (F02+ group attribute, in F01 it is
;				       data field with attributes)
;		Scan_Resolution (F03+)
;               Pressure        (group attribute)
;
;		In this context, group attributes do not have
;		associated attriutes (see below) like the data fields.
;
;	[nadir or limb]_geo_data = a structure containing all of the geolocation
;	fields found in the species swath group.  These fields
;	are :
;
;           x BoresightAzimuth
;	    x BoresightNadirAngle
;           x BoresightNadirAngleUnc
;	    BoresightTangentHeight (F03+) (Limb Only)
;	    BoresightTangentHeightUnc (F03+) (Limb Only)
;	    DayNightFlag (F03+)
;	    DominantSurfaceType (F03+) (Nadir Only)
;           Latitude
;           Latitude_Footprint_1
;           Latitude_Footprint_2
;           Latitude_Footprint_3
;           Latitude_Footprint_4
;           xLocalSolarTime
;           Longitude
;           Longitude_Footprint_1
;           Longitude_Footprint_2
;           Longitude_Footprint_3
;           Longitude_Footprint_4
;	    Scan (F02+)
;	    Sequence (F02+)
;           SolarZenithAngle
;           SurfaceElevStandardDeviation
;	    SurfaceTypeFootprint (F03+)
;           Tgt_SpacecraftAzimuth
;           Tgt_SpacecraftZenith
;           Time
;	    UTCTime (F03) Note: in the HDF file this is stored as an
;	                 27xN array (where N is the number of observations
;	                 in this file) and 27 is the number of
;	                 characters in the string.  This program concatinates
;	                 all of the characters into a single string
;	                 and returns a string array of N length.
;			This entry has no _FillValue field in its attributes
;
;
; KEYWORD OUTPUT:
;	[nadir or limb]_ancillary_attributes = a structure of attributes structures, with
;		one entry for each of the species data fields.  An attributes
;		structure containts the following fields:
;
;		_FillValue
;		MissingValue
;		Title
;		Units
;	 	UniqueFieldDefinition
;	 
;
;		NOTE: the attributes for the PixelsUsedFlag and
;		UTCTime do not contain the _FillValue field.
; 
;
;	[nadir or limb]_geo_attributes = structure of attributes structues, with
;		one entry for each of the geolocation data fields.  
;		An attributes structure containts the following fields:
;
;		_FillValue
;		MissingValue
;		Title
;		Units
;	 	UniqueFieldDefinition
; 
;	file_attributes = a structure containing the information
;		contained in the file attributes group in the
;		additional group.  The fields in this structure are:
;                  ProductionHistory 
;                  InstrumentName 
;                  ProcessLevel 
;                  TAI93AtOzOfGranule (F01) (returned as TAI93At0zOfGranule)
;                  TAI93At0zOfGranule (F02+)
;                  GlobalSurveyNumber
;                  GranuleMonth
;                  GranuleDay
;                  GranuleYear
;                  SurveyMode
;                  PGEVersion
;		   Command_Seq_ID (F02+)
;
;	hdfeos_data = a structure that contains the information found 
;		in the HDFEOS Information group.  The fields are:
;              		StructMetadata 
;			coremetadata
;			HDFEOSVersion
;
;	status = Contains success or failure status of the read
;		 0 for success
;		 1 if file does not exist
;		 2 the file name does not have F01, F02, or F03 in name
;		 3 the file is not HDF 5
;		 4 (obsolete)
;		 5 the file name does not have the expected .he5 extension
;		 6 File is not ANCILLARY  file
;		 7 IDL version incompatability
;		 
; ORIGINAL AUTHOR:
;	Brendan Fisher, JPL, 08/2005
;
; UPDATE HISTORY:
;	11/2005, Brendan Fisher, JPL
;	-updated to read F02 files
;	01/2006, Brendan Fisher, JPL.
;	-updated for F03 files
;	-Added Limb functionality & changed interface accordingly
;	-Changed version checking to check version part of filename
;	only (ignor science content version number)
;	2006/08/22, Brendan Fisher, JPL, started update to F04
;	-fixed some older bugs related to versioning
;	2010/04, Brendan Fisher, JPL, added F06 capability
;
;-
pro read_l2_hdf_ancillary,file,$
 	  nadir_ancillary_data = nadir_ancillary_data, $
	  limb_ancillary_data = limb_ancillary_data, $
 	  nadir_geo_data = nadir_geo_data, $
 	  limb_geo_data = limb_geo_data, $
	  nadir_ancillary_attributes = nadir_ancillary_attributes, $
	  limb_ancillary_attributes = limb_ancillary_attributes, $
	  nadir_geo_attributes = nadir_geo_attributes, $
	  limb_geo_attributes = limb_geo_attributes, $
	  file_attributes = file_attributes, $
	  hdfeos_data = hdfeos_data, $
	  status = status,read_ph=read_ph


status=0
if n_elements(read_ph) eq 0 then read_ph = 0


; check for file existence
ff = file_search(file,count=count)
if count eq 0 then begin
    print,'Cannot find file:'
    print,file
    status = 1
    return
endif
; check type
apos = strpos(file,'ANCILLARY')
if apos eq -1 then begin
    print,'This file is not an ANCILLARY product files.'
    print,'File name: ',file
    status=6
    return
end
; check version
pos1 = strpos(file,'_F01_')
pos2 = strpos(file,'_F02_')
pos3 = strpos(file,'_F03_')
pos4 = strpos(file,'_F04_')
pos5 = strpos(file,'_F05_')
pos6 = strpos(file,'_F06_')
if pos1 eq -1 and pos2 eq -1 and pos3 eq -1 and pos4 eq -1 $
  and pos5 eq -1 then begin
    print,'Expected version _F01_, _F02_, _F03_,  _F04_,  _F05_ or _F06_  in file name not found.'
    print,file
    print,'This file format not supported.'
    status = 2
    return
endif

if pos1 ne -1 then version = 1
if pos2 ne -1 then version = 2
if pos3 ne -1 then version = 3
if pos4 ne -1 then version = 4
if pos5 ne -1 then version = 5
if pos6 ne -1 then version = 6
; check to see if file is an HDF5 
if not h5f_is_hdf5(file) then begin
    print,'File is not HDF 5:'
    print,file
    status=3
    return
endif
;check for .he5 extension
;
len = strlen(file)
exten = strmid(file,len-3,3)
if exten ne 'he5' then begin
    print,'Expected .he5 file name extension not found.'
    print,file
    status=5
    return
endif
; IDL version checking
iversion = !version
idl_version = float(iversion.release)
if version ge 4 and idl_version lt 6.3 then begin
    print,'IDL v6.3+ required for TES format version F04 files.'
    status = 7
    return
endif
if idl_version lt 6.1 and version le 3 then begin
    print,'IDL v6.1+ required for TES format version F01, F02, and F03 files.'
    status = 7
    return
endif
;
;
;
; open file to data fields
;
fid = h5f_open(file)
hdfid = h5g_open(fid,'HDFEOS')
swid = h5g_open(hdfid,'SWATHS')

; check which swaths are in file
ngroup = h5g_get_nmembers(fid,'/HDFEOS/SWATHS/')
names = strarr(ngroup)
for i=0,ngroup-1 do names[i] = h5g_get_member_name(fid,'/HDFEOS/SWATHS/',i)
dn = 0
gn = 0

nad = where(names eq 'AncillaryNadirSwath')
if nad ne -1 then begin ; check to see if populated
    dn = h5g_get_nmembers(fid,'/HDFEOS/SWATHS/AncillaryNadirSwath/Data Fields')
    gn = h5g_get_nmembers(fid,'/HDFEOS/SWATHS/AncillaryNadirSwath/Geolocation Fields')
endif

if dn ne 0 and gn ne 0 then begin
    specid = h5g_open(swid,'AncillaryNadirSwath')
; data fields
    fieldid = h5g_open(specid,'Data Fields')
;
; actual data, alphabetically
;
    if version eq 1 then begin
        dataid = h5d_open(fieldid,'EmissivityWavenumber')
        EmissivityWavenumber = h5d_read(dataid)
        get_data_attributes,dataid,size(EmissivityWavenumber,/type),attrib0
        h5d_close,dataid
    endif

    dataid = h5d_open(fieldid,'Filter_Position_1A')
    Filter_Position_1A = h5d_read(dataid)
    get_data_attributes,dataid,size(Filter_Position_1A,/type),attrib1
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'Filter_Position_1B')
    Filter_Position_1B = h5d_read(dataid)
    get_data_attributes,dataid,size(Filter_Position_1B,/type),attrib2
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Filter_Position_2A')
    Filter_Position_2A = h5d_read(dataid)
    get_data_attributes,dataid,size(Filter_Position_2A,/type),attrib3
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'Filter_Position_2B')
    Filter_Position_2B = h5d_read(dataid)
    get_data_attributes,dataid,size(Filter_Position_2B,/type),attrib4
    h5d_close,dataid

    if version ge 5 then begin
        dataid = h5d_open(fieldid,'H2O_HDOAveragingKernel')
        H2O_HDOAveragingKernel = h5d_read(dataid)
        get_data_attributes,dataid,size(H2O_HDOAveragingKernel,/type),attrib5a
        h5d_close,dataid

        dataid = h5d_open(fieldid,'HDO_H2OAveragingKernel')
        HDO_H2OAveragingKernel = h5d_read(dataid)
        get_data_attributes,dataid,size(HDO_H2OAveragingKernel,/type),attrib5b
        h5d_close,dataid

        dataid = h5d_open(fieldid,'HDO_H2ORatioMeasurementErrorCovariance')
        HDO_H2ORatioMeasurementErrorCovariance = h5d_read(dataid)
        get_data_attributes,dataid,size(HDO_H2ORatioMeasurementErrorCovariance,/type),attrib5c
        h5d_close,dataid

        dataid = h5d_open(fieldid,'HDO_H2ORatioSystematicErrorCovariance')
        HDO_H2ORatioSystematicErrorCovariance = h5d_read(dataid)
        get_data_attributes,dataid,size(HDO_H2ORatioSystematicErrorCovariance,/type),attrib5d
        h5d_close,dataid

        dataid = h5d_open(fieldid,'HDO_H2ORatioTotalErrorCovariance')
        HDO_H2ORatioTotalErrorCovariance = h5d_read(dataid)
        get_data_attributes,dataid,size(HDO_H2ORatioTotalErrorCovariance,/type),attrib5e
        h5d_close,dataid
    endif
    
    if version le 4 then begin
        dataid = h5d_open(fieldid,'H2O_TAveragingKernel')
        H2O_TAveragingKernel = h5d_read(dataid)
        get_data_attributes,dataid,size(H2O_TAveragingKernel,/type),attrib5
        h5d_close,dataid
    endif
    
    if version le 2 then begin
        dataid = h5d_open(fieldid,'LandSurfaceEmissErrors')
        LandSurfaceEmissErrors = h5d_read(dataid)
        get_data_attributes,dataid,size(LandSurfaceEmissErrors,/type),attrib6
        h5d_close,dataid
    
        dataid = h5d_open(fieldid,'LandSurfaceEmissivity')
        LandSurfaceEmissivity = h5d_read(dataid)
        get_data_attributes,dataid,size(LandSurfaceEmissivity,/type),attrib7
        h5d_close,dataid
    endif
;    
    dataid = h5d_open(fieldid,'OrbitAscendingFlag')
    OrbitAscendingFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(OrbitAscendingFlag,/type),attrib8
    h5d_close,dataid
; 
    if version ge 3 and version le 5 then begin

        dataid = h5d_open(fieldid,'OzoneTroposphericColumn')
        OzoneTroposphericColumn = h5d_read(dataid)
        get_data_attributes,dataid,size(OzoneTroposphericColumn,/type),attrib9
        h5d_close,dataid
        
        dataid = h5d_open(fieldid,'OzoneTroposphericColumnError')
        OzoneTroposphericColumnError = h5d_read(dataid)
        get_data_attributes,dataid,size(OzoneTroposphericColumnError,/type),attrib10
        h5d_close,dataid
        
        dataid = h5d_open(fieldid,'OzoneTroposphericColumnInitial')
        OzoneTroposphericColumnInitial = h5d_read(dataid)
        get_data_attributes,dataid,size(OzoneTroposphericColumnInitial,/type),attrib11
        h5d_close,dataid
    endif
   
    dataid = h5d_open(fieldid,'PixelsUsedFlag')
    PixelsUsedFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(PixelsUsedFlag,/type),attrib12,/special
    h5d_close,dataid
; (limb fields)   
    dataid = h5d_open(fieldid,'SolarAzimuthAngle')
    SolarAzimuthAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(SolarAzimuthAngle,/type),attrib13
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'SpacecraftAltitude')
    SpacecraftAltitude = h5d_read(dataid)
    get_data_attributes,dataid,size(SpacecraftAltitude,/type),attrib14
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'SpacecraftLatitude')
    SpacecraftLatitude = h5d_read(dataid)
    get_data_attributes,dataid,size(SpacecraftLatitude,/type),attrib15
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'SpacecraftLongitude')
    SpacecraftLongitude = h5d_read(dataid)
    get_data_attributes,dataid,size(SpacecraftLongitude,/type),attrib16
    h5d_close,dataid

    if version ge 3 then begin
        dataid = h5d_open(fieldid,'SurfaceEmissConstraint')
        SurfaceEmissConstraint = h5d_read(dataid)
        get_data_attributes,dataid,size(SurfaceEmissConstraint,/type),attrib17
        h5d_close,dataid
    
        dataid = h5d_open(fieldid,'SurfaceEmissErrors')
        SurfaceEmissErrors = h5d_read(dataid)
        get_data_attributes,dataid,size(SurfaceEmissErrors,/type),attrib18
        h5d_close,dataid

        dataid = h5d_open(fieldid,'SurfaceEmissInitial')
        SurfaceEmissInitial = h5d_read(dataid)
        get_data_attributes,dataid,size(SurfaceEmissInitial,/type),attrib19
        h5d_close,dataid

        dataid = h5d_open(fieldid,'SurfaceEmissivity')
        SurfaceEmissivity = h5d_read(dataid)
        get_data_attributes,dataid,size(SurfaceEmissivity,/type),attrib20
        h5d_close,dataid
    endif

    if version ge 5 then begin
        dataid = h5d_open(fieldid,'SurfaceTempInitialRefined')
        SurfaceTempInitialRefined = h5d_read(dataid)
        get_data_attributes,dataid,size(SurfaceTempInitialRefined,/type),attrib20a
        h5d_close,dataid
    endif

    if version le 4 then begin
        dataid = h5d_open(fieldid,'T_H2OAveragingKernel')
        T_H2OAveragingKernel = h5d_read(dataid)
        get_data_attributes,dataid,size(T_H2OAveragingKernel,/type),attrib21
        h5d_close,dataid
    
        dataid = h5d_open(fieldid,'T_H2OCovariance')
        T_H2OCovariance = h5d_read(dataid)
        get_data_attributes,dataid,size(T_H2OCovariance,/type),attrib22
        h5d_close,dataid
    endif

    if version ge 3 then begin
        dataid = h5d_open(fieldid,'TropopausePressure')
        TropopausePressure = h5d_read(dataid)
        get_data_attributes,dataid,size(TropopausePressure,/type),attrib23
        h5d_close,dataid
    endif
    if version ge 4 then begin
        dataid = h5d_open(fieldid,'UTCTime')
        UTCTime = h5d_read(dataid)
        get_data_attributes,dataid,size(utctime,/type),attrib_utc,/utctime,version=version
        h5d_close,dataid
    endif
;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressure = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    if version ge 2 then begin
        id = h5a_open_name(specid,'CloudFrequency')
        CloudFrequency = h5a_read(id)
        h5a_close,id

        id = h5a_open_name(specid,'EmissivityWavenumber')
        EmissivityWavenumber = h5a_read(id)
        h5a_close,id
    endif else begin
        cloudfrequency = [0]
    endelse
    id = h5a_open_name(specid,'Scan_Resolution')
    Scan_Resolution = h5a_read(id)
    h5a_close,id

;
; put all species data into return structure
;
    if version eq 1 then begin
        nadir_ancillary_data = { $
               Filter_Position_1A : temporary(Filter_Position_1A), $
               Filter_Position_1B : temporary(Filter_Position_1B), $
               Filter_Position_2A : temporary(Filter_Position_2A), $
               Filter_Position_2B : temporary(Filter_Position_2B), $
               H2O_TAveragingKernel : temporary(H2O_TAveragingKernel), $
               LandSurfaceEmissErrors : temporary(LandSurfaceEmissErrors), $
               LandSurfaceEmissivity : temporary(LandSurfaceEmissivity), $
               OrbitAscendingFlag : temporary(OrbitAscendingFlag), $
               PixelsUsedFlag : temporary(PixelsUsedFlag), $
               SolarAzimuthAngle : temporary(SolarAzimuthAngle), $
               SpacecraftAltitude : temporary(SpacecraftAltitude), $
               SpacecraftLatitude : temporary(SpacecraftLatitude), $
               SpacecraftLongitude : temporary(SpacecraftLongitude), $
               T_H2OAveragingKernel : temporary(T_H2OAveragingKernel), $
               T_H2OCovariance : temporary(T_H2OCovariance), $
               pressure : temporary(pressure), $
               VerticalCoordinate : temporary(VerticalCoordinate), $
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               Scan_Resolution : temporary(Scan_Resolution) $
               }
;
; put all attribute data into return structure
;
        nadir_ancillary_attributes = { $
                     EmissivityWavenumber : temporary(attrib0), $
                     Filter_Position_1A : temporary(attrib1), $
                     Filter_Position_1B : temporary(attrib2), $
                     Filter_Position_2A : temporary(attrib3), $
                     Filter_Position_2B : temporary(attrib4), $
                     H2O_TAveragingKernel : temporary(attrib5), $
                     LandSurfaceEmissErrors : temporary(attrib6),$
                     LandSurfaceEmissivity : temporary(attrib7), $
                     OrbitAscendingFlag : temporary(attrib8), $
                     PixelsUsedFlag : temporary(attrib12), $
                     SolarAzimuthAngle : temporary(attrib13), $
                     SpacecraftAltitude : temporary(attrib14), $
                     SpacecraftLatitude : temporary(attrib15), $
                     SpacecraftLongitude : temporary(attrib16), $
                     T_H2OAveragingKernel : temporary(attrib21), $
                     T_H2OCovariance : temporary(attrib22) $
                     }
    endif 
    if version eq 2 then begin
        nadir_ancillary_data = { $
               Filter_Position_1A : temporary(Filter_Position_1A), $
               Filter_Position_1B : temporary(Filter_Position_1B), $
               Filter_Position_2A : temporary(Filter_Position_2A), $
               Filter_Position_2B : temporary(Filter_Position_2B), $
               H2O_TAveragingKernel : temporary(H2O_TAveragingKernel), $
               LandSurfaceEmissErrors : temporary(LandSurfaceEmissErrors), $
               LandSurfaceEmissivity : temporary(LandSurfaceEmissivity), $
               OrbitAscendingFlag : temporary(OrbitAscendingFlag), $
               PixelsUsedFlag : temporary(PixelsUsedFlag), $
               SolarAzimuthAngle : temporary(SolarAzimuthAngle), $
               SpacecraftAltitude : temporary(SpacecraftAltitude), $
               SpacecraftLatitude : temporary(SpacecraftLatitude), $
               SpacecraftLongitude : temporary(SpacecraftLongitude), $
               T_H2OAveragingKernel : temporary(T_H2OAveragingKernel), $
               T_H2OCovariance : temporary(T_H2OCovariance), $
               pressure : temporary(pressure), $
               VerticalCoordinate : temporary(VerticalCoordinate), $
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               CloudFrequency : temporary(CloudFrequency), $
               Scan_Resolution : temporary(Scan_Resolution)  $
               }
;
; put all attribute data into return structure
;
        nadir_ancillary_attributes = { $
                     Filter_Position_1A : temporary(attrib1), $
                     Filter_Position_1B : temporary(attrib2), $
                     Filter_Position_2A : temporary(attrib3), $
                     Filter_Position_2B : temporary(attrib4), $
                     H2O_TAveragingKernel : temporary(attrib5), $
                     LandSurfaceEmissErrors : temporary(attrib6),$
                     LandSurfaceEmissivity : temporary(attrib7), $
                     OrbitAscendingFlag : temporary(attrib8), $
                     PixelsUsedFlag : temporary(attrib12), $
                     SolarAzimuthAngle : temporary(attrib13), $
                     SpacecraftAltitude : temporary(attrib14), $
                     SpacecraftLatitude : temporary(attrib15), $
                     SpacecraftLongitude : temporary(attrib16), $
                     T_H2OAveragingKernel : temporary(attrib21), $
                     T_H2OCovariance : temporary(attrib22) $
                     }
    endif 

    if version eq 3 then begin
        nadir_ancillary_data = { $
               Filter_Position_1A : temporary(Filter_Position_1A), $
               Filter_Position_1B : temporary(Filter_Position_1B), $
               Filter_Position_2A : temporary(Filter_Position_2A), $
               Filter_Position_2B : temporary(Filter_Position_2B), $
               H2O_TAveragingKernel : temporary(H2O_TAveragingKernel), $
               OrbitAscendingFlag : temporary(OrbitAscendingFlag), $
               OzoneTroposphericColumn : temporary(OzoneTroposphericColumn), $
               OzoneTroposphericColumnError : temporary(OzoneTroposphericColumnError), $
               OzoneTroposphericColumnInitial : temporary(OzoneTroposphericColumnInitial), $
               PixelsUsedFlag : temporary(PixelsUsedFlag), $
               SolarAzimuthAngle : temporary(SolarAzimuthAngle), $
               SpacecraftAltitude : temporary(SpacecraftAltitude), $
               SpacecraftLatitude : temporary(SpacecraftLatitude), $
               SpacecraftLongitude : temporary(SpacecraftLongitude), $
               SurfaceEmissConstraint : temporary(SurfaceEmissConstraint), $
               SurfaceEmissErrors : temporary(SurfaceEmissErrors), $
               SurfaceEmissInitial : temporary(SurfaceEmissInitial), $
               SurfaceEmissivity : temporary(SurfaceEmissivity), $
               T_H2OAveragingKernel : temporary(T_H2OAveragingKernel), $
               T_H2OCovariance : temporary(T_H2OCovariance), $
               TropopausePressure : temporary(TropopausePressure), $
               pressure : temporary(pressure), $
               VerticalCoordinate : temporary(VerticalCoordinate), $
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               CloudFrequency : temporary(CloudFrequency), $
               Scan_Resolution : temporary(Scan_Resolution)  $
               }
;
; put all attribute data into return structure
;
        nadir_ancillary_attributes = { $
                     Filter_Position_1A : temporary(attrib1), $
                     Filter_Position_1B : temporary(attrib2), $
                     Filter_Position_2A : temporary(attrib3), $
                     Filter_Position_2B : temporary(attrib4), $
                     H2O_TAveragingKernel : temporary(attrib5), $
                     OrbitAscendingFlag : temporary(attrib8), $
          	     OzoneTroposphericColumn : temporary(attrib9), $
          	     OzoneTroposphericColumnError : temporary(attrib10), $
          	     OzoneTroposphericColumnInitial : temporary(attrib11), $
                     PixelsUsedFlag : temporary(attrib12), $
                     SolarAzimuthAngle : temporary(attrib13), $
                     SpacecraftAltitude : temporary(attrib14), $
                     SpacecraftLatitude : temporary(attrib15), $
                     SpacecraftLongitude : temporary(attrib16), $
          	     SurfaceEmissConstraint : temporary(attrib17), $
          	     SurfaceEmissErrors : temporary(attrib18), $
          	     SurfaceEmissInitial : temporary(attrib19), $
          	     SurfaceEmissivity : temporary(attrib20), $
                     T_H2OAveragingKernel : temporary(attrib21), $
                     T_H2OCovariance : temporary(attrib22), $
                     TropopausePressure : temporary(attrib23) $
                     }
    endif 

   if version eq 4 then begin
        nadir_ancillary_data = { $
               Filter_Position_1A : temporary(Filter_Position_1A), $
               Filter_Position_1B : temporary(Filter_Position_1B), $
               Filter_Position_2A : temporary(Filter_Position_2A), $
               Filter_Position_2B : temporary(Filter_Position_2B), $
               H2O_TAveragingKernel : temporary(H2O_TAveragingKernel), $
               OrbitAscendingFlag : temporary(OrbitAscendingFlag), $
               OzoneTroposphericColumn : temporary(OzoneTroposphericColumn), $
               OzoneTroposphericColumnError : temporary(OzoneTroposphericColumnError), $
               OzoneTroposphericColumnInitial : temporary(OzoneTroposphericColumnInitial), $
               PixelsUsedFlag : temporary(PixelsUsedFlag), $
               SolarAzimuthAngle : temporary(SolarAzimuthAngle), $
               SpacecraftAltitude : temporary(SpacecraftAltitude), $
               SpacecraftLatitude : temporary(SpacecraftLatitude), $
               SpacecraftLongitude : temporary(SpacecraftLongitude), $
               SurfaceEmissConstraint : temporary(SurfaceEmissConstraint), $
               SurfaceEmissErrors : temporary(SurfaceEmissErrors), $
               SurfaceEmissInitial : temporary(SurfaceEmissInitial), $
               SurfaceEmissivity : temporary(SurfaceEmissivity), $
               T_H2OAveragingKernel : temporary(T_H2OAveragingKernel), $
               T_H2OCovariance : temporary(T_H2OCovariance), $
               TropopausePressure : temporary(TropopausePressure), $
               UTCTime : temporary(UTCTime), $
               pressure : temporary(pressure), $
               VerticalCoordinate : temporary(VerticalCoordinate), $
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               CloudFrequency : temporary(CloudFrequency), $
               Scan_Resolution : temporary(Scan_Resolution)  $
               }
;
; put all attribute data into return structure
;
        nadir_ancillary_attributes = { $
                     Filter_Position_1A : temporary(attrib1), $
                     Filter_Position_1B : temporary(attrib2), $
                     Filter_Position_2A : temporary(attrib3), $
                     Filter_Position_2B : temporary(attrib4), $
                     H2O_TAveragingKernel : temporary(attrib5), $
                     OrbitAscendingFlag : temporary(attrib8), $
          	     OzoneTroposphericColumn : temporary(attrib9), $
          	     OzoneTroposphericColumnError : temporary(attrib10), $
          	     OzoneTroposphericColumnInitial : temporary(attrib11), $
                     PixelsUsedFlag : temporary(attrib12), $
                     SolarAzimuthAngle : temporary(attrib13), $
                     SpacecraftAltitude : temporary(attrib14), $
                     SpacecraftLatitude : temporary(attrib15), $
                     SpacecraftLongitude : temporary(attrib16), $
          	     SurfaceEmissConstraint : temporary(attrib17), $
          	     SurfaceEmissErrors : temporary(attrib18), $
          	     SurfaceEmissInitial : temporary(attrib19), $
          	     SurfaceEmissivity : temporary(attrib20), $
                     T_H2OAveragingKernel : temporary(attrib21), $
                     T_H2OCovariance : temporary(attrib22), $
          	     TropopausePressure : temporary(attrib23), $ 
                     UTCTime : temporary(attrib_utc) $
                     }
    endif 

;		H2O_HDOAveragingKernel (F05+)
;		HDO_H2OAveragingKernel (F05+)
;		HDO_H2ORatioMeasurementErrorCovariance (F05+)
;		HDO_H2ORatioSystematicErrorCovariance (F05+)
;		HDO_H2ORatioTotalErrorCovariance (F05+)
   if version eq 5 then begin
        nadir_ancillary_data = { $
               Filter_Position_1A : temporary(Filter_Position_1A), $
               Filter_Position_1B : temporary(Filter_Position_1B), $
               Filter_Position_2A : temporary(Filter_Position_2A), $
               Filter_Position_2B : temporary(Filter_Position_2B), $

               H2O_HDOAveragingKernel : temporary(H2O_HDOAveragingKernel), $
               HDO_H2OAveragingKernel : temporary(HDO_H2OAveragingKernel), $
               HDO_H2ORatioMeasurementErrorCovariance : temporary(HDO_H2ORatioMeasurementErrorCovariance), $
               HDO_H2ORatioSystematicErrorCovariance : temporary(HDO_H2ORatioSystematicErrorCovariance), $
               HDO_H2ORatioTotalErrorCovariance : temporary(HDO_H2ORatioTotalErrorCovariance), $

               OrbitAscendingFlag : temporary(OrbitAscendingFlag), $
               OzoneTroposphericColumn : temporary(OzoneTroposphericColumn), $
               OzoneTroposphericColumnError : temporary(OzoneTroposphericColumnError), $
               OzoneTroposphericColumnInitial : temporary(OzoneTroposphericColumnInitial), $
               PixelsUsedFlag : temporary(PixelsUsedFlag), $
               SolarAzimuthAngle : temporary(SolarAzimuthAngle), $
               SpacecraftAltitude : temporary(SpacecraftAltitude), $
               SpacecraftLatitude : temporary(SpacecraftLatitude), $
               SpacecraftLongitude : temporary(SpacecraftLongitude), $
               SurfaceEmissConstraint : temporary(SurfaceEmissConstraint), $
               SurfaceEmissErrors : temporary(SurfaceEmissErrors), $
               SurfaceEmissInitial : temporary(SurfaceEmissInitial), $
               SurfaceEmissivity : temporary(SurfaceEmissivity), $ $
               SurfaceTempInitialRefined : temporary(SurfaceTempInitialRefined), $
               TropopausePressure : temporary(TropopausePressure), $
               UTCTime : temporary(UTCTime), $
               pressure : temporary(pressure), $
               VerticalCoordinate : temporary(VerticalCoordinate), $
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               CloudFrequency : temporary(CloudFrequency), $
               Scan_Resolution : temporary(Scan_Resolution)  $
               }
;
; put all attribute data into return structure
;
        nadir_ancillary_attributes = { $
                     Filter_Position_1A : temporary(attrib1), $
                     Filter_Position_1B : temporary(attrib2), $
                     Filter_Position_2A : temporary(attrib3), $
                     Filter_Position_2B : temporary(attrib4), $
                     H2O_HDOAveragingKernel : temporary(attrib5a), $
                     HDO_H2OAveragingKernel : temporary(attrib5b), $
                     HDO_H2ORatioMeasurementErrorCovariance : temporary(attrib5c), $
                     HDO_H2ORatioSystematicErrorCovariance : temporary(attrib5d), $
                     HDO_H2ORatioTotalErrorCovariance : temporary(attrib5e), $
                     OrbitAscendingFlag : temporary(attrib8), $
          	     OzoneTroposphericColumn : temporary(attrib9), $
          	     OzoneTroposphericColumnError : temporary(attrib10), $
          	     OzoneTroposphericColumnInitial : temporary(attrib11), $
                     PixelsUsedFlag : temporary(attrib12), $
                     SolarAzimuthAngle : temporary(attrib13), $
                     SpacecraftAltitude : temporary(attrib14), $
                     SpacecraftLatitude : temporary(attrib15), $
                     SpacecraftLongitude : temporary(attrib16), $
          	     SurfaceEmissConstraint : temporary(attrib17), $
          	     SurfaceEmissErrors : temporary(attrib18), $
          	     SurfaceEmissInitial : temporary(attrib19), $
          	     SurfaceEmissivity : temporary(attrib20), $
                     SurfaceTempInitialRefined : temporary(attrib20a), $
          	     TropopausePressure : temporary(attrib23), $ 
                     UTCTime : temporary(attrib_utc) $
                     }
    endif 


    h5g_close,fieldid

;
; get geolocation data
;
    geoid = h5g_open(specid,'Geolocation Fields')
;
;
    dataid = h5d_open(geoid,'BoresightAzimuth')
    BoresightAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightAzimuth,/type),attrib0
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'BoresightNadirAngle')
    BoresightNadirAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngle,/type),attrib1
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'BoresightNadirAngleUnc')
    BoresightNadirAngleUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngleUnc,/type),attrib2
    h5d_close,dataid
    
    if version ge 3 then begin
        dataid = h5d_open(geoid,'DayNightFlag')
        DayNightFlag = h5d_read(dataid)
        get_data_attributes,dataid,size(DayNightFlag,/type),attrib3
        h5d_close,dataid

        dataid = h5d_open(geoid,'DominantSurfaceType')
        DominantSurfaceType = h5d_read(dataid)
        get_data_attributes,dataid,size(DominantSurfaceType,/type),attrib4
        h5d_close,dataid
    endif

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude,/type),attrib5
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Latitude_Footprint_1')
    Latitude_Footprint_1 = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude_Footprint_1,/type),attrib6
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Latitude_Footprint_2')
    Latitude_Footprint_2 = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude_Footprint_2,/type),attrib7
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Latitude_Footprint_3')
    Latitude_Footprint_3 = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude_Footprint_3,/type),attrib8
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Latitude_Footprint_4')
    Latitude_Footprint_4 = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude_Footprint_4,/type),attrib9
    h5d_close,dataid

    
    dataid = h5d_open(geoid,'LocalSolarTime')
    LocalSolarTime = h5d_read(dataid)
    get_data_attributes,dataid,size(LocalSolarTime,/type),attrib10
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude,/type),attrib11
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Longitude_Footprint_1')
    Longitude_Footprint_1 = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude_Footprint_1,/type),attrib12
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Longitude_Footprint_2')
    Longitude_Footprint_2 = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude_Footprint_2,/type),attrib13
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Longitude_Footprint_3')
    Longitude_Footprint_3 = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude_Footprint_3,/type),attrib14
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Longitude_Footprint_4')
    Longitude_Footprint_4 = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude_Footprint_4,/type),attrib15
    h5d_close,dataid
    
    if version ge 2 then begin
        dataid = h5d_open(geoid,'Scan')
        Scan = h5d_read(dataid)
        get_data_attributes,dataid,size(Scan,/type),attrib16
        h5d_close,dataid
        
        dataid = h5d_open(geoid,'Sequence')
        Sequence = h5d_read(dataid)
        get_data_attributes,dataid,size(Sequence,/type),attrib17
        h5d_close,dataid
    endif

    dataid = h5d_open(geoid,'SolarZenithAngle')
    SolarZenithAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(SolarZenithAngle,/type),attrib18
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'SurfaceElevStandardDeviation')
    SurfaceElevStandardDeviation = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceElevStandardDeviation,/type),attrib19
    h5d_close,dataid
    
    if version ge 3 then begin
        dataid = h5d_open(geoid,'SurfaceTypeFootprint')
        SurfaceTypeFootprint = h5d_read(dataid)
        get_data_attributes,dataid,size(SurfaceTypeFootprint,/type),attrib20
        h5d_close,dataid
    endif

    dataid = h5d_open(geoid,'Tgt_SpacecraftAzimuth')
    Tgt_SpacecraftAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftAzimuth,/type),attrib21
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Tgt_SpacecraftZenith')
    Tgt_SpacecraftZenith = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftZenith,/type),attrib22
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Time')
    Time = h5d_read(dataid)
    get_data_attributes,dataid,size(Time,/type),attrib23
    h5d_close,dataid

    if version eq 3 then begin
        dataid = h5d_open(geoid,'UTCTime')
        ua = h5d_read(dataid)
        get_data_attributes,dataid,size(ua,/type),attrib24,/special
        h5d_close,dataid
        ss = size(ua)
        UTCTime = strarr(ss[2])
        for i=0l,ss[2]-1 do begin
            utctime[i] = ua[0] +ua[1] +ua[2] +ua[3] +ua[4] +ua[5] + $
                         ua[6] +ua[7] +ua[8] +ua[9] +ua[10]+ua[11]+ $
                         ua[12]+ua[13]+ua[14]+ua[15]+ua[16]+ua[17]+ $
                         ua[18]+ua[19]+ua[20]+ua[21]+ua[22]+ua[23]+ $
                         ua[24]+ua[25]+ua[26]
        endfor
    endif

; clean up
    h5g_close,geoid
    h5g_close,specid

;
; put data into return structure
;
    if version eq 1 then begin
        nadir_geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           Latitude : temporary(Latitude), $
           Latitude_Footprint_1 : temporary(Latitude_Footprint_1), $
           Latitude_Footprint_2 : temporary(Latitude_Footprint_2), $
           Latitude_Footprint_3 : temporary(Latitude_Footprint_3), $
           Latitude_Footprint_4 : temporary(Latitude_Footprint_4), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Longitude_Footprint_1 : temporary(Longitude_Footprint_1), $
           Longitude_Footprint_2 : temporary(Longitude_Footprint_2), $
           Longitude_Footprint_3 : temporary(Longitude_Footprint_3), $
           Longitude_Footprint_4 : temporary(Longitude_Footprint_4), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) }

        nadir_geo_attributes = { $
                 BoresightAzimuth : temporary(attrib0), $
                 BoresightNadirAngle : temporary(attrib1), $
                 BoresightNadirAngleUnc : temporary(attrib2), $
                 Latitude : temporary(attrib5), $
                 Latitude_Footprint_1 : temporary(attrib6), $
                 Latitude_Footprint_2 : temporary(attrib7), $
                 Latitude_Footprint_3 : temporary(attrib8), $
                 Latitude_Footprint_4 : temporary(attrib9), $
                 LocalSolarTime : temporary(attrib10), $
                 Longitude : temporary(attrib11), $
                 Longitude_Footprint_1 : temporary(attrib12), $
                 Longitude_Footprint_2 : temporary(attrib13), $
                 Longitude_Footprint_3 : temporary(attrib14), $
                 Longitude_Footprint_4 : temporary(attrib15), $
                 SolarZenithAngle : temporary(attrib18), $
                 SurfaceElevStandardDeviation :  temporary(attrib19), $
                 Tgt_SpacecraftAzimuth : temporary(attrib21), $
                 Tgt_SpacecraftZenith : temporary(attrib22), $
                 Time : temporary(attrib23) }
    endif
    if version eq 2 then begin
        nadir_geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           Latitude : temporary(Latitude), $
           Latitude_Footprint_1 : temporary(Latitude_Footprint_1), $
           Latitude_Footprint_2 : temporary(Latitude_Footprint_2), $
           Latitude_Footprint_3 : temporary(Latitude_Footprint_3), $
           Latitude_Footprint_4 : temporary(Latitude_Footprint_4), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Longitude_Footprint_1 : temporary(Longitude_Footprint_1), $
           Longitude_Footprint_2 : temporary(Longitude_Footprint_2), $
           Longitude_Footprint_3 : temporary(Longitude_Footprint_3), $
           Longitude_Footprint_4 : temporary(Longitude_Footprint_4), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) }

        nadir_geo_attributes = { $
                 BoresightAzimuth : temporary(attrib0), $
                 BoresightNadirAngle : temporary(attrib1), $
                 BoresightNadirAngleUnc : temporary(attrib2), $
                 Latitude : temporary(attrib5), $
                 Latitude_Footprint_1 : temporary(attrib6), $
                 Latitude_Footprint_2 : temporary(attrib7), $
                 Latitude_Footprint_3 : temporary(attrib8), $
                 Latitude_Footprint_4 : temporary(attrib9), $
                 LocalSolarTime : temporary(attrib10), $
                 Longitude : temporary(attrib11), $
                 Longitude_Footprint_1 : temporary(attrib12), $
                 Longitude_Footprint_2 : temporary(attrib13), $
                 Longitude_Footprint_3 : temporary(attrib14), $
                 Longitude_Footprint_4 : temporary(attrib15), $
                 Scan : temporary(attrib16), $
                 sequence : temporary(attrib17), $
                 SolarZenithAngle : temporary(attrib18), $
                 SurfaceElevStandardDeviation :  temporary(attrib19), $
                 Tgt_SpacecraftAzimuth : temporary(attrib21), $
                 Tgt_SpacecraftZenith : temporary(attrib22), $
                 Time : temporary(attrib23) }
    endif
    if version eq 3 then begin
        nadir_geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           DayNightFlag : temporary(DayNightFlag), $
           DominantSurfaceType : temporary(DominantSurfaceType), $
           Latitude : temporary(Latitude), $
           Latitude_Footprint_1 : temporary(Latitude_Footprint_1), $
           Latitude_Footprint_2 : temporary(Latitude_Footprint_2), $
           Latitude_Footprint_3 : temporary(Latitude_Footprint_3), $
           Latitude_Footprint_4 : temporary(Latitude_Footprint_4), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Longitude_Footprint_1 : temporary(Longitude_Footprint_1), $
           Longitude_Footprint_2 : temporary(Longitude_Footprint_2), $
           Longitude_Footprint_3 : temporary(Longitude_Footprint_3), $
           Longitude_Footprint_4 : temporary(Longitude_Footprint_4), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           SurfaceTypeFootprint : temporary(SurfaceTypeFootprint), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time), $
           UTCTime : temporary(UTCTime) $
                         }

        nadir_geo_attributes = { $
                 BoresightAzimuth : temporary(attrib0), $
                 BoresightNadirAngle : temporary(attrib1), $
                 BoresightNadirAngleUnc : temporary(attrib2), $
                 DayNightFlag : temporary(attrib3), $
                 DominantSurfaceType : temporary(attrib4), $
                 Latitude : temporary(attrib5), $
                 Latitude_Footprint_1 : temporary(attrib6), $
                 Latitude_Footprint_2 : temporary(attrib7), $
                 Latitude_Footprint_3 : temporary(attrib8), $
                 Latitude_Footprint_4 : temporary(attrib9), $
                 LocalSolarTime : temporary(attrib10), $
                 Longitude : temporary(attrib11), $
                 Longitude_Footprint_1 : temporary(attrib12), $
                 Longitude_Footprint_2 : temporary(attrib13), $
                 Longitude_Footprint_3 : temporary(attrib14), $
                 Longitude_Footprint_4 : temporary(attrib15), $
                 Scan : temporary(attrib16), $
                 sequence : temporary(attrib17), $
                 SolarZenithAngle : temporary(attrib18), $
                 SurfaceElevStandardDeviation :  temporary(attrib19), $
                 SurfaceTypeFootprint :  temporary(attrib20), $
                 Tgt_SpacecraftAzimuth : temporary(attrib21), $
                 Tgt_SpacecraftZenith : temporary(attrib22), $
                 Time : temporary(attrib23), $
                 UTCTime : temporary(attrib24) $
          }
    endif

    if version ge 4 then begin
        nadir_geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           DayNightFlag : temporary(DayNightFlag), $
           DominantSurfaceType : temporary(DominantSurfaceType), $
           Latitude : temporary(Latitude), $
           Latitude_Footprint_1 : temporary(Latitude_Footprint_1), $
           Latitude_Footprint_2 : temporary(Latitude_Footprint_2), $
           Latitude_Footprint_3 : temporary(Latitude_Footprint_3), $
           Latitude_Footprint_4 : temporary(Latitude_Footprint_4), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Longitude_Footprint_1 : temporary(Longitude_Footprint_1), $
           Longitude_Footprint_2 : temporary(Longitude_Footprint_2), $
           Longitude_Footprint_3 : temporary(Longitude_Footprint_3), $
           Longitude_Footprint_4 : temporary(Longitude_Footprint_4), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           SurfaceTypeFootprint : temporary(SurfaceTypeFootprint), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) $
                         }

        nadir_geo_attributes = { $
                 BoresightAzimuth : temporary(attrib0), $
                 BoresightNadirAngle : temporary(attrib1), $
                 BoresightNadirAngleUnc : temporary(attrib2), $
                 DayNightFlag : temporary(attrib3), $
                 DominantSurfaceType : temporary(attrib4), $
                 Latitude : temporary(attrib5), $
                 Latitude_Footprint_1 : temporary(attrib6), $
                 Latitude_Footprint_2 : temporary(attrib7), $
                 Latitude_Footprint_3 : temporary(attrib8), $
                 Latitude_Footprint_4 : temporary(attrib9), $
                 LocalSolarTime : temporary(attrib10), $
                 Longitude : temporary(attrib11), $
                 Longitude_Footprint_1 : temporary(attrib12), $
                 Longitude_Footprint_2 : temporary(attrib13), $
                 Longitude_Footprint_3 : temporary(attrib14), $
                 Longitude_Footprint_4 : temporary(attrib15), $
                 Scan : temporary(attrib16), $
                 sequence : temporary(attrib17), $
                 SolarZenithAngle : temporary(attrib18), $
                 SurfaceElevStandardDeviation :  temporary(attrib19), $
                 SurfaceTypeFootprint :  temporary(attrib20), $
                 Tgt_SpacecraftAzimuth : temporary(attrib21), $
                 Tgt_SpacecraftZenith : temporary(attrib22), $
                 Time : temporary(attrib23) $
          }
    endif



endif ; AncillaryNadirSwath


dn = 0
gn = 0

nad = where(names eq 'AncillaryLimbSwath')
if nad ne -1 then begin ; check to see if populated
    dn = h5g_get_nmembers(fid,'/HDFEOS/SWATHS/AncillaryLimbSwath/Data Fields')
    gn = h5g_get_nmembers(fid,'/HDFEOS/SWATHS/AncillaryLimbSwath/Geolocation Fields')
endif

if dn ne 0 and gn ne 0 then begin
    specid = h5g_open(swid,'AncillaryLimbSwath')
    fieldid = h5g_open(specid,'Data Fields')
;
; actual data, alphabetically
;
    if version eq 1 then begin
        dataid = h5d_open(fieldid,'EmissivityWavenumber')
        EmissivityWavenumber = h5d_read(dataid)
        get_data_attributes,dataid,size(EmissivityWavenumber,/type),attrib0
        h5d_close,dataid
    endif

    dataid = h5d_open(fieldid,'Filter_Position_1A')
    Filter_Position_1A = h5d_read(dataid)
    get_data_attributes,dataid,size(Filter_Position_1A,/type),attrib1
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'Filter_Position_1B')
    Filter_Position_1B = h5d_read(dataid)
    get_data_attributes,dataid,size(Filter_Position_1B,/type),attrib2
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Filter_Position_2A')
    Filter_Position_2A = h5d_read(dataid)
    get_data_attributes,dataid,size(Filter_Position_2A,/type),attrib3
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'Filter_Position_2B')
    Filter_Position_2B = h5d_read(dataid)
    get_data_attributes,dataid,size(Filter_Position_2B,/type),attrib4
    h5d_close,dataid
    
    if version ge 5 then begin
        dataid = h5d_open(fieldid,'H2O_HDOAveragingKernel')
        H2O_HDOAveragingKernel = h5d_read(dataid)
        get_data_attributes,dataid,size(H2O_HDOAveragingKernel,/type),attrib5a
        h5d_close,dataid

        dataid = h5d_open(fieldid,'HDO_H2OAveragingKernel')
        HDO_H2OAveragingKernel = h5d_read(dataid)
        get_data_attributes,dataid,size(HDO_H2OAveragingKernel,/type),attrib5b
        h5d_close,dataid

        dataid = h5d_open(fieldid,'HDO_H2ORatioMeasurementErrorCovariance')
        HDO_H2ORatioMeasurementErrorCovariance = h5d_read(dataid)
        get_data_attributes,dataid,size(HDO_H2ORatioMeasurementErrorCovariance,/type),attrib5c
        h5d_close,dataid

        dataid = h5d_open(fieldid,'HDO_H2ORatioSystematicErrorCovariance')
        HDO_H2ORatioSystematicErrorCovariance = h5d_read(dataid)
        get_data_attributes,dataid,size(HDO_H2ORatioSystematicErrorCovariance,/type),attrib5d
        h5d_close,dataid

        dataid = h5d_open(fieldid,'HDO_H2ORatioTotalErrorCovariance')
        HDO_H2ORatioTotalErrorCovariance = h5d_read(dataid)
        get_data_attributes,dataid,size(HDO_H2ORatioTotalErrorCovariance,/type),attrib5e
        h5d_close,dataid
    endif
 
    if version le 4 then begin
        dataid = h5d_open(fieldid,'H2O_TAveragingKernel')
        H2O_TAveragingKernel = h5d_read(dataid)
        get_data_attributes,dataid,size(H2O_TAveragingKernel,/type),attrib5
        h5d_close,dataid
    endif
    
;    
    dataid = h5d_open(fieldid,'OrbitAscendingFlag')
    OrbitAscendingFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(OrbitAscendingFlag,/type),attrib6
    h5d_close,dataid
; 
   
    dataid = h5d_open(fieldid,'PixelsUsedFlag')
    PixelsUsedFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(PixelsUsedFlag,/type),attrib7,/special
    h5d_close,dataid

    if version ge 3 then begin
        dataid = h5d_open(fieldid,'RetrievedPointingAngle')
        RetrievedPointingAngle = h5d_read(dataid)
        get_data_attributes,dataid,size(RetrievedPointingAngle,/type),attrib8
        h5d_close,dataid

        dataid = h5d_open(fieldid,'RetrievedPointingAngleError')
        RetrievedPointingAngleError = h5d_read(dataid)
        get_data_attributes,dataid,size(RetrievedPointingAngleError,/type), $
                            attrib9
        h5d_close,dataid
    endif


    dataid = h5d_open(fieldid,'SolarAzimuthAngle')
    SolarAzimuthAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(SolarAzimuthAngle,/type),attrib10
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'SpacecraftAltitude')
    SpacecraftAltitude = h5d_read(dataid)
    get_data_attributes,dataid,size(SpacecraftAltitude,/type),attrib11
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'SpacecraftLatitude')
    SpacecraftLatitude = h5d_read(dataid)
    get_data_attributes,dataid,size(SpacecraftLatitude,/type),attrib12
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'SpacecraftLongitude')
    SpacecraftLongitude = h5d_read(dataid)
    get_data_attributes,dataid,size(SpacecraftLongitude,/type),attrib13
    h5d_close,dataid

    if version le 4 then begin
        dataid = h5d_open(fieldid,'T_H2OAveragingKernel')
        T_H2OAveragingKernel = h5d_read(dataid)
        get_data_attributes,dataid,size(T_H2OAveragingKernel,/type),attrib14
        h5d_close,dataid
    
        dataid = h5d_open(fieldid,'T_H2OCovariance')
        T_H2OCovariance = h5d_read(dataid)
        get_data_attributes,dataid,size(T_H2OCovariance,/type),attrib15
        h5d_close,dataid
    endif

    if version ge 3 then begin
        dataid = h5d_open(fieldid,'TropopausePressure')
        TropopausePressure = h5d_read(dataid)
        get_data_attributes,dataid,size(TropopausePressure,/type),attrib16
        h5d_close,dataid
    endif
    if version ge 4 then begin
        dataid = h5d_open(fieldid,'UTCTime')
        UTCTime = h5d_read(dataid)
        get_data_attributes,dataid,size(utctime,/type),attrib_utc,/utctime
        h5d_close,dataid
    endif
;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressure = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    if version ge 2 then begin
        id = h5a_open_name(specid,'CloudFrequency')
        CloudFrequency = h5a_read(id)
        h5a_close,id

        id = h5a_open_name(specid,'EmissivityWavenumber')
        EmissivityWavenumber = h5a_read(id)
        h5a_close,id
    endif else begin
        cloudfrequency = [0]
    endelse
;
; put all species data into return structure
;
    if version eq 1 then begin
        limb_ancillary_data = { $
               Filter_Position_1A : temporary(Filter_Position_1A), $
               Filter_Position_1B : temporary(Filter_Position_1B), $
               Filter_Position_2A : temporary(Filter_Position_2A), $
               Filter_Position_2B : temporary(Filter_Position_2B), $
               H2O_TAveragingKernel : temporary(H2O_TAveragingKernel), $
               OrbitAscendingFlag : temporary(OrbitAscendingFlag), $
               PixelsUsedFlag : temporary(PixelsUsedFlag), $
               SolarAzimuthAngle : temporary(SolarAzimuthAngle), $
               SpacecraftAltitude : temporary(SpacecraftAltitude), $
               SpacecraftLatitude : temporary(SpacecraftLatitude), $
               SpacecraftLongitude : temporary(SpacecraftLongitude), $
               T_H2OAveragingKernel : temporary(T_H2OAveragingKernel), $
               T_H2OCovariance : temporary(T_H2OCovariance), $
               pressure : temporary(pressure), $
               VerticalCoordinate : temporary(VerticalCoordinate), $
               EmissivityWavenumber : temporary(EmissivityWavenumber) $
               }
;
; put all attribute data into return structure
;
        limb_ancillary_attributes = { $
                     EmissivityWavenumber : temporary(attrib0), $
                     Filter_Position_1A : temporary(attrib1), $
                     Filter_Position_1B : temporary(attrib2), $
                     Filter_Position_2A : temporary(attrib3), $
                     Filter_Position_2B : temporary(attrib4), $
                     H2O_TAveragingKernel : temporary(attrib5), $
                     OrbitAscendingFlag : temporary(attrib6), $
                     PixelsUsedFlag : temporary(attrib7), $
                     SolarAzimuthAngle : temporary(attrib10), $
                     SpacecraftAltitude : temporary(attrib11), $
                     SpacecraftLatitude : temporary(attrib12), $
                     SpacecraftLongitude : temporary(attrib13), $
                     T_H2OAveragingKernel : temporary(attrib14), $
                     T_H2OCovariance : temporary(attrib15) $
                     }
    endif 
    if version eq 2 then begin
        limb_ancillary_data = { $
               Filter_Position_1A : temporary(Filter_Position_1A), $
               Filter_Position_1B : temporary(Filter_Position_1B), $
               Filter_Position_2A : temporary(Filter_Position_2A), $
               Filter_Position_2B : temporary(Filter_Position_2B), $
               H2O_TAveragingKernel : temporary(H2O_TAveragingKernel), $
               OrbitAscendingFlag : temporary(OrbitAscendingFlag), $
               PixelsUsedFlag : temporary(PixelsUsedFlag), $
               SolarAzimuthAngle : temporary(SolarAzimuthAngle), $
               SpacecraftAltitude : temporary(SpacecraftAltitude), $
               SpacecraftLatitude : temporary(SpacecraftLatitude), $
               SpacecraftLongitude : temporary(SpacecraftLongitude), $
               T_H2OAveragingKernel : temporary(T_H2OAveragingKernel), $
               T_H2OCovariance : temporary(T_H2OCovariance), $
               pressure : temporary(pressure), $
               VerticalCoordinate : temporary(VerticalCoordinate), $
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               CloudFrequency : temporary(CloudFrequency) $
               }
;
; put all attribute data into return structure
;
        limb_ancillary_attributes = { $
                     Filter_Position_1A : temporary(attrib1), $
                     Filter_Position_1B : temporary(attrib2), $
                     Filter_Position_2A : temporary(attrib3), $
                     Filter_Position_2B : temporary(attrib4), $
                     H2O_TAveragingKernel : temporary(attrib5), $
                     OrbitAscendingFlag : temporary(attrib6), $
                     PixelsUsedFlag : temporary(attrib7), $
                     SolarAzimuthAngle : temporary(attrib110), $
                     SpacecraftAltitude : temporary(attrib11), $
                     SpacecraftLatitude : temporary(attrib12), $
                     SpacecraftLongitude : temporary(attrib13), $
                     T_H2OAveragingKernel : temporary(attrib14), $
                     T_H2OCovariance : temporary(attrib15) $
                     }
    endif 

    if version eq 3 then begin
        limb_ancillary_data = { $
               Filter_Position_1A : temporary(Filter_Position_1A), $
               Filter_Position_1B : temporary(Filter_Position_1B), $
               Filter_Position_2A : temporary(Filter_Position_2A), $
               Filter_Position_2B : temporary(Filter_Position_2B), $
               H2O_TAveragingKernel : temporary(H2O_TAveragingKernel), $
               OrbitAscendingFlag : temporary(OrbitAscendingFlag), $
               PixelsUsedFlag : temporary(PixelsUsedFlag), $
               RetrievedPointingAngle : temporary(RetrievedPointingAngle), $
	       RetrievedPointingAngleError :temporary(RetrievedPointingAngleError), $
               SolarAzimuthAngle : temporary(SolarAzimuthAngle), $
               SpacecraftAltitude : temporary(SpacecraftAltitude), $
               SpacecraftLatitude : temporary(SpacecraftLatitude), $
               SpacecraftLongitude : temporary(SpacecraftLongitude), $
               T_H2OAveragingKernel : temporary(T_H2OAveragingKernel), $
               T_H2OCovariance : temporary(T_H2OCovariance), $
               TropopausePressure : temporary(TropopausePressure), $
               pressure : temporary(pressure), $
               VerticalCoordinate : temporary(VerticalCoordinate), $
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               CloudFrequency : temporary(CloudFrequency) $
               }
;
; put all attribute data into return structure
;
        limb_ancillary_attributes = { $
                     Filter_Position_1A : temporary(attrib1), $
                     Filter_Position_1B : temporary(attrib2), $
                     Filter_Position_2A : temporary(attrib3), $
                     Filter_Position_2B : temporary(attrib4), $
                     H2O_TAveragingKernel : temporary(attrib5), $
                     OrbitAscendingFlag : temporary(attrib6), $
                     PixelsUsedFlag : temporary(attrib7), $
                     RetrievedPointingAngle : temporary(attrib8), $
	             RetrievedPointingAngleError :temporary(attrib9), $
                     SolarAzimuthAngle : temporary(attrib10), $
                     SpacecraftAltitude : temporary(attrib11), $
                     SpacecraftLatitude : temporary(attrib12), $
                     SpacecraftLongitude : temporary(attrib13), $
                     T_H2OAveragingKernel : temporary(attrib14), $
                     T_H2OCovariance : temporary(attrib15), $
          	     TropopausePressure : temporary(attrib16) $
                     }
    endif 

    if version eq 4 then begin
        limb_ancillary_data = { $
               Filter_Position_1A : temporary(Filter_Position_1A), $
               Filter_Position_1B : temporary(Filter_Position_1B), $
               Filter_Position_2A : temporary(Filter_Position_2A), $
               Filter_Position_2B : temporary(Filter_Position_2B), $
               H2O_TAveragingKernel : temporary(H2O_TAveragingKernel), $
               OrbitAscendingFlag : temporary(OrbitAscendingFlag), $
               PixelsUsedFlag : temporary(PixelsUsedFlag), $
               RetrievedPointingAngle : temporary(RetrievedPointingAngle), $
	       RetrievedPointingAngleError :temporary(RetrievedPointingAngleError), $
               SolarAzimuthAngle : temporary(SolarAzimuthAngle), $
               SpacecraftAltitude : temporary(SpacecraftAltitude), $
               SpacecraftLatitude : temporary(SpacecraftLatitude), $
               SpacecraftLongitude : temporary(SpacecraftLongitude), $
               T_H2OAveragingKernel : temporary(T_H2OAveragingKernel), $
               T_H2OCovariance : temporary(T_H2OCovariance), $
               TropopausePressure : temporary(TropopausePressure), $
               UTCTime : temporary(UTCTime), $
               pressure : temporary(pressure), $
               VerticalCoordinate : temporary(VerticalCoordinate), $
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               CloudFrequency : temporary(CloudFrequency) $
               }
;
; put all attribute data into return structure
;
        limb_ancillary_attributes = { $
                     Filter_Position_1A : temporary(attrib1), $
                     Filter_Position_1B : temporary(attrib2), $
                     Filter_Position_2A : temporary(attrib3), $
                     Filter_Position_2B : temporary(attrib4), $
                     H2O_TAveragingKernel : temporary(attrib5), $
                     OrbitAscendingFlag : temporary(attrib6), $
                     PixelsUsedFlag : temporary(attrib7), $
                     RetrievedPointingAngle : temporary(attrib8), $
	             RetrievedPointingAngleError :temporary(attrib9), $
                     SolarAzimuthAngle : temporary(attrib10), $
                     SpacecraftAltitude : temporary(attrib11), $
                     SpacecraftLatitude : temporary(attrib12), $
                     SpacecraftLongitude : temporary(attrib13), $
                     T_H2OAveragingKernel : temporary(attrib14), $
                     T_H2OCovariance : temporary(attrib15), $
          	     TropopausePressure : temporary(attrib16), $ 
                     UTCTime : temporary(attrib_utc) $
                     }
    endif 

    if version eq 5 then begin
        limb_ancillary_data = { $
               Filter_Position_1A : temporary(Filter_Position_1A), $
               Filter_Position_1B : temporary(Filter_Position_1B), $
               Filter_Position_2A : temporary(Filter_Position_2A), $
               Filter_Position_2B : temporary(Filter_Position_2B), $
               H2O_HDOAveragingKernel : temporary(H2O_HDOAveragingKernel), $
               HDO_H2OAveragingKernel : temporary(HDO_H2OAveragingKernel), $
               HDO_H2ORatioMeasurementErrorCovariance : temporary(HDO_H2ORatioMeasurementErrorCovariance), $
               HDO_H2ORatioSystematicErrorCovariance : temporary(HDO_H2ORatioSystematicErrorCovariance), $
               HDO_H2ORatioTotalErrorCovariance : temporary(HDO_H2ORatioTotalErrorCovariance), $
               OrbitAscendingFlag : temporary(OrbitAscendingFlag), $
               PixelsUsedFlag : temporary(PixelsUsedFlag), $
               RetrievedPointingAngle : temporary(RetrievedPointingAngle), $
	       RetrievedPointingAngleError :temporary(RetrievedPointingAngleError), $
               SolarAzimuthAngle : temporary(SolarAzimuthAngle), $
               SpacecraftAltitude : temporary(SpacecraftAltitude), $
               SpacecraftLatitude : temporary(SpacecraftLatitude), $
               SpacecraftLongitude : temporary(SpacecraftLongitude), $
               TropopausePressure : temporary(TropopausePressure), $
               UTCTime : temporary(UTCTime), $
               pressure : temporary(pressure), $
               VerticalCoordinate : temporary(VerticalCoordinate), $
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               CloudFrequency : temporary(CloudFrequency) $
               }
;
; put all attribute data into return structure
;
        limb_ancillary_attributes = { $
                     Filter_Position_1A : temporary(attrib1), $
                     Filter_Position_1B : temporary(attrib2), $
                     Filter_Position_2A : temporary(attrib3), $
                     Filter_Position_2B : temporary(attrib4), $
                     H2O_HDOAveragingKernel : temporary(attrib5a), $
                     HDO_H2OAveragingKernel : temporary(attrib5b), $
                     HDO_H2ORatioMeasurementErrorCovariance : temporary(attrib5c), $
                     HDO_H2ORatioSystematicErrorCovariance : temporary(attrib5d), $
                     HDO_H2ORatioTotalErrorCovariance : temporary(attrib5e), $
                     OrbitAscendingFlag : temporary(attrib6), $
                     PixelsUsedFlag : temporary(attrib7), $
                     RetrievedPointingAngle : temporary(attrib8), $
	             RetrievedPointingAngleError :temporary(attrib9), $
                     SolarAzimuthAngle : temporary(attrib10), $
                     SpacecraftAltitude : temporary(attrib11), $
                     SpacecraftLatitude : temporary(attrib12), $
                     SpacecraftLongitude : temporary(attrib13), $
          	     TropopausePressure : temporary(attrib16), $ 
                     UTCTime : temporary(attrib_utc) $
                     }
    endif 



    h5g_close,fieldid

;
; get geolocation data
;
    geoid = h5g_open(specid,'Geolocation Fields')
;
;
    dataid = h5d_open(geoid,'BoresightAzimuth')
    BoresightAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightAzimuth,/type),attrib0
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'BoresightNadirAngle')
    BoresightNadirAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngle,/type),attrib1
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'BoresightNadirAngleUnc')
    BoresightNadirAngleUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngleUnc,/type),attrib2
    h5d_close,dataid

    if version ge 3 then begin

        dataid = h5d_open(geoid,'BoresightTangentHeight')
        BoresightTangentHeight = h5d_read(dataid)
        get_data_attributes,dataid,size(BoresightTangentHeight,/type),attrib3
        h5d_close,dataid

        dataid = h5d_open(geoid,'BoresightTangentHeightUnc')
        BoresightTangentHeightUnc = h5d_read(dataid)
        get_data_attributes,dataid,size(BoresightTangentHeightUnc,/type),attrib4
        h5d_close,dataid

        dataid = h5d_open(geoid,'DayNightFlag')
        DayNightFlag = h5d_read(dataid)
        get_data_attributes,dataid,size(DayNightFlag,/type),attrib5
        h5d_close,dataid
    endif

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude,/type),attrib6
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Latitude_Footprint_1')
    Latitude_Footprint_1 = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude_Footprint_1,/type),attrib7
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Latitude_Footprint_2')
    Latitude_Footprint_2 = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude_Footprint_2,/type),attrib8
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Latitude_Footprint_3')
    Latitude_Footprint_3 = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude_Footprint_3,/type),attrib9
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Latitude_Footprint_4')
    Latitude_Footprint_4 = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude_Footprint_4,/type),attrib10
    h5d_close,dataid

    
    dataid = h5d_open(geoid,'LocalSolarTime')
    LocalSolarTime = h5d_read(dataid)
    get_data_attributes,dataid,size(LocalSolarTime,/type),attrib11
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude,/type),attrib12
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Longitude_Footprint_1')
    Longitude_Footprint_1 = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude_Footprint_1,/type),attrib13
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Longitude_Footprint_2')
    Longitude_Footprint_2 = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude_Footprint_2,/type),attrib14
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Longitude_Footprint_3')
    Longitude_Footprint_3 = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude_Footprint_3,/type),attrib15
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Longitude_Footprint_4')
    Longitude_Footprint_4 = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude_Footprint_4,/type),attrib16
    h5d_close,dataid
    
    if version ge 2 then begin
        dataid = h5d_open(geoid,'Scan')
        Scan = h5d_read(dataid)
        get_data_attributes,dataid,size(Scan,/type),attrib17
        h5d_close,dataid
        
        dataid = h5d_open(geoid,'Sequence')
        Sequence = h5d_read(dataid)
        get_data_attributes,dataid,size(Sequence,/type),attrib18
        h5d_close,dataid
    endif

    dataid = h5d_open(geoid,'SolarZenithAngle')
    SolarZenithAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(SolarZenithAngle,/type),attrib19
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'SurfaceElevStandardDeviation')
    SurfaceElevStandardDeviation = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceElevStandardDeviation,/type),attrib20
    h5d_close,dataid
    
    if version ge 3 then begin
        dataid = h5d_open(geoid,'SurfaceTypeFootprint')
        SurfaceTypeFootprint = h5d_read(dataid)
        get_data_attributes,dataid,size(SurfaceTypeFootprint,/type),attrib21
        h5d_close,dataid
    endif

    dataid = h5d_open(geoid,'Tgt_SpacecraftAzimuth')
    Tgt_SpacecraftAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftAzimuth,/type),attrib22
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Tgt_SpacecraftZenith')
    Tgt_SpacecraftZenith = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftZenith,/type),attrib23
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Time')
    Time = h5d_read(dataid)
    get_data_attributes,dataid,size(Time,/type),attrib24
    h5d_close,dataid

    if version eq 3 then begin
        dataid = h5d_open(geoid,'UTCTime')
        ua = h5d_read(dataid)
        get_data_attributes,dataid,size(ua,/type),attrib25,/special
        h5d_close,dataid
        ss = size(ua)
        UTCTime = strarr(ss[2])
        for i=0l,ss[2]-1 do begin
            utctime[i] = ua[0] +ua[1] +ua[2] +ua[3] +ua[4] +ua[5] + $
                         ua[6] +ua[7] +ua[8] +ua[9] +ua[10]+ua[11]+ $
                         ua[12]+ua[13]+ua[14]+ua[15]+ua[16]+ua[17]+ $
                         ua[18]+ua[19]+ua[20]+ua[21]+ua[22]+ua[23]+ $
                         ua[24]+ua[25]+ua[26]
        endfor
    endif

; clean up
    h5g_close,geoid
    h5g_close,specid

;
; put data into return structure
;
    if version eq 1 then begin
        limb_geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           Latitude : temporary(Latitude), $
           Latitude_Footprint_1 : temporary(Latitude_Footprint_1), $
           Latitude_Footprint_2 : temporary(Latitude_Footprint_2), $
           Latitude_Footprint_3 : temporary(Latitude_Footprint_3), $
           Latitude_Footprint_4 : temporary(Latitude_Footprint_4), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Longitude_Footprint_1 : temporary(Longitude_Footprint_1), $
           Longitude_Footprint_2 : temporary(Longitude_Footprint_2), $
           Longitude_Footprint_3 : temporary(Longitude_Footprint_3), $
           Longitude_Footprint_4 : temporary(Longitude_Footprint_4), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) }

        limb_geo_attributes = { $
                 BoresightAzimuth : temporary(attrib0), $
                 BoresightNadirAngle : temporary(attrib1), $
                 BoresightNadirAngleUnc : temporary(attrib2), $
                 Latitude : temporary(attrib6), $
                 Latitude_Footprint_1 : temporary(attrib7), $
                 Latitude_Footprint_2 : temporary(attrib8), $
                 Latitude_Footprint_3 : temporary(attrib9), $
                 Latitude_Footprint_4 : temporary(attrib10), $
                 LocalSolarTime : temporary(attrib11), $
                 Longitude : temporary(attrib12), $
                 Longitude_Footprint_1 : temporary(attrib13), $
                 Longitude_Footprint_2 : temporary(attrib14), $
                 Longitude_Footprint_3 : temporary(attrib15), $
                 Longitude_Footprint_4 : temporary(attrib16), $
                 SolarZenithAngle : temporary(attrib19), $
                 SurfaceElevStandardDeviation :  temporary(attrib20), $
                 Tgt_SpacecraftAzimuth : temporary(attrib22), $
                 Tgt_SpacecraftZenith : temporary(attrib23), $
                 Time : temporary(attrib24) }
    endif
    if version eq 2 then begin
        limb_geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           Latitude : temporary(Latitude), $
           Latitude_Footprint_1 : temporary(Latitude_Footprint_1), $
           Latitude_Footprint_2 : temporary(Latitude_Footprint_2), $
           Latitude_Footprint_3 : temporary(Latitude_Footprint_3), $
           Latitude_Footprint_4 : temporary(Latitude_Footprint_4), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Longitude_Footprint_1 : temporary(Longitude_Footprint_1), $
           Longitude_Footprint_2 : temporary(Longitude_Footprint_2), $
           Longitude_Footprint_3 : temporary(Longitude_Footprint_3), $
           Longitude_Footprint_4 : temporary(Longitude_Footprint_4), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) }

        limb_geo_attributes = { $
                 BoresightAzimuth : temporary(attrib0), $
                 BoresightNadirAngle : temporary(attrib1), $
                 BoresightNadirAngleUnc : temporary(attrib2), $
                 Latitude : temporary(attrib6), $
                 Latitude_Footprint_1 : temporary(attrib7), $
                 Latitude_Footprint_2 : temporary(attrib8), $
                 Latitude_Footprint_3 : temporary(attrib9), $
                 Latitude_Footprint_4 : temporary(attrib10), $
                 LocalSolarTime : temporary(attrib11), $
                 Longitude : temporary(attrib12), $
                 Longitude_Footprint_1 : temporary(attrib13), $
                 Longitude_Footprint_2 : temporary(attrib14), $
                 Longitude_Footprint_3 : temporary(attrib15), $
                 Longitude_Footprint_4 : temporary(attrib16), $
                 Scan : temporary(attrib17), $
                 sequence : temporary(attrib18), $
                 SolarZenithAngle : temporary(attrib19), $
                 SurfaceElevStandardDeviation :  temporary(attrib20), $
                 Tgt_SpacecraftAzimuth : temporary(attrib22), $
                 Tgt_SpacecraftZenith : temporary(attrib23), $
                 Time : temporary(attrib24) }
    endif
    if version eq 3 then begin
        limb_geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
	   BoresightTangentHeight : temporary(BoresightTangentHeight), $
	   BoresightTangentHeightUnc : temporary(BoresightTangentHeightUnc), $
           DayNightFlag : temporary(DayNightFlag), $
           Latitude : temporary(Latitude), $
           Latitude_Footprint_1 : temporary(Latitude_Footprint_1), $
           Latitude_Footprint_2 : temporary(Latitude_Footprint_2), $
           Latitude_Footprint_3 : temporary(Latitude_Footprint_3), $
           Latitude_Footprint_4 : temporary(Latitude_Footprint_4), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Longitude_Footprint_1 : temporary(Longitude_Footprint_1), $
           Longitude_Footprint_2 : temporary(Longitude_Footprint_2), $
           Longitude_Footprint_3 : temporary(Longitude_Footprint_3), $
           Longitude_Footprint_4 : temporary(Longitude_Footprint_4), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           SurfaceTypeFootprint : temporary(SurfaceTypeFootprint), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time), $
           UTCTime : temporary(UTCTime) $
                         }

        limb_geo_attributes = { $
                 BoresightAzimuth : temporary(attrib0), $
                 BoresightNadirAngle : temporary(attrib1), $
                 BoresightNadirAngleUnc : temporary(attrib2), $
		 BoresightTangentHeight : temporary(attrib3), $
		 BoresightTangentHeightUnc : temporary(attrib4), $
                 DayNightFlag : temporary(attrib5), $
                 Latitude : temporary(attrib6), $
                 Latitude_Footprint_1 : temporary(attrib7), $
                 Latitude_Footprint_2 : temporary(attrib8), $
                 Latitude_Footprint_3 : temporary(attrib9), $
                 Latitude_Footprint_4 : temporary(attrib10), $
                 LocalSolarTime : temporary(attrib11), $
                 Longitude : temporary(attrib12), $
                 Longitude_Footprint_1 : temporary(attrib13), $
                 Longitude_Footprint_2 : temporary(attrib14), $
                 Longitude_Footprint_3 : temporary(attrib15), $
                 Longitude_Footprint_4 : temporary(attrib16), $
                 Scan : temporary(attrib17), $
                 sequence : temporary(attrib18), $
                 SolarZenithAngle : temporary(attrib19), $
                 SurfaceElevStandardDeviation :  temporary(attrib20), $
                 SurfaceTypeFootprint :  temporary(attrib21), $
                 Tgt_SpacecraftAzimuth : temporary(attrib22), $
                 Tgt_SpacecraftZenith : temporary(attrib23), $
                 Time : temporary(attrib24), $
                 UTCTime : temporary(attrib25) $
          }
    endif

    if version eq 4 then begin
        limb_geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
	   BoresightTangentHeight : temporary(BoresightTangentHeight), $
	   BoresightTangentHeightUnc : temporary(BoresightTangentHeightUnc), $
           DayNightFlag : temporary(DayNightFlag), $
           Latitude : temporary(Latitude), $
           Latitude_Footprint_1 : temporary(Latitude_Footprint_1), $
           Latitude_Footprint_2 : temporary(Latitude_Footprint_2), $
           Latitude_Footprint_3 : temporary(Latitude_Footprint_3), $
           Latitude_Footprint_4 : temporary(Latitude_Footprint_4), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Longitude_Footprint_1 : temporary(Longitude_Footprint_1), $
           Longitude_Footprint_2 : temporary(Longitude_Footprint_2), $
           Longitude_Footprint_3 : temporary(Longitude_Footprint_3), $
           Longitude_Footprint_4 : temporary(Longitude_Footprint_4), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           SurfaceTypeFootprint : temporary(SurfaceTypeFootprint), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) $
                         }

        limb_geo_attributes = { $
                 BoresightAzimuth : temporary(attrib0), $
                 BoresightNadirAngle : temporary(attrib1), $
                 BoresightNadirAngleUnc : temporary(attrib2), $
		 BoresightTangentHeight : temporary(attrib3), $
		 BoresightTangentHeightUnc : temporary(attrib4), $
                 DayNightFlag : temporary(attrib5), $
                 Latitude : temporary(attrib6), $
                 Latitude_Footprint_1 : temporary(attrib7), $
                 Latitude_Footprint_2 : temporary(attrib8), $
                 Latitude_Footprint_3 : temporary(attrib9), $
                 Latitude_Footprint_4 : temporary(attrib10), $
                 LocalSolarTime : temporary(attrib11), $
                 Longitude : temporary(attrib12), $
                 Longitude_Footprint_1 : temporary(attrib13), $
                 Longitude_Footprint_2 : temporary(attrib14), $
                 Longitude_Footprint_3 : temporary(attrib15), $
                 Longitude_Footprint_4 : temporary(attrib16), $
                 Scan : temporary(attrib17), $
                 sequence : temporary(attrib18), $
                 SolarZenithAngle : temporary(attrib19), $
                 SurfaceElevStandardDeviation :  temporary(attrib20), $
                 SurfaceTypeFootprint :  temporary(attrib21), $
                 Tgt_SpacecraftAzimuth : temporary(attrib22), $
                 Tgt_SpacecraftZenith : temporary(attrib23), $
                 Time : temporary(attrib24) $
          }
    endif


endif ; AncillarLimbSwath
;





;
; get 'Additional' information
;
aid = h5g_open(hdfid,'ADDITIONAL')
faaid = h5g_open(aid,'FILE_ATTRIBUTES')

if read_ph ne 0 then begin
    dataid = h5d_open(faaid,'ProductionHistory')
    ProductionHistory = h5d_read(dataid)
    h5d_close,dataid
endif else begin
    ProductionHistory = ''
endelse

id = h5a_open_name(faaid,'InstrumentName')
InstrumentName = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'ProcessLevel')
ProcessLevel = h5a_read(id)
h5a_close,id

if version eq 1 then begin
    id = h5a_open_name(faaid,'TAI93AtOzOfGranule')
    TAI93At0zOfGranule = h5a_read(id)
    h5a_close,id
endif
if version ge 2 then begin
    id = h5a_open_name(faaid,'TAI93At0zOfGranule')
    TAI93At0zOfGranule = h5a_read(id)
    h5a_close,id
endif   

id = h5a_open_name(faaid,'GlobalSurveyNumber')
GlobalSurveyNumber = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'GranuleMonth')
GranuleMonth = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'GranuleDay')
GranuleDay = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'GranuleYear')
GranuleYear = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'SurveyMode')
SurveyMode = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'PGEVersion')
PGEVersion = h5a_read(id)
h5a_close,id

if version ge 2 then begin
    id = h5a_open_name(faaid,'Command_Seq_ID')
    Command_Seq_ID = h5a_read(id)
    h5a_close,id
endif   


h5g_close,faaid
h5g_close,aid
h5g_close,hdfid

if version eq 1 then begin
    additional_data = { $
                  ProductionHistory : temporary(ProductionHistory), $
                  InstrumentName : temporary(InstrumentName), $
                  ProcessLevel : temporary(ProcessLevel), $
                  TAI93At0zOfGranule : temporary(TAI93At0zOfGranule), $
                  GlobalSurveyNumber : temporary(GlobalSurveyNumber), $
                  GranuleMonth : temporary(GranuleMonth), $
                  GranuleDay : temporary(GranuleDay), $
                  GranuleYear : temporary(GranuleYear), $
                  SurveyMode : temporary(SurveyMode), $
                  PGEVersion : temporary(PGEVersion) }
endif
if version ge 2 then begin
    file_attributes = { $
                  ProductionHistory : temporary(ProductionHistory), $
                  InstrumentName : temporary(InstrumentName), $
                  ProcessLevel : temporary(ProcessLevel), $
                  TAI93At0zOfGranule : temporary(TAI93At0zOfGranule), $
                  GlobalSurveyNumber : temporary(GlobalSurveyNumber), $
                  GranuleMonth : temporary(GranuleMonth), $
                  GranuleDay : temporary(GranuleDay), $
                  GranuleYear : temporary(GranuleYear), $
                  SurveyMode : temporary(SurveyMode), $
                  PGEVersion : temporary(PGEVersion), $
                  Command_Seq_ID : temporary(Command_Seq_ID)    }
endif


hdfinfid = h5g_open(fid,'HDFEOS INFORMATION')

dataid = h5d_open(hdfinfid,'StructMetadata.0')
StructMetadata = h5d_read(dataid)
h5d_close,dataid

dataid = h5d_open(hdfinfid,'coremetadata')
coremetadata = h5d_read(dataid)
h5d_close,dataid

id = h5a_open_name(hdfinfid,'HDFEOSVersion')
HDFEOSVersion = h5a_read(id)
h5a_close,id

h5g_close,hdfinfid
h5f_close,fid

hdfeos_data = { $
              StructMetadata : temporary(StructMetadata), $
              coremetadata : temporary(coremetadata), $
              HDFEOSVersion : temporary(HDFEOSVersion) }


return

end
