;+
;
;                      Tropospheric Emission
;                          Spectrometer
;                             (TES)
;
;                         Copyright 2008-2011
;              California Institute of Technology.
;                       ALL RIGHTS RESERVED.
;           U.S. Government Sponsorship acknowledged.
;
;           Any commercial use must be negotiated with the
;           Office of Technology Transfer at the California
;           Institute of Technology.
;
; 
;  This software may be subject to U.S. export control laws. 
;  By accepting this software, the user agrees to comply with all 
;  applicable U.S. export laws and regulations. User has the 
;  responsibility to obtain export licenses, or other export 
;  authority as may be required before exporting such information 
;  to foreign countries or providing access to foreign persons.
;
;
; read_l2_hdf_summary
;
; PURPOSE:
;	Read a TES L2 HDF SUMMARY data product file and
;	returns the data fields and geolocation for data
;	versions F01, F02.  Data are returned in simple
;	structures of simple arrays with no processing, filtering,
;	etc.
;
;	IDL 6.3+ REQUIRED to read version F01 files.
;
; CALL: 
; 	read_l2_hdf_summary,file, $
; 	  read_swaths = read_swaths,$
; 	  anc_nadir = anc_nadir, $
; 	  anc_limb = anc_limb, $
; 	  tatm_nadir = tatm_nadir, $
; 	  tatm_limb = tatm_limb, $
; 	  o3_nadir = o3_nadir, $
; 	  o3_limb = o3_limb, $
; 	  co_nadir = co_nadir, $
; 	  co_limb = co_limb, $
; 	  h2o_nadir = h2o_nadir, $
; 	  h2o_limb = h2o_limb, $
; 	  hdo_nadir = hdo_nadir, $
; 	  hdo_limb = hdo_limb, $
; 	  ch4_nadir = ch4_nadir, $
; 	  ch4_limb = ch4_limb, $
; 	  hno3_limb=hno3_limb, $
; 	  no2_limb=no2_limb, $
; 	  file_attributes = file_attributes, $
; 	  hdfeos_data = hdfeos_data, $
; 	  status = status,read_ph=read_ph, $
; 	  show_options = show_options, $
; 	  list_swaths = list_swaths, $
; 	  actual_swaths = actual_swaths
;
;
; INPUTS:
;	file = name of file to be read (string)
;
; KEYWORD INPUT:
;	read_swaths = a string (or string array) of the names of the
;		swaths to be read.  By default all swaths in the file are read,
;		but this can take some time due to the size of the file.  If
;		a subset of the data is desired, then specify the
;		subset using this keyword.  For example, if only
;		the nadir O3 data is desired, then read_swaths
;		would be 'o3_nadir'.  If O3 and CO nadir are desired, the
;		read_swaths = ['o3_nadir', 'co_nadir']
;
;		Recognized values for the read_swaths keyword (and
;		meaning):
;		ancillary 	: return ancillary nadir and limb swaths
;		ancillary_nadir : ancillary nadir swath
;		ancillary_limb 	: ancillary limb swath
;		tatm 		: tatm nadir and limb swathes
;		tatm_nadir	: tatn nadir swath
;		tatm_limb	: tatm limb swath
;		o3		: o3 nadir and limb swathes
;		o3_nadir	: o3 nadir swath
;		o3_limb		: o3 limb swath
;		co 		: co nadir and limb swathes
;		co_nadir	: co nadir swath
;		co_limb		: co limb swath
;		h2o 		: h2o nadir and limb swathes
;		h2o_nadir	: h2o nadir swath
;		h2o_limb	: h2o limb swath
;		hdo 		: hdo nadir and limb swathes
;		hdo_nadir	: hdo nadir swath
;		hdo_limb	: hdo limb swath
;		ch4 		: ch4 nadir and limb swathes
;		ch4_nadir	: ch4 nadir swath
;		ch4_limb	: ch4 limb swath
;		hno3_limb	: hno3 limb swath
;		no2_limb	: no2 limb swath
;
;		Swath names as they appear in the file
;		(e.g. O3NadirSwath) may also be used
;
;
;		Specified swaths that are not found in the file will
;		have their output keywords undefined.
;
;	read_ph = if set, read production history.  If not set, the
;		production history will be a null string
;
;	show_options : if this keyword is set, a list of the available
;		swaths will be printed to the screen.  All other input
;		will be ignored in this case.
;
;	list_swaths : if this keyword is set, a list of the swaths
;		present in the file will be printed to the command line.
;
;
; OUTPUT:
;	(none)
;
; KEYWORD OUTPUT:
;	actual_swaths : a string array of the names of the swaths
;		found in the input file
;
;	The complete list of possible outputs is shown below.
;	Not all files will have all of the swaths present.
;	The definition of the swath structure is shown below the list.
;	Note that if the user requests a swath that is not present in
;	the file the program will print a warning and set a status flag,
;	and the corresponding keyword will be undefined.
;
;
;	Possible swaths:
;		ancillary_nadir 	: ancillary nadir swath
;		ancillary_limb 		: ancillary limb swath
;		tatm_nadir		: tatm nadir swath
;		tatm_limb		: tatm limb swath
;		o3_nadir		: o3 nadir swath
;		o3_limb			: o3 limb swath
;		co_nadir		: co nadir swath
;		co_limb			: co limb swath
;		h2o_nadir		: h2o nadir swath
;		h2o_limb		: h2o limb swath
;		hdo_nadir		: hdo nadir swath
;		hdo_limb		: hdo limb swath
;		ch4_nadir		: ch4 nadir swath
;		ch4_limb		: ch4 limb swath
;		hno3_limb		: hno3 limb swath
;		no2_limb		: no2 limb swath
;
;	A returned swath output keyword has the following structure
;	(note that ancillary swaths and species swaths have different
;	 structures)
;
;       (IDL code used to define the structures)
;
;	ancillary_(limb or nadir) = {
;		  anc_data : anc_data, $
;		  geo_data : geo_data, $
;		  CloudFrequency : CloudFrequency, $
;		  EmissivityWavenumber : EmissivityWavenumber, $
;		  Pressure : Pressure, $
;		  VerticalCoordinate : VerticalCoordinate, $
;		  Scan_Resolution : Scan_Resolution }
;		  
;
;	species_(limb or nadir)= {
;		  species_data : species_data, $
;		  geo_data : geo_data, $
;		  CloudFrequency : CloudFrequency, $
;		  EmissivityWavenumber : EmissivityWavenumber, $
;		  Pressure : Pressure, $
;		  VerticalCoordinate : VerticalCoordinate }
;
;	CloudFrequency, EmissivityWavenumber, Pressure,
;	VerticalCoordinate, and Scan_Resolution are swath level
;	attributes.  Pressure is the TES standard pressure grid.
;  
;	The structure members are detailed below.
;
;	anc_data :
;               Filter_Position_1A
;               Filter_Position_1B
;               Filter_Position_2A
;               Filter_Position_2B
;               OrbitAscendingFlag
;		OzoneTroposphericColumn  (Nadir only)
;		OzoneTroposphericColumnError  (Nadir only)
;		OzoneTroposphericColumnInitial  (Nadir only)
;               PixelsUsedFlag
;		RetrievedPointingAngle  (Limb Only)
;		RetrievedPointingAngleError  (Limb Only)
;               SolarAzimuthAngle
;               SpacecraftAltitude
;               SpacecraftLatitude
;               SpacecraftLongitude
;		SurfaceEmissErrors (Nadir only)
;		SurfaceEmissInitial  (Nadir only)
;		SurfaceEmissivity  (Nadir only)
;		TropopausePressure 
;	    	UTCTime  
;
;	species_data = a structure containing all of the data fields
;		       found in the species swath group.  These
;		       fields are (alphabetically except the first two):
;                species  (i.e. O3 [for ozone] in file)
;                Precision (in file this is 'species'Precision .i.e O3Precision)
;		 Altitude
;		 AverageCloudEffOpticalDepth 
;		 AverageCloudEffOpticalDepthError 
;		 Calibration_QA
;                CloudTopPressure
;                CloudTopPressureError 
;		 CloudVariability_QA 
;                DegreesOfFreedomForSignal
;                DeviationVsRetrievalCovariance
;		 H2O_HDO_Corr_QA  
;                InformationContent
;		 Initial 
;		 KDotDL_QA 
;		 LDotDL_QA 
;                MaxNumIterations
;                NumberIterPerformed 
;		 O3_Ccurve_QA (F02+) (Nadir only)
;                Pressure
;                RadianceResidualMax
;                RadianceResidualMean
;                RadianceResidualRMS
;                Scan_Averaged_Count (Nadir Only)
;                SpeciesRetrievalConverged
;                SpeciesRetrievalQuality 
;		 SurfaceEmissMean_QA  (Nadir Only)
;		 SurfaceEmissionLayer_QA   (Nadir Only)
;		 SurfaceTempDegreesOfFreedom  (Nadir Only)
;                SurfaceTempError (Nadir Only) 
;		 SurfaceTempInitial  (Nadir Only)
;		 SurfaceTempObservationError  (Nadir Only)
;		 SurfaceTempPrecision (Nadir Only)
;		 SurfaceTempVsApriori_QA  (Nadir Only)
;		 SurfaceTempVsAtmTemp_QA  (Nadir Only)
;                SurfaceTemperature (Nadir Only)
;                TotalColumnDensity
;                TotalColumnDensityError
;                TotalColumnDensityInitial 
;                TotalError
;	    	 UTCTime  
;	
;		where (species) is the retrieved species or
;		temperature (O3, CH4, CO, H2O, ATM, ...)
;
;
;
;	geo_data = a structure containing all of the geolocation
;	fields found in the species swath group.  These fields
;	are :
;
;           BoresightAzimuth
;	    BoresightNadirAngle 
;           BoresightNadirAngleUnc
;	    BoresightTangentHeight  (Limb Only)
;	    BoresightTangentHeightUnc  (Limb Only)
;	    DayNightFlag 
;	    DominantSurfaceType 
;           Latitude
;           LocalSolarTime
;           Longitude
;	    Scan 
;	    Sequence 
;           SolarZenithAngle
;           SurfaceElevStandardDeviation
;	    SurfaceTypeFootprint 
;           Tgt_SpacecraftAzimuth
;           Tgt_SpacecraftZenith
;           Time
;;
;    Note:  All anc_data fields, species_data fields, and geo_data
;    	fields have the following attributes in the HDF file.  This
;    	program does NOT return them.
;
;		_FillValue
;		MissingValue
;		Title
;		Units
;	 	UniqueFieldDefinition
;
;	The code hooks are in place, but no attemp to keep track of
;	them has been made.  The program author does not anticipate 
;	user demand for these and decided to skip the complexity of
;	having them in the returned data.  Since they are all read,
;	a user wanting this information needs only to collect and
;	return the information (by creating a altered version of this
;	program).
; 
; 
;	file_attributes = a structure containing the information
;		contained in the file attributes group in the
;		additional group.  The fields in this structure are:
;
;                  ProductionHistory  (not read by default--see input
;                      keywords--this is added by the TES SIPS and is thus 
;                      not present in files produced by the TES SCF)
;                  InstrumentName 
;                  ProcessLevel 
;                  TAI93At0zOfGranule (TAI93 At "zero" z Of Granule)
;                  GlobalSurveyNumber
;                  GranuleMonth
;                  GranuleDay
;                  GranuleYear
;                  SurveyMode
;                  PGEVersion
;		   Command_Seq_ID 
;
;	hdfeos_data = a structure that contains the information found 
;		in the HDFEOS Information group.  The fields are:
;              		StructMetadata 
;			coremetadata
;			HDFEOSVersion
;
;	status = Contains success or failure status of the read
;		 0 for success
;		 1 if file does not exist
;		 2 the file name does not have  F01, F02, or F03 in
;		     name
;		 3 the file is not HDF 5
;		 4 No valid swaths found in read_swaths input
;		 5 the file name does not have the expected .he5
;		     extension
;		 6 File is  not SUMMARY type file
;		 7 IDL version incompatability
;		 8 A requested swath not found in file
;
; NOTE:  On any error except those listed, the program will stop in
; the standard IDL fashion at the error location.  No other error
; handling is built into this procedure other than those shown in the 
; status list.
;		 
; ORIGINAL AUTHOR:
;	Brendan Fisher, JPL, 09/2006
;
; UPDATE HISTORY:
;
;	Fri Sep 22 15:16:28 2006, Brendan Fisher 
;	Brendan Fisher, JPL, 10/2006 completed initial version
;	2007/12, Brendan Fisher, JPL, R11 capability
;
;-

pro read_l2_hdf_summary,file, $
  read_swaths = read_swaths,$
  anc_nadir = anc_nadir, $
  anc_limb = anc_limb, $
  tatm_nadir = tatm_nadir, $
  tatm_limb = tatm_limb, $
  o3_nadir = o3_nadir, $
  o3_limb = o3_limb, $
  co_nadir = co_nadir, $
  co_limb = co_limb, $
  h2o_nadir = h2o_nadir, $
  h2o_limb = h2o_limb, $
  hdo_nadir = hdo_nadir, $
  hdo_limb = hdo_limb, $
  ch4_nadir = ch4_nadir, $
  ch4_limb = ch4_limb, $
  hno3_limb=hno3_limb, $
  no2_limb=no2_limb, $
  file_attributes = file_attributes, $
  hdfeos_data = hdfeos_data, $
  status = status,read_ph=read_ph, $
  show_options = show_options, $
  list_swaths = list_swaths, $
  actual_swaths = actual_swaths

if keyword_set(show_options) then begin
    print,'Available swath names: '
    print,'Keyword name        	: definition'
    print,'----------------------------------'
    print,'ancillary 		: return ancillary nadir and limb swaths'
    print,'ancillary_nadir 	: ancillary nadir swath'
    print,'ancillary_limb		: ancillary limb swath'
    print,'tatm			: tatm nadir and limb swathes'
    print,'tatm_nadir		: tatn nadir swath'
    print,'tatm_limb		: tatm limb swath'
    print,'o3			: o3 nadir and limb swathes'
    print,'o3_nadir		: o3 nadir swath'
    print,'o3_limb			: o3 limb swath'
    print,'co 			: co nadir and limb swathes'
    print,'co_nadir		: co nadir swath'
    print,'co_limb			: co limb swath'
    print,'h2o 			: h2o nadir and limb swathes'
    print,'h2o_nadir		: h2o nadir swath'
    print,'h2o_limb		: h2o limb swath'
    print,'hdo 			: hdo nadir and limb swathes'
    print,'hdo_nadir		: hdo nadir swath'
    print,'hdo_limb		: hdo limb swath'
    print,'ch4 			: ch4 nadir and limb swathes'
    print,'ch4_nadir		: ch4 nadir swath'
    print,'ch4_limb		: ch4 limb swath'
    print,'hno3_limb		: hno3 limb swath'
    print,'no2_limb		: no2 limb swath'
    print
    print,'Swath names as they appear in the file may also be used.'

    return

endif


status=0
if n_elements(read_ph) eq 0 then read_ph = 0

; check for file existence
ff = file_search(file,count=count)
if count eq 0 then begin
    print,'Cannot find file:'
    print,file
    status = 1
    return
endif

; check version
pos1 = strpos(file,'_F01_')
pos2 = strpos(file,'_F02_')
if pos1 eq -1 and pos2 eq -1 then begin
    print,'Expected version _F01_, or _F02_ in file name not found.'
    print,file
    print,'This file format not supported.'
    status = 2
    return
endif
if pos1 ne -1 then version = 1
if pos2 ne -1 then version = 2

; check to see if file is an HDF5 
if not h5f_is_hdf5(file) then begin
    print,'File is not HDF 5:'
    print,file
    status=3
    return
endif

; check type
spos = strpos(file,'SUMMARY')
if spos eq -1 then begin
    print,'File does not seem to be a SUMMARY-type product file.'
    print,'File name: ',file
    status=6
    return
endif
; IDL version checking
iversion = !version
idl_version = float(iversion.release)
if version ge 1 and idl_version lt 6.3 then begin
    print,'IDL v6.3+ required for TES SUMMARY format version F01 files.'
    status = 7
    return
endif

;
; open file to data fields
;
fid = h5f_open(file)
hdfid = h5g_open(fid,'HDFEOS')
swid = h5g_open(hdfid,'SWATHS')

; check which swaths are in file
ngroup = h5g_get_nmembers(fid,'/HDFEOS/SWATHS/')
names = strarr(ngroup)
for i=0,ngroup-1 do names[i] = h5g_get_member_name(fid,'/HDFEOS/SWATHS/',i)
actual_swaths = names
if keyword_set(list_swaths) then print,names
;print,names
; start parsing read_swaths
;
; default to all
if n_elements(read_swaths) eq 0 then begin
    read_swaths = names
    rswaths = read_swaths
endif else begin
;
; translate read_swaths to actual swath names
; 
    real_swaths = ['AncillaryNadirSwath',$
                   'AncillaryLimbSwath',$
                   'TATMNadirSwath',$
                   'TATMLimbSwath',$
                   'O3NadirSwath',$
                   'O3LimbSwath',$
                   'CONadirSwath',$
                   'COLimbSwath',$
                   'H2ONadirSwath',$
                   'H2OLimbSwath',$
                   'HDONadirSwath',$
                   'HDOLimbSwath',$
                   'CH4NadirSwath',$
                   'CH4LimbSwath',$
                   'HNO3LimbSwath',$
                   'NO2LimbSwath']

    rswaths = ['']
    for i=0,n_elements(read_swaths)-1 do begin
        case strlowcase(read_swaths[i]) of
            'ancillary' : begin
                rswaths = [rswaths,  $
                           'AncillaryNadirSwath','AncillaryLimbSwath']
            end
            'ancillary_nadir' : begin
                rswaths = [rswaths,'AncillaryNadirSwath']
            end
            'ancillary_limb' : begin
                rswaths = [rswaths,'AncillaryLimbSwath']
            end
            'tatm' : begin
                rswaths = [rswaths,'TATMNadirSwath','TATMLimbSwath']
            end
            'tatm_nadir' : begin
                rswaths = [rswaths,'TATMNadirSwath']
            end
            'tatm_limb' : begin
                rswaths = [rswaths,'TATMLimbSwath']
            end
            'o3' : begin
                rswaths = [rswaths,'O3NadirSwath','O3LimbSwath']
            end
            'o3_nadir' : begin
                rswaths = [rswaths,'O3NadirSwath']
            end
            'o3_limb' : begin
                rswaths = [rswaths,'O3LimbSwath']
            end
            'co' : begin
                rswaths = [rswaths,'CONadirSwath','COLimbSwath']
            end
            'co_nadir' : begin
                rswaths = [rswaths,'CONadirSwath']
            end
            'co_limb' : begin
                rswaths = [rswaths,'COLimbSwath']
            end
            'h2o' : begin
                rswaths = [rswaths,'H2ONadirSwath','H2OLimbSwath']
            end
            'h2o_nadir' : begin
                rswaths = [rswaths,'H2ONadirSwath']
            end
            'h2o_limb' : begin
                rswaths = [rswaths,'H2OLimbSwath']
            end
            'hdo' : begin
                rswaths = [rswaths,'HDONadirSwath','HDOLimbSwath']
            end
            'hdo_nadir' : begin
                rswaths = [rswaths,'HDONadirSwath']
            end
            'hdo_limb' : begin
                rswaths = [rswaths,'HDOLimbSwath']
            end
            'ch4' : begin
                rswaths = [rswaths,'CH4NadirSwath','CH4LimbSwath']
            end
            'ch4_nadir' : begin
                rswaths = [rswaths,'CH4NadirSwath']
            end
            'ch4_limb' : begin
                rswaths = [rswaths,'CH4LimbSwath']
            end
            'hno3_limb' : begin
                rswaths = [rswaths,'HNO3LimbSwath']
            end
            'no2_limb' : begin
                rswaths = [rswaths,'NO2LimbSwath']
            end
            else : begin
                pos = where(strlowcase(real_swaths) eq $
                            strlowcase(read_swaths[i]))
                if pos[0] ne -1 then rswaths = [rswaths,read_swaths[i]]
            end
        endcase
    endfor
    if n_elements(rswaths) gt 1 then $
      rswaths = rswaths[1:*]
endelse
if n_elements(rswaths) eq 1 and rswaths[0] eq '' then begin
    print,'No valid swaths found'
    status=4
    return
endif
;
; read the data
;
swath = 'AncillaryLimbSwath'
swpos = where(rswaths eq swath)
nspos = where(names eq swath)
if nspos eq -1 then begin
    status = 8
    print,'WARNING:  Requested swath '+swath+' not present in file.'
endif
if swpos ne -1 and nspos ne -1 then begin
    specid = h5g_open(swid,swath)
    fieldid = h5g_open(specid,'Data Fields')

    dataid = h5d_open(fieldid,'Filter_Position_1A')
    Filter_Position_1A = h5d_read(dataid)
    get_data_attributes,dataid,size(Filter_Position_1A,/type),attrib1
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'Filter_Position_1B')
    Filter_Position_1B = h5d_read(dataid)
    get_data_attributes,dataid,size(Filter_Position_1B,/type),attrib2
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Filter_Position_2A')
    Filter_Position_2A = h5d_read(dataid)
    get_data_attributes,dataid,size(Filter_Position_2A,/type),attrib3
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'Filter_Position_2B')
    Filter_Position_2B = h5d_read(dataid)
    get_data_attributes,dataid,size(Filter_Position_2B,/type),attrib4
    h5d_close,dataid
    ;    
    dataid = h5d_open(fieldid,'OrbitAscendingFlag')
    OrbitAscendingFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(OrbitAscendingFlag,/type),attrib6
    h5d_close,dataid
; 
    dataid = h5d_open(fieldid,'PixelsUsedFlag')
    PixelsUsedFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(PixelsUsedFlag,/type),attrib7,/special
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RetrievedPointingAngle')
    RetrievedPointingAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(RetrievedPointingAngle,/type),attrib8
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RetrievedPointingAngleError')
    RetrievedPointingAngleError = h5d_read(dataid)
    get_data_attributes,dataid,size(RetrievedPointingAngleError,/type), $
                        attrib9
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SolarAzimuthAngle')
    SolarAzimuthAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(SolarAzimuthAngle,/type),attrib10
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'SpacecraftAltitude')
    SpacecraftAltitude = h5d_read(dataid)
    get_data_attributes,dataid,size(SpacecraftAltitude,/type),attrib11
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'SpacecraftLatitude')
    SpacecraftLatitude = h5d_read(dataid)
    get_data_attributes,dataid,size(SpacecraftLatitude,/type),attrib12
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'SpacecraftLongitude')
    SpacecraftLongitude = h5d_read(dataid)
    get_data_attributes,dataid,size(SpacecraftLongitude,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TropopausePressure')
    TropopausePressure = h5d_read(dataid)
    get_data_attributes,dataid,size(TropopausePressure,/type),attrib16
    h5d_close,dataid

    dataid = h5d_open(fieldid,'UTCTime')
    UTCTime = h5d_read(dataid)
    get_data_attributes,dataid,size(utctime,/type),attrib_utc,/utctime
    h5d_close,dataid
;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressure = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'Scan_Resolution')
    Scan_Resolution = h5a_read(id)
    h5a_close,id

    anc_data = { $
               Filter_Position_1A : temporary(Filter_Position_1A), $
               Filter_Position_1B : temporary(Filter_Position_1B), $
               Filter_Position_2A : temporary(Filter_Position_2A), $
               Filter_Position_2B : temporary(Filter_Position_2B), $
               OrbitAscendingFlag : temporary(OrbitAscendingFlag), $
               PixelsUsedFlag : temporary(PixelsUsedFlag), $
               RetrievedPointingAngle : temporary(RetrievedPointingAngle), $
	       RetrievedPointingAngleError :temporary(RetrievedPointingAngleError), $
               SolarAzimuthAngle : temporary(SolarAzimuthAngle), $
               SpacecraftAltitude : temporary(SpacecraftAltitude), $
               SpacecraftLatitude : temporary(SpacecraftLatitude), $
               SpacecraftLongitude : temporary(SpacecraftLongitude), $
               TropopausePressure : temporary(TropopausePressure), $
               UTCTime : temporary(UTCTime) }

    h5g_close,fieldid
; get geolocation data
    geoid = h5g_open(specid,'Geolocation Fields')
;
;
    dataid = h5d_open(geoid,'BoresightAzimuth')
    BoresightAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightAzimuth,/type),attrib0
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'BoresightNadirAngle')
    BoresightNadirAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngle,/type),attrib1
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'BoresightNadirAngleUnc')
    BoresightNadirAngleUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngleUnc,/type),attrib2
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightTangentHeight')
    BoresightTangentHeight = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightTangentHeight,/type),attrib3
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'BoresightTangentHeightUnc')
    BoresightTangentHeightUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightTangentHeightUnc,/type),attrib4
    h5d_close,dataid

    dataid = h5d_open(geoid,'DayNightFlag')
    DayNightFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(DayNightFlag,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude,/type),attrib6
    h5d_close,dataid
       
    dataid = h5d_open(geoid,'LocalSolarTime')
    LocalSolarTime = h5d_read(dataid)
    get_data_attributes,dataid,size(LocalSolarTime,/type),attrib11
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude,/type),attrib12
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    get_data_attributes,dataid,size(Scan,/type),attrib17
    h5d_close,dataid
        
    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    get_data_attributes,dataid,size(Sequence,/type),attrib18
    h5d_close,dataid
 
    dataid = h5d_open(geoid,'SolarZenithAngle')
    SolarZenithAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(SolarZenithAngle,/type),attrib19
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'SurfaceElevStandardDeviation')
    SurfaceElevStandardDeviation = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceElevStandardDeviation,/type),attrib20
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'SurfaceTypeFootprint')
    SurfaceTypeFootprint = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTypeFootprint,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(geoid,'Tgt_SpacecraftAzimuth')
    Tgt_SpacecraftAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftAzimuth,/type),attrib22
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Tgt_SpacecraftZenith')
    Tgt_SpacecraftZenith = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftZenith,/type),attrib23
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Time')
    Time = h5d_read(dataid)
    get_data_attributes,dataid,size(Time,/type),attrib24
    h5d_close,dataid

; clean up
    h5g_close,geoid
    h5g_close,specid

    geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
	   BoresightTangentHeight : temporary(BoresightTangentHeight), $
	   BoresightTangentHeightUnc : temporary(BoresightTangentHeightUnc), $
           DayNightFlag : temporary(DayNightFlag), $
           Latitude : temporary(Latitude), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           SurfaceTypeFootprint : temporary(SurfaceTypeFootprint), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) $
               }

    anc_limb = { $
      anc_data : temporary(anc_data), $
      geo_data : temporary(geo_data), $
      CloudFrequency : temporary(CloudFrequency), $
      EmissivityWavenumber : temporary(EmissivityWavenumber), $
      Pressure : temporary(Pressure), $
      VerticalCoordinate : temporary(VerticalCoordinate), $
      Scan_Resolution : temporary(Scan_Resolution) }
		  
endif ; ancillary limb swath


swath = 'AncillaryNadirSwath'
swpos = where(rswaths eq swath)
nspos = where(names eq swath)
if nspos eq -1 then begin
    status = 8
    print,'WARNING:  Requested swath '+swath+' not present in file.'
endif
if swpos ne -1 and nspos ne -1 then begin
    specid = h5g_open(swid,swath)
; data fields
    fieldid = h5g_open(specid,'Data Fields')
;
; actual data, alphabetically
;

    dataid = h5d_open(fieldid,'Filter_Position_1A')
    Filter_Position_1A = h5d_read(dataid)
    get_data_attributes,dataid,size(Filter_Position_1A,/type),attrib1
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'Filter_Position_1B')
    Filter_Position_1B = h5d_read(dataid)
    get_data_attributes,dataid,size(Filter_Position_1B,/type),attrib2
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Filter_Position_2A')
    Filter_Position_2A = h5d_read(dataid)
    get_data_attributes,dataid,size(Filter_Position_2A,/type),attrib3
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'Filter_Position_2B')
    Filter_Position_2B = h5d_read(dataid)
    get_data_attributes,dataid,size(Filter_Position_2B,/type),attrib4
    h5d_close,dataid
        
    dataid = h5d_open(fieldid,'OrbitAscendingFlag')
    OrbitAscendingFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(OrbitAscendingFlag,/type),attrib8
    h5d_close,dataid
; 
    dataid = h5d_open(fieldid,'OzoneTroposphericColumn')
    OzoneTroposphericColumn = h5d_read(dataid)
    get_data_attributes,dataid,size(OzoneTroposphericColumn,/type),attrib9
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'OzoneTroposphericColumnError')
    OzoneTroposphericColumnError = h5d_read(dataid)
    get_data_attributes,dataid,size(OzoneTroposphericColumnError,/type),attrib10
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'OzoneTroposphericColumnInitial')
    OzoneTroposphericColumnInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(OzoneTroposphericColumnInitial,/type),attrib11
    h5d_close,dataid

    dataid = h5d_open(fieldid,'PixelsUsedFlag')
    PixelsUsedFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(PixelsUsedFlag,/type),attrib12,/special
    h5d_close,dataid
;    
    dataid = h5d_open(fieldid,'SolarAzimuthAngle')
    SolarAzimuthAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(SolarAzimuthAngle,/type),attrib13
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'SpacecraftAltitude')
    SpacecraftAltitude = h5d_read(dataid)
    get_data_attributes,dataid,size(SpacecraftAltitude,/type),attrib14
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'SpacecraftLatitude')
    SpacecraftLatitude = h5d_read(dataid)
    get_data_attributes,dataid,size(SpacecraftLatitude,/type),attrib15
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'SpacecraftLongitude')
    SpacecraftLongitude = h5d_read(dataid)
    get_data_attributes,dataid,size(SpacecraftLongitude,/type),attrib16
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'SurfaceEmissErrors')
    SurfaceEmissErrors = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceEmissErrors,/type),attrib18
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'SurfaceEmissInitial')
    SurfaceEmissInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceEmissInitial,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceEmissivity')
    SurfaceEmissivity = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceEmissivity,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TropopausePressure')
    TropopausePressure = h5d_read(dataid)
    get_data_attributes,dataid,size(TropopausePressure,/type),attrib23
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'UTCTime')
    UTCTime = h5d_read(dataid)
    get_data_attributes,dataid,size(utctime,/type),attrib_utc,/utctime
    h5d_close,dataid
;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressure = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'Scan_Resolution')
    Scan_Resolution = h5a_read(id)
    h5a_close,id

    h5g_close,fieldid

    anc_data = { $
               Filter_Position_1A : temporary(Filter_Position_1A), $
               Filter_Position_1B : temporary(Filter_Position_1B), $
               Filter_Position_2A : temporary(Filter_Position_2A), $
               Filter_Position_2B : temporary(Filter_Position_2B), $
               OrbitAscendingFlag : temporary(OrbitAscendingFlag), $
               OzoneTroposphericColumn : temporary(OzoneTroposphericColumn), $
               OzoneTroposphericColumnError : temporary(OzoneTroposphericColumnError), $
               OzoneTroposphericColumnInitial : temporary(OzoneTroposphericColumnInitial), $
               PixelsUsedFlag : temporary(PixelsUsedFlag), $
               SolarAzimuthAngle : temporary(SolarAzimuthAngle), $
               SpacecraftAltitude : temporary(SpacecraftAltitude), $
               SpacecraftLatitude : temporary(SpacecraftLatitude), $
               SpacecraftLongitude : temporary(SpacecraftLongitude), $
               SurfaceEmissErrors : temporary(SurfaceEmissErrors), $
               SurfaceEmissInitial : temporary(SurfaceEmissInitial), $
               SurfaceEmissivity : temporary(SurfaceEmissivity), $
               TropopausePressure : temporary(TropopausePressure), $
               UTCTime : temporary(UTCTime) $
                     }


; get geolocation data
;
    geoid = h5g_open(specid,'Geolocation Fields')
;
;
    dataid = h5d_open(geoid,'BoresightAzimuth')
    BoresightAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightAzimuth,/type),attrib0
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'BoresightNadirAngle')
    BoresightNadirAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngle,/type),attrib1
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'BoresightNadirAngleUnc')
    BoresightNadirAngleUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngleUnc,/type),attrib2
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'DayNightFlag')
    DayNightFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(DayNightFlag,/type),attrib3
    h5d_close,dataid

    dataid = h5d_open(geoid,'DominantSurfaceType')
    DominantSurfaceType = h5d_read(dataid)
    get_data_attributes,dataid,size(DominantSurfaceType,/type),attrib4
    h5d_close,dataid

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude,/type),attrib5
    h5d_close,dataid
       
    dataid = h5d_open(geoid,'LocalSolarTime')
    LocalSolarTime = h5d_read(dataid)
    get_data_attributes,dataid,size(LocalSolarTime,/type),attrib10
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude,/type),attrib11
    h5d_close,dataid
        
    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    get_data_attributes,dataid,size(Scan,/type),attrib16
    h5d_close,dataid
        
    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    get_data_attributes,dataid,size(Sequence,/type),attrib17
    h5d_close,dataid

    dataid = h5d_open(geoid,'SolarZenithAngle')
    SolarZenithAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(SolarZenithAngle,/type),attrib18
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'SurfaceElevStandardDeviation')
    SurfaceElevStandardDeviation = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceElevStandardDeviation,/type),attrib19
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'SurfaceTypeFootprint')
    SurfaceTypeFootprint = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTypeFootprint,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(geoid,'Tgt_SpacecraftAzimuth')
    Tgt_SpacecraftAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftAzimuth,/type),attrib21
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Tgt_SpacecraftZenith')
    Tgt_SpacecraftZenith = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftZenith,/type),attrib22
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Time')
    Time = h5d_read(dataid)
    get_data_attributes,dataid,size(Time,/type),attrib23
    h5d_close,dataid

; clean up
    h5g_close,geoid
    h5g_close,specid

    geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           DayNightFlag : temporary(DayNightFlag), $
           DominantSurfaceType : temporary(DominantSurfaceType), $
           Latitude : temporary(Latitude), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           SurfaceTypeFootprint : temporary(SurfaceTypeFootprint), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) $
               }


    anc_nadir = { $
      anc_data : temporary(anc_data), $
      geo_data : temporary(geo_data), $
      CloudFrequency : temporary(CloudFrequency), $
      EmissivityWavenumber : temporary(EmissivityWavenumber), $
      Pressure : temporary(Pressure), $
      VerticalCoordinate : temporary(VerticalCoordinate), $
      Scan_Resolution : temporary(Scan_Resolution) }

endif ; ancillary nadir swath


swath = 'TATMNadirSwath'
swpos = where(rswaths eq swath)
nspos = where(names eq swath)
if nspos eq -1 then begin
    status = 8
    print,'WARNING:  Requested swath '+swath+' not present in file.'
endif
if swpos ne -1 and nspos ne -1 then begin
    specid = h5g_open(swid,swath)
; data fields
    fieldid = h5g_open(specid,'Data Fields')

; READ DATA
    gas = 'TATM'
    dataid = h5d_open(fieldid,gas)
    species = h5d_read(dataid)
    get_data_attributes,dataid,size(species,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(fieldid,gas+'Precision')
    Precision = h5d_read(dataid)
    get_data_attributes,dataid,size(Precision,/type),attrib1
    h5d_close,dataid

; the rest, alphabetically

    dataid = h5d_open(fieldid,'Altitude')
    altitude = h5d_read(dataid)
    get_data_attributes,dataid,size(altitude,/type),attrib4
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepth')
    AverageCloudEffOpticalDepth = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepth,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepthError')
    AverageCloudEffOpticalDepthError = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepthError,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Calibration_QA')
    Calibration_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(Calibration_QA,/type),attrib8
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudTopPressure')
    CloudTopPressure = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressure,/type),attrib12
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudTopPressureError')
    CloudTopPressureError = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressureError,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudVariability_QA')
    CloudVariability_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudVariability_QA,/type),attrib14
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DegreesOfFreedomForSignal')
    DegreesOfFreedomForSignal = h5d_read(dataid)
    get_data_attributes,dataid,size(DegreesOfFreedomForSignal,/type),attrib16
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DeviationVsRetrievalCovariance')
    DeviationVsRetrievalCovariance = h5d_read(dataid)
    get_data_attributes,dataid,size(DeviationVsRetrievalCovariance,/type),attrib17
    h5d_close,dataid

    dataid = h5d_open(fieldid,'H2O_HDO_Corr_QA')
    H2O_HDO_Corr_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(H2O_HDO_Corr_QA,/type),attrib51
    h5d_close,dataid

    dataid = h5d_open(fieldid,'InformationContent')
    InformationContent = h5d_read(dataid)
    get_data_attributes,dataid,size(InformationContent,/type),attrib18
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Initial')
    Initial = h5d_read(dataid)
    get_data_attributes,dataid,size(Initial,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(fieldid,'KDotDL_QA')
    KDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(KDotDL_QA,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(fieldid,'LDotDL_QA')
    LDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(LDotDL_QA,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(fieldid,'MaxNumIterations')
    MaxNumIterations = h5d_read(dataid)
    get_data_attributes,dataid,size(MaxNumIterations,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(fieldid,'NumberIterPerformed')
    NumberIterPerformed = h5d_read(dataid)
    get_data_attributes,dataid,size(NumberIterPerformed,/type),attrib24
    h5d_close,dataid

    if version ge 2 then begin
        dataid = h5d_open(fieldid,'O3_Ccurve_QA')
        O3_Ccurve_QA = h5d_read(dataid)
        get_data_attributes,dataid,size(O3_Ccurve_QA,/type),attrib26a
        h5d_close,dataid
    endif
    
    dataid = h5d_open(fieldid,'Pressure')
    Pressure = h5d_read(dataid)
    get_data_attributes,dataid,size(Pressure,/type),attrib26
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMax')
    RadianceResidualMax = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMax,/type),attrib27
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMean')
    RadianceResidualMean = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMean,/type),attrib28
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualRMS')
    RadianceResidualRMS = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualRMS,/type),attrib29
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Scan_Averaged_Count')
    Scan_Averaged_Count = h5d_read(dataid)
    get_data_attributes,dataid,size(Scan_Averaged_Count,/type),attrib30
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalConverged')
    SpeciesRetrievalConverged = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalConverged,/type),attrib31
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalQuality')
    SpeciesRetrievalQuality = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalQuality,/type),attrib32
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceEmissMean_QA')
    SurfaceEmissMean_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceEmissMean_QA,/type),attrib33
    h5d_close,dataid

;XXXX
    dataid = h5d_open(fieldid,'SurfaceEmissionLayer_QA')
    SurfaceEmissionLayer_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceEmissionLayer_QA,/type),attrib50
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempDegreesOfFreedom')
    SurfaceTempDegreesOfFreedom = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempDegreesOfFreedom,/type),attrib35
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempError')
    SurfaceTempError = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempError,/type),attrib36
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempInitial')
    SurfaceTempInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempInitial,/type),attrib37
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempObservationError')
    SurfaceTempObservationError = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempObservationError,/type),attrib38
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempPrecision')
    SurfaceTempPrecision = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempPrecision,/type),attrib39
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempVsApriori_QA')
    SurfaceTempVsApriori_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempVsApriori_QA,/type),attrib40
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempVsAtmTemp_QA')
    SurfaceTempVsAtmTemp_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempVsAtmTemp_QA,/type),attrib41
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTemperature')
    SurfaceTemperature = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTemperature,/type),attrib42
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensity')
    TotalColumnDensity = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib44
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityError')
    TotalColumnDensityError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityError,/type),attrib45
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityInitial')
    TotalColumnDensityInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityInitial,/type),attrib46
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalError')
    TotalError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalError,/type),attrib47
    h5d_close,dataid

    dataid = h5d_open(fieldid,'UTCTime')
    UTCTime = h5d_read(dataid)
    get_data_attributes,dataid,size(utctime,/type),attrib49,/utctime
    h5d_close,dataid

    h5g_close,fieldid

    if version eq 1 then begin
        species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AverageCloudEffOpticalDepthError : temporary(AverageCloudEffOpticalDepthError), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               Scan_Averaged_Count : temporary(Scan_Averaged_Count), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               SurfaceEmissMean_QA : temporary(SurfaceEmissMean_QA), $    
;XXXX
               SurfaceEmissionLayer_QA : temporary(SurfaceEmissionLayer_QA), $    
               SurfaceTempDegreesOfFreedom : temporary(SurfaceTempDegreesOfFreedom), $    
               SurfaceTempError : temporary(SurfaceTempError), $
               SurfaceTempInitial : temporary(SurfaceTempInitial), $    
               SurfaceTempObservationError : temporary(SurfaceTempObservationError), $    
               SurfaceTempPrecision : temporary(SurfaceTempPrecision), $    
               SurfaceTempVsApriori_QA : temporary(SurfaceTempVsApriori_QA), $    
               SurfaceTempVsAtmTemp_QA : temporary(SurfaceTempVsAtmTemp_QA), $    
               SurfaceTemperature : temporary(SurfaceTemperature), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               UTCTime : temporary(UTCTime) $
                   }

    endif
    if version ge 2 then begin
        species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AverageCloudEffOpticalDepthError : temporary(AverageCloudEffOpticalDepthError), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               O3_Ccurve_QA : temporary(O3_Ccurve_QA), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               Scan_Averaged_Count : temporary(Scan_Averaged_Count), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               SurfaceEmissMean_QA : temporary(SurfaceEmissMean_QA), $    
;XXXX
               SurfaceEmissionLayer_QA : temporary(SurfaceEmissionLayer_QA), $    
               SurfaceTempDegreesOfFreedom : temporary(SurfaceTempDegreesOfFreedom), $    
               SurfaceTempError : temporary(SurfaceTempError), $
               SurfaceTempInitial : temporary(SurfaceTempInitial), $    
               SurfaceTempObservationError : temporary(SurfaceTempObservationError), $    
               SurfaceTempPrecision : temporary(SurfaceTempPrecision), $    
               SurfaceTempVsApriori_QA : temporary(SurfaceTempVsApriori_QA), $    
               SurfaceTempVsAtmTemp_QA : temporary(SurfaceTempVsAtmTemp_QA), $    
               SurfaceTemperature : temporary(SurfaceTemperature), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               UTCTime : temporary(UTCTime) $
                   }
    endif
;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressure= h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id

;
; read in geolocation data
;
    geoid = h5g_open(specid,'Geolocation Fields')
;
;
    dataid = h5d_open(geoid,'BoresightAzimuth')
    BoresightAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightAzimuth,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngle')
    BoresightNadirAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngle,/type),attrib1
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngleUnc')
    BoresightNadirAngleUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngleUnc,/type),attrib2
    h5d_close,dataid

    dataid = h5d_open(geoid,'DayNightFlag')
    DayNightFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(DayNightFlag,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(geoid,'DominantSurfaceType')
    DominantSurfaceType = h5d_read(dataid)
    get_data_attributes,dataid,size(DominantSurfaceType,/type),attrib6
    h5d_close,dataid

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'LocalSolarTime')
    LocalSolarTime = h5d_read(dataid)
    get_data_attributes,dataid,size(LocalSolarTime,/type),attrib12
    h5d_close,dataid

    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    get_data_attributes,dataid,size(Scan,/type),attrib18
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    get_data_attributes,dataid,size(Sequence,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(geoid,'SolarZenithAngle')
    SolarZenithAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(SolarZenithAngle,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceElevStandardDeviation')
    SurfaceElevStandardDeviation = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceElevStandardDeviation,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceTypeFootprint')
    SurfaceTypeFootprint = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTypeFootprint,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(geoid,'Tgt_SpacecraftAzimuth')
    Tgt_SpacecraftAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftAzimuth,/type),attrib23
    h5d_close,dataid

    dataid = h5d_open(geoid,'Tgt_SpacecraftZenith')
    Tgt_SpacecraftZenith = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftZenith,/type),attrib24
    h5d_close,dataid

    dataid = h5d_open(geoid,'Time')
    Time = h5d_read(dataid)
    get_data_attributes,dataid,size(Time,/type),attrib25
    h5d_close,dataid

    geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           DayNightFlag : temporary(DayNightFlag), $
           DominantSurfaceType : temporary(DominantSurfaceType), $
           Latitude : temporary(Latitude), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           SurfaceTypeFootprint : temporary(SurfaceTypeFootprint), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) $
           }


    tatm_nadir= { $
                species_data : temporary(species_data), $
                geo_data : temporary(geo_data), $
                CloudFrequency : temporary(CloudFrequency), $
                EmissivityWavenumber : temporary(EmissivityWavenumber), $
                Pressure : temporary(Pressure), $
                VerticalCoordinate : temporary(VerticalCoordinate) }

endif ; temperature nadir swath


swath = 'TATMLimbSwath'
swpos = where(rswaths eq swath)
nspos = where(names eq swath)
if nspos eq -1 then begin
    status = 8
    print,'WARNING:  Requested swath '+swath+' not present in file.'
endif
if swpos ne -1 and nspos ne -1 then begin
    specid = h5g_open(swid,swath)
; data fields
    fieldid = h5g_open(specid,'Data Fields')

; READ DATA
;
; actual data, alphabetically except for the gas, gasPrecision, and
; gasVerticalResolution
;
    gas = 'TATM'
    dataid = h5d_open(fieldid,gas)
    species = h5d_read(dataid)
    get_data_attributes,dataid,size(species,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(fieldid,gas+'Precision')
    Precision = h5d_read(dataid)
    get_data_attributes,dataid,size(Precision,/type),attrib1
    h5d_close,dataid

; the rest, alphabetically

    dataid = h5d_open(fieldid,'Altitude')
    altitude = h5d_read(dataid)
    get_data_attributes,dataid,size(altitude,/type),attrib4
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepth')
    AverageCloudEffOpticalDepth = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepth,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepthError')
    AverageCloudEffOpticalDepthError = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepthError,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Calibration_QA')
    Calibration_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(Calibration_QA,/type),attrib8
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudTopPressure')
    CloudTopPressure = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressure,/type),attrib12
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'CloudTopPressureError')
    CloudTopPressureError = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressureError,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudVariability_QA')
    CloudVariability_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudVariability_QA,/type),attrib14
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DegreesOfFreedomForSignal')
    DegreesOfFreedomForSignal = h5d_read(dataid)
    get_data_attributes,dataid,size(DegreesOfFreedomForSignal,/type),attrib16
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DeviationVsRetrievalCovariance')
    DeviationVsRetrievalCovariance = h5d_read(dataid)
    get_data_attributes,dataid,size(DeviationVsRetrievalCovariance,/type),attrib17
    h5d_close,dataid

    dataid = h5d_open(fieldid,'H2O_HDO_Corr_QA')
    H2O_HDO_Corr_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(H2O_HDO_Corr_QA,/type),attrib51
    h5d_close,dataid

    dataid = h5d_open(fieldid,'InformationContent')
    InformationContent = h5d_read(dataid)
    get_data_attributes,dataid,size(InformationContent,/type),attrib18
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Initial')
    Initial = h5d_read(dataid)
    get_data_attributes,dataid,size(Initial,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(fieldid,'KDotDL_QA')
    KDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(KDotDL_QA,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(fieldid,'LDotDL_QA')
    LDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(LDotDL_QA,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(fieldid,'MaxNumIterations')
    MaxNumIterations = h5d_read(dataid)
    get_data_attributes,dataid,size(MaxNumIterations,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(fieldid,'NumberIterPerformed')
    NumberIterPerformed = h5d_read(dataid)
    get_data_attributes,dataid,size(NumberIterPerformed,/type),attrib24
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Pressure')
    Pressure = h5d_read(dataid)
    get_data_attributes,dataid,size(Pressure,/type),attrib26
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMax')
    RadianceResidualMax = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMax,/type),attrib27
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMean')
    RadianceResidualMean = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMean,/type),attrib28
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualRMS')
    RadianceResidualRMS = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualRMS,/type),attrib29
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalConverged')
    SpeciesRetrievalConverged = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalConverged,/type),attrib31
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalQuality')
    SpeciesRetrievalQuality = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalQuality,/type),attrib32
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensity')
    TotalColumnDensity = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib44
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityError')
    TotalColumnDensityError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityError,/type),attrib45
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityInitial')
    TotalColumnDensityInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityInitial,/type),attrib46
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalError')
    TotalError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalError,/type),attrib47
    h5d_close,dataid

    dataid = h5d_open(fieldid,'UTCTime')
    UTCTime = h5d_read(dataid)
    get_data_attributes,dataid,size(utctime,/type),attrib49,/utctime
    h5d_close,dataid

    h5g_close,fieldid

    species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AverageCloudEffOpticalDepthError : temporary(AverageCloudEffOpticalDepthError), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               UTCTime : temporary(UTCTime) $
                   }


;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressure= h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id

;
; get geolocation data
;
    geoid = h5g_open(specid,'Geolocation Fields')
;
;
    dataid = h5d_open(geoid,'BoresightAzimuth')
    BoresightAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightAzimuth,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngle')
    BoresightNadirAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngle,/type),attrib1
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngleUnc')
    BoresightNadirAngleUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngleUnc,/type),attrib2
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightTangentHeight')
    BoresightTangentHeight = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightTangentHeight,/type),attrib3
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightTangentHeightUnc')
    BoresightTangentHeightUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightTangentHeightUnc,/type),attrib4
    h5d_close,dataid

    dataid = h5d_open(geoid,'DayNightFlag')
    DayNightFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(DayNightFlag,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'LocalSolarTime')
    LocalSolarTime = h5d_read(dataid)
    get_data_attributes,dataid,size(LocalSolarTime,/type),attrib12
    h5d_close,dataid

    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    get_data_attributes,dataid,size(Scan,/type),attrib18
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    get_data_attributes,dataid,size(Sequence,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(geoid,'SolarZenithAngle')
    SolarZenithAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(SolarZenithAngle,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceElevStandardDeviation')
    SurfaceElevStandardDeviation = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceElevStandardDeviation,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceTypeFootprint')
    SurfaceTypeFootprint = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTypeFootprint,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(geoid,'Tgt_SpacecraftAzimuth')
    Tgt_SpacecraftAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftAzimuth,/type),attrib23
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Tgt_SpacecraftZenith')
    Tgt_SpacecraftZenith = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftZenith,/type),attrib24
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Time')
    Time = h5d_read(dataid)
    get_data_attributes,dataid,size(Time,/type),attrib25
    h5d_close,dataid

    geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           BoresightTangentHeight : temporary(BoresightTangentHeight), $
           BoresightTangentHeightUnc : temporary(BoresightTangentHeightUnc), $
           DayNightFlag : temporary(DayNightFlag), $
           Latitude : temporary(Latitude), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           SurfaceTypeFootprint : temporary(SurfaceTypeFootprint), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) $
           }

    tatm_limb= { $
               species_data : temporary(species_data), $
               geo_data : temporary(geo_data), $
               CloudFrequency : temporary(CloudFrequency), $
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               Pressure : temporary(Pressure), $
               VerticalCoordinate : temporary(VerticalCoordinate) }


endif ; temperature limb swath


swath = 'O3NadirSwath'
swpos = where(rswaths eq swath)
nspos = where(names eq swath)
if nspos eq -1 then begin
    status = 8
    print,'WARNING:  Requested swath '+swath+' not present in file.'
endif
if swpos ne -1 and nspos ne -1 then begin
    specid = h5g_open(swid,swath)
; data fields
    fieldid = h5g_open(specid,'Data Fields')

; READ DATA
    gas = 'O3'
    dataid = h5d_open(fieldid,gas)
    species = h5d_read(dataid)
    get_data_attributes,dataid,size(species,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(fieldid,gas+'Precision')
    Precision = h5d_read(dataid)
    get_data_attributes,dataid,size(Precision,/type),attrib1
    h5d_close,dataid

; the rest, alphabetically

    dataid = h5d_open(fieldid,'Altitude')
    altitude = h5d_read(dataid)
    get_data_attributes,dataid,size(altitude,/type),attrib4
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepth')
    AverageCloudEffOpticalDepth = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepth,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepthError')
    AverageCloudEffOpticalDepthError = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepthError,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Calibration_QA')
    Calibration_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(Calibration_QA,/type),attrib8
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudTopPressure')
    CloudTopPressure = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressure,/type),attrib12
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudTopPressureError')
    CloudTopPressureError = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressureError,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudVariability_QA')
    CloudVariability_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudVariability_QA,/type),attrib14
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DegreesOfFreedomForSignal')
    DegreesOfFreedomForSignal = h5d_read(dataid)
    get_data_attributes,dataid,size(DegreesOfFreedomForSignal,/type),attrib16
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DeviationVsRetrievalCovariance')
    DeviationVsRetrievalCovariance = h5d_read(dataid)
    get_data_attributes,dataid,size(DeviationVsRetrievalCovariance,/type),attrib17
    h5d_close,dataid

    dataid = h5d_open(fieldid,'H2O_HDO_Corr_QA')
    H2O_HDO_Corr_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(H2O_HDO_Corr_QA,/type),attrib51
    h5d_close,dataid

    dataid = h5d_open(fieldid,'InformationContent')
    InformationContent = h5d_read(dataid)
    get_data_attributes,dataid,size(InformationContent,/type),attrib18
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Initial')
    Initial = h5d_read(dataid)
    get_data_attributes,dataid,size(Initial,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(fieldid,'KDotDL_QA')
    KDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(KDotDL_QA,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(fieldid,'LDotDL_QA')
    LDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(LDotDL_QA,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(fieldid,'MaxNumIterations')
    MaxNumIterations = h5d_read(dataid)
    get_data_attributes,dataid,size(MaxNumIterations,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(fieldid,'NumberIterPerformed')
    NumberIterPerformed = h5d_read(dataid)
    get_data_attributes,dataid,size(NumberIterPerformed,/type),attrib24
    h5d_close,dataid

    if version ge 2 then begin
        dataid = h5d_open(fieldid,'O3_Ccurve_QA')
        O3_Ccurve_QA = h5d_read(dataid)
        get_data_attributes,dataid,size(O3_Ccurve_QA,/type),attrib26a
        h5d_close,dataid
    endif

    dataid = h5d_open(fieldid,'Pressure')
    Pressure = h5d_read(dataid)
    get_data_attributes,dataid,size(Pressure,/type),attrib26
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMax')
    RadianceResidualMax = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMax,/type),attrib27
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMean')
    RadianceResidualMean = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMean,/type),attrib28
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualRMS')
    RadianceResidualRMS = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualRMS,/type),attrib29
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Scan_Averaged_Count')
    Scan_Averaged_Count = h5d_read(dataid)
    get_data_attributes,dataid,size(Scan_Averaged_Count,/type),attrib30
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalConverged')
    SpeciesRetrievalConverged = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalConverged,/type),attrib31
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalQuality')
    SpeciesRetrievalQuality = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalQuality,/type),attrib32
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceEmissMean_QA')
    SurfaceEmissMean_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceEmissMean_QA,/type),attrib33
    h5d_close,dataid

;XXXX
    dataid = h5d_open(fieldid,'SurfaceEmissionLayer_QA')
    SurfaceEmissionLayer_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceEmissionLayer_QA,/type),attrib50
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempDegreesOfFreedom')
    SurfaceTempDegreesOfFreedom = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempDegreesOfFreedom,/type),attrib35
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempError')
    SurfaceTempError = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempError,/type),attrib36
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempInitial')
    SurfaceTempInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempInitial,/type),attrib37
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempObservationError')
    SurfaceTempObservationError = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempObservationError,/type),attrib38
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempPrecision')
    SurfaceTempPrecision = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempPrecision,/type),attrib39
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempVsApriori_QA')
    SurfaceTempVsApriori_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempVsApriori_QA,/type),attrib40
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempVsAtmTemp_QA')
    SurfaceTempVsAtmTemp_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempVsAtmTemp_QA,/type),attrib41
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTemperature')
    SurfaceTemperature = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTemperature,/type),attrib42
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensity')
    TotalColumnDensity = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib44
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityError')
    TotalColumnDensityError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityError,/type),attrib45
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityInitial')
    TotalColumnDensityInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityInitial,/type),attrib46
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalError')
    TotalError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalError,/type),attrib47
    h5d_close,dataid

    dataid = h5d_open(fieldid,'UTCTime')
    UTCTime = h5d_read(dataid)
    get_data_attributes,dataid,size(utctime,/type),attrib49,/utctime
    h5d_close,dataid

    h5g_close,fieldid

    if version eq 1 then begin
        species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AverageCloudEffOpticalDepthError : temporary(AverageCloudEffOpticalDepthError), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               Scan_Averaged_Count : temporary(Scan_Averaged_Count), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               SurfaceEmissMean_QA : temporary(SurfaceEmissMean_QA), $    
;XXXX
               SurfaceEmissionLayer_QA : temporary(SurfaceEmissionLayer_QA), $    
               SurfaceTempDegreesOfFreedom : temporary(SurfaceTempDegreesOfFreedom), $    
               SurfaceTempError : temporary(SurfaceTempError), $
               SurfaceTempInitial : temporary(SurfaceTempInitial), $    
               SurfaceTempObservationError : temporary(SurfaceTempObservationError), $    
               SurfaceTempPrecision : temporary(SurfaceTempPrecision), $    
               SurfaceTempVsApriori_QA : temporary(SurfaceTempVsApriori_QA), $    
               SurfaceTempVsAtmTemp_QA : temporary(SurfaceTempVsAtmTemp_QA), $    
               SurfaceTemperature : temporary(SurfaceTemperature), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               UTCTime : temporary(UTCTime) $
                   }
    endif
    if version ge 2 then begin
        species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AverageCloudEffOpticalDepthError : temporary(AverageCloudEffOpticalDepthError), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               O3_Ccurve_QA : temporary(O3_Ccurve_QA), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               Scan_Averaged_Count : temporary(Scan_Averaged_Count), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               SurfaceEmissMean_QA : temporary(SurfaceEmissMean_QA), $    
;XXXX
               SurfaceEmissionLayer_QA : temporary(SurfaceEmissionLayer_QA), $    
               SurfaceTempDegreesOfFreedom : temporary(SurfaceTempDegreesOfFreedom), $    
               SurfaceTempError : temporary(SurfaceTempError), $
               SurfaceTempInitial : temporary(SurfaceTempInitial), $    
               SurfaceTempObservationError : temporary(SurfaceTempObservationError), $    
               SurfaceTempPrecision : temporary(SurfaceTempPrecision), $    
               SurfaceTempVsApriori_QA : temporary(SurfaceTempVsApriori_QA), $    
               SurfaceTempVsAtmTemp_QA : temporary(SurfaceTempVsAtmTemp_QA), $    
               SurfaceTemperature : temporary(SurfaceTemperature), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               UTCTime : temporary(UTCTime) $
                   }
    endif

;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressure= h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id
;
; read in geolocation data
;
    geoid = h5g_open(specid,'Geolocation Fields')
;
;
    dataid = h5d_open(geoid,'BoresightAzimuth')
    BoresightAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightAzimuth,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngle')
    BoresightNadirAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngle,/type),attrib1
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngleUnc')
    BoresightNadirAngleUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngleUnc,/type),attrib2
    h5d_close,dataid

    dataid = h5d_open(geoid,'DayNightFlag')
    DayNightFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(DayNightFlag,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(geoid,'DominantSurfaceType')
    DominantSurfaceType = h5d_read(dataid)
    get_data_attributes,dataid,size(DominantSurfaceType,/type),attrib6
    h5d_close,dataid

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'LocalSolarTime')
    LocalSolarTime = h5d_read(dataid)
    get_data_attributes,dataid,size(LocalSolarTime,/type),attrib12
    h5d_close,dataid

    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    get_data_attributes,dataid,size(Scan,/type),attrib18
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    get_data_attributes,dataid,size(Sequence,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(geoid,'SolarZenithAngle')
    SolarZenithAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(SolarZenithAngle,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceElevStandardDeviation')
    SurfaceElevStandardDeviation = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceElevStandardDeviation,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceTypeFootprint')
    SurfaceTypeFootprint = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTypeFootprint,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(geoid,'Tgt_SpacecraftAzimuth')
    Tgt_SpacecraftAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftAzimuth,/type),attrib23
    h5d_close,dataid

    dataid = h5d_open(geoid,'Tgt_SpacecraftZenith')
    Tgt_SpacecraftZenith = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftZenith,/type),attrib24
    h5d_close,dataid

    dataid = h5d_open(geoid,'Time')
    Time = h5d_read(dataid)
    get_data_attributes,dataid,size(Time,/type),attrib25
    h5d_close,dataid

    geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           DayNightFlag : temporary(DayNightFlag), $
           DominantSurfaceType : temporary(DominantSurfaceType), $
           Latitude : temporary(Latitude), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           SurfaceTypeFootprint : temporary(SurfaceTypeFootprint), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) $
           }


    o3_nadir= { $
                species_data : temporary(species_data), $
                geo_data : temporary(geo_data), $
                CloudFrequency : temporary(CloudFrequency), $
                EmissivityWavenumber : temporary(EmissivityWavenumber), $
                Pressure : temporary(Pressure), $
                VerticalCoordinate : temporary(VerticalCoordinate) }


endif ; o3 nadir swath


swath = 'O3LimbSwath'
swpos = where(rswaths eq swath)
nspos = where(names eq swath)
if nspos eq -1 then begin
    status = 8
    print,'WARNING:  Requested swath '+swath+' not present in file.'
endif
if swpos ne -1 and nspos ne -1 then begin
    specid = h5g_open(swid,swath)
; data fields
    fieldid = h5g_open(specid,'Data Fields')

; READ DATA
    gas = 'O3'
    dataid = h5d_open(fieldid,gas)
    species = h5d_read(dataid)
    get_data_attributes,dataid,size(species,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(fieldid,gas+'Precision')
    Precision = h5d_read(dataid)
    get_data_attributes,dataid,size(Precision,/type),attrib1
    h5d_close,dataid

; the rest, alphabetically

    dataid = h5d_open(fieldid,'Altitude')
    altitude = h5d_read(dataid)
    get_data_attributes,dataid,size(altitude,/type),attrib4
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepth')
    AverageCloudEffOpticalDepth = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepth,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepthError')
    AverageCloudEffOpticalDepthError = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepthError,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Calibration_QA')
    Calibration_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(Calibration_QA,/type),attrib8
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudTopPressure')
    CloudTopPressure = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressure,/type),attrib12
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'CloudTopPressureError')
    CloudTopPressureError = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressureError,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudVariability_QA')
    CloudVariability_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudVariability_QA,/type),attrib14
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DegreesOfFreedomForSignal')
    DegreesOfFreedomForSignal = h5d_read(dataid)
    get_data_attributes,dataid,size(DegreesOfFreedomForSignal,/type),attrib16
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DeviationVsRetrievalCovariance')
    DeviationVsRetrievalCovariance = h5d_read(dataid)
    get_data_attributes,dataid,size(DeviationVsRetrievalCovariance,/type),attrib17
    h5d_close,dataid

    dataid = h5d_open(fieldid,'H2O_HDO_Corr_QA')
    H2O_HDO_Corr_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(H2O_HDO_Corr_QA,/type),attrib51
    h5d_close,dataid

    dataid = h5d_open(fieldid,'InformationContent')
    InformationContent = h5d_read(dataid)
    get_data_attributes,dataid,size(InformationContent,/type),attrib18
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Initial')
    Initial = h5d_read(dataid)
    get_data_attributes,dataid,size(Initial,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(fieldid,'KDotDL_QA')
    KDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(KDotDL_QA,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(fieldid,'LDotDL_QA')
    LDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(LDotDL_QA,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(fieldid,'MaxNumIterations')
    MaxNumIterations = h5d_read(dataid)
    get_data_attributes,dataid,size(MaxNumIterations,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(fieldid,'NumberIterPerformed')
    NumberIterPerformed = h5d_read(dataid)
    get_data_attributes,dataid,size(NumberIterPerformed,/type),attrib24
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Pressure')
    Pressure = h5d_read(dataid)
    get_data_attributes,dataid,size(Pressure,/type),attrib26
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMax')
    RadianceResidualMax = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMax,/type),attrib27
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMean')
    RadianceResidualMean = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMean,/type),attrib28
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualRMS')
    RadianceResidualRMS = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualRMS,/type),attrib29
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalConverged')
    SpeciesRetrievalConverged = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalConverged,/type),attrib31
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalQuality')
    SpeciesRetrievalQuality = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalQuality,/type),attrib32
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensity')
    TotalColumnDensity = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib44
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityError')
    TotalColumnDensityError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityError,/type),attrib45
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityInitial')
    TotalColumnDensityInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityInitial,/type),attrib46
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalError')
    TotalError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalError,/type),attrib47
    h5d_close,dataid

    dataid = h5d_open(fieldid,'UTCTime')
    UTCTime = h5d_read(dataid)
    get_data_attributes,dataid,size(utctime,/type),attrib49,/utctime
    h5d_close,dataid

    h5g_close,fieldid

    species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AverageCloudEffOpticalDepthError : temporary(AverageCloudEffOpticalDepthError), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               UTCTime : temporary(UTCTime) $
                   }

;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressure= h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id
;
; get geolocation data
;
    geoid = h5g_open(specid,'Geolocation Fields')
;
;
    dataid = h5d_open(geoid,'BoresightAzimuth')
    BoresightAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightAzimuth,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngle')
    BoresightNadirAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngle,/type),attrib1
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngleUnc')
    BoresightNadirAngleUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngleUnc,/type),attrib2
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightTangentHeight')
    BoresightTangentHeight = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightTangentHeight,/type),attrib3
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightTangentHeightUnc')
    BoresightTangentHeightUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightTangentHeightUnc,/type),attrib4
    h5d_close,dataid

    dataid = h5d_open(geoid,'DayNightFlag')
    DayNightFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(DayNightFlag,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'LocalSolarTime')
    LocalSolarTime = h5d_read(dataid)
    get_data_attributes,dataid,size(LocalSolarTime,/type),attrib12
    h5d_close,dataid

    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    get_data_attributes,dataid,size(Scan,/type),attrib18
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    get_data_attributes,dataid,size(Sequence,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(geoid,'SolarZenithAngle')
    SolarZenithAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(SolarZenithAngle,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceElevStandardDeviation')
    SurfaceElevStandardDeviation = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceElevStandardDeviation,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceTypeFootprint')
    SurfaceTypeFootprint = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTypeFootprint,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(geoid,'Tgt_SpacecraftAzimuth')
    Tgt_SpacecraftAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftAzimuth,/type),attrib23
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Tgt_SpacecraftZenith')
    Tgt_SpacecraftZenith = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftZenith,/type),attrib24
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Time')
    Time = h5d_read(dataid)
    get_data_attributes,dataid,size(Time,/type),attrib25
    h5d_close,dataid

    geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           BoresightTangentHeight : temporary(BoresightTangentHeight), $
           BoresightTangentHeightUnc : temporary(BoresightTangentHeightUnc), $
           DayNightFlag : temporary(DayNightFlag), $
           Latitude : temporary(Latitude), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           SurfaceTypeFootprint : temporary(SurfaceTypeFootprint), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) $
           }

    o3_limb= { $
               species_data : temporary(species_data), $
               geo_data : temporary(geo_data), $
               CloudFrequency : temporary(CloudFrequency), $
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               Pressure : temporary(Pressure), $
               VerticalCoordinate : temporary(VerticalCoordinate) }


endif ; o3 limb swath


swath = 'CONadirSwath'
swpos = where(rswaths eq swath)
nspos = where(names eq swath)
if nspos eq -1 then begin
    status = 8
    print,'WARNING:  Requested swath '+swath+' not present in file.'
endif
if swpos ne -1 and nspos ne -1 then begin
    specid = h5g_open(swid,swath)
; data fields
    fieldid = h5g_open(specid,'Data Fields')


; READ DATA
    gas = 'CO'
    dataid = h5d_open(fieldid,gas)
    species = h5d_read(dataid)
    get_data_attributes,dataid,size(species,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(fieldid,gas+'Precision')
    Precision = h5d_read(dataid)
    get_data_attributes,dataid,size(Precision,/type),attrib1
    h5d_close,dataid

; the rest, alphabetically

    dataid = h5d_open(fieldid,'Altitude')
    altitude = h5d_read(dataid)
    get_data_attributes,dataid,size(altitude,/type),attrib4
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepth')
    AverageCloudEffOpticalDepth = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepth,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepthError')
    AverageCloudEffOpticalDepthError = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepthError,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Calibration_QA')
    Calibration_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(Calibration_QA,/type),attrib8
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudTopPressure')
    CloudTopPressure = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressure,/type),attrib12
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudTopPressureError')
    CloudTopPressureError = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressureError,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudVariability_QA')
    CloudVariability_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudVariability_QA,/type),attrib14
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DegreesOfFreedomForSignal')
    DegreesOfFreedomForSignal = h5d_read(dataid)
    get_data_attributes,dataid,size(DegreesOfFreedomForSignal,/type),attrib16
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DeviationVsRetrievalCovariance')
    DeviationVsRetrievalCovariance = h5d_read(dataid)
    get_data_attributes,dataid,size(DeviationVsRetrievalCovariance,/type),attrib17
    h5d_close,dataid

    dataid = h5d_open(fieldid,'H2O_HDO_Corr_QA')
    H2O_HDO_Corr_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(H2O_HDO_Corr_QA,/type),attrib51
    h5d_close,dataid

    dataid = h5d_open(fieldid,'InformationContent')
    InformationContent = h5d_read(dataid)
    get_data_attributes,dataid,size(InformationContent,/type),attrib18
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Initial')
    Initial = h5d_read(dataid)
    get_data_attributes,dataid,size(Initial,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(fieldid,'KDotDL_QA')
    KDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(KDotDL_QA,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(fieldid,'LDotDL_QA')
    LDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(LDotDL_QA,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(fieldid,'MaxNumIterations')
    MaxNumIterations = h5d_read(dataid)
    get_data_attributes,dataid,size(MaxNumIterations,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(fieldid,'NumberIterPerformed')
    NumberIterPerformed = h5d_read(dataid)
    get_data_attributes,dataid,size(NumberIterPerformed,/type),attrib24
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Pressure')
    Pressure = h5d_read(dataid)
    get_data_attributes,dataid,size(Pressure,/type),attrib26
    h5d_close,dataid

    if version ge 2 then begin
        dataid = h5d_open(fieldid,'O3_Ccurve_QA')
        O3_Ccurve_QA = h5d_read(dataid)
        get_data_attributes,dataid,size(O3_Ccurve_QA,/type),attrib26a
        h5d_close,dataid
    endif


    dataid = h5d_open(fieldid,'RadianceResidualMax')
    RadianceResidualMax = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMax,/type),attrib27
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMean')
    RadianceResidualMean = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMean,/type),attrib28
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualRMS')
    RadianceResidualRMS = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualRMS,/type),attrib29
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Scan_Averaged_Count')
    Scan_Averaged_Count = h5d_read(dataid)
    get_data_attributes,dataid,size(Scan_Averaged_Count,/type),attrib30
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalConverged')
    SpeciesRetrievalConverged = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalConverged,/type),attrib31
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalQuality')
    SpeciesRetrievalQuality = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalQuality,/type),attrib32
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceEmissMean_QA')
    SurfaceEmissMean_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceEmissMean_QA,/type),attrib33
    h5d_close,dataid

;XXXX
    dataid = h5d_open(fieldid,'SurfaceEmissionLayer_QA')
    SurfaceEmissionLayer_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceEmissionLayer_QA,/type),attrib50
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempDegreesOfFreedom')
    SurfaceTempDegreesOfFreedom = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempDegreesOfFreedom,/type),attrib35
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempError')
    SurfaceTempError = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempError,/type),attrib36
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempInitial')
    SurfaceTempInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempInitial,/type),attrib37
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempObservationError')
    SurfaceTempObservationError = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempObservationError,/type),attrib38
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempPrecision')
    SurfaceTempPrecision = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempPrecision,/type),attrib39
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempVsApriori_QA')
    SurfaceTempVsApriori_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempVsApriori_QA,/type),attrib40
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempVsAtmTemp_QA')
    SurfaceTempVsAtmTemp_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempVsAtmTemp_QA,/type),attrib41
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTemperature')
    SurfaceTemperature = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTemperature,/type),attrib42
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensity')
    TotalColumnDensity = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib44
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityError')
    TotalColumnDensityError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityError,/type),attrib45
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityInitial')
    TotalColumnDensityInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityInitial,/type),attrib46
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalError')
    TotalError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalError,/type),attrib47
    h5d_close,dataid

    dataid = h5d_open(fieldid,'UTCTime')
    UTCTime = h5d_read(dataid)
    get_data_attributes,dataid,size(utctime,/type),attrib49,/utctime
    h5d_close,dataid

    h5g_close,fieldid
    if version eq 1 then begin
        species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AverageCloudEffOpticalDepthError : temporary(AverageCloudEffOpticalDepthError), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               Scan_Averaged_Count : temporary(Scan_Averaged_Count), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               SurfaceEmissMean_QA : temporary(SurfaceEmissMean_QA), $    
;XXXX
               SurfaceEmissionLayer_QA : temporary(SurfaceEmissionLayer_QA), $    
               SurfaceTempDegreesOfFreedom : temporary(SurfaceTempDegreesOfFreedom), $    
               SurfaceTempError : temporary(SurfaceTempError), $
               SurfaceTempInitial : temporary(SurfaceTempInitial), $    
               SurfaceTempObservationError : temporary(SurfaceTempObservationError), $    
               SurfaceTempPrecision : temporary(SurfaceTempPrecision), $    
               SurfaceTempVsApriori_QA : temporary(SurfaceTempVsApriori_QA), $    
               SurfaceTempVsAtmTemp_QA : temporary(SurfaceTempVsAtmTemp_QA), $    
               SurfaceTemperature : temporary(SurfaceTemperature), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               UTCTime : temporary(UTCTime) $
                   }
    endif
    if version ge 2 then begin
        species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AverageCloudEffOpticalDepthError : temporary(AverageCloudEffOpticalDepthError), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               O3_Ccurve_QA : temporary(O3_Ccurve_QA), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               Scan_Averaged_Count : temporary(Scan_Averaged_Count), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               SurfaceEmissMean_QA : temporary(SurfaceEmissMean_QA), $    
;XXXX
               SurfaceEmissionLayer_QA : temporary(SurfaceEmissionLayer_QA), $    
               SurfaceTempDegreesOfFreedom : temporary(SurfaceTempDegreesOfFreedom), $    
               SurfaceTempError : temporary(SurfaceTempError), $
               SurfaceTempInitial : temporary(SurfaceTempInitial), $    
               SurfaceTempObservationError : temporary(SurfaceTempObservationError), $    
               SurfaceTempPrecision : temporary(SurfaceTempPrecision), $    
               SurfaceTempVsApriori_QA : temporary(SurfaceTempVsApriori_QA), $    
               SurfaceTempVsAtmTemp_QA : temporary(SurfaceTempVsAtmTemp_QA), $    
               SurfaceTemperature : temporary(SurfaceTemperature), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               UTCTime : temporary(UTCTime) $
                   }
endif
;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressure= h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id

;
; read in geolocation data
;
    geoid = h5g_open(specid,'Geolocation Fields')
;
;
    dataid = h5d_open(geoid,'BoresightAzimuth')
    BoresightAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightAzimuth,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngle')
    BoresightNadirAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngle,/type),attrib1
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngleUnc')
    BoresightNadirAngleUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngleUnc,/type),attrib2
    h5d_close,dataid

    dataid = h5d_open(geoid,'DayNightFlag')
    DayNightFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(DayNightFlag,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(geoid,'DominantSurfaceType')
    DominantSurfaceType = h5d_read(dataid)
    get_data_attributes,dataid,size(DominantSurfaceType,/type),attrib6
    h5d_close,dataid

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'LocalSolarTime')
    LocalSolarTime = h5d_read(dataid)
    get_data_attributes,dataid,size(LocalSolarTime,/type),attrib12
    h5d_close,dataid

    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    get_data_attributes,dataid,size(Scan,/type),attrib18
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    get_data_attributes,dataid,size(Sequence,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(geoid,'SolarZenithAngle')
    SolarZenithAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(SolarZenithAngle,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceElevStandardDeviation')
    SurfaceElevStandardDeviation = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceElevStandardDeviation,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceTypeFootprint')
    SurfaceTypeFootprint = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTypeFootprint,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(geoid,'Tgt_SpacecraftAzimuth')
    Tgt_SpacecraftAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftAzimuth,/type),attrib23
    h5d_close,dataid

    dataid = h5d_open(geoid,'Tgt_SpacecraftZenith')
    Tgt_SpacecraftZenith = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftZenith,/type),attrib24
    h5d_close,dataid

    dataid = h5d_open(geoid,'Time')
    Time = h5d_read(dataid)
    get_data_attributes,dataid,size(Time,/type),attrib25
    h5d_close,dataid

    geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           DayNightFlag : temporary(DayNightFlag), $
           DominantSurfaceType : temporary(DominantSurfaceType), $
           Latitude : temporary(Latitude), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           SurfaceTypeFootprint : temporary(SurfaceTypeFootprint), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) $
           }


    co_nadir= { $
                species_data : temporary(species_data), $
                geo_data : temporary(geo_data), $
                CloudFrequency : temporary(CloudFrequency), $
                EmissivityWavenumber : temporary(EmissivityWavenumber), $
                Pressure : temporary(Pressure), $
                VerticalCoordinate : temporary(VerticalCoordinate) }


endif ; co nadir swath


swath = 'COLimbSwath'
swpos = where(rswaths eq swath)
nspos = where(names eq swath)
if nspos eq -1 then begin
    status = 8
    print,'WARNING:  Requested swath '+swath+' not present in file.'
endif
if swpos ne -1 and nspos ne -1 then begin
    specid = h5g_open(swid,swath)
; data fields
    fieldid = h5g_open(specid,'Data Fields')

; READ DATA
    gas = 'CO'
    dataid = h5d_open(fieldid,gas)
    species = h5d_read(dataid)
    get_data_attributes,dataid,size(species,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(fieldid,gas+'Precision')
    Precision = h5d_read(dataid)
    get_data_attributes,dataid,size(Precision,/type),attrib1
    h5d_close,dataid

; the rest, alphabetically

    dataid = h5d_open(fieldid,'Altitude')
    altitude = h5d_read(dataid)
    get_data_attributes,dataid,size(altitude,/type),attrib4
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepth')
    AverageCloudEffOpticalDepth = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepth,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepthError')
    AverageCloudEffOpticalDepthError = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepthError,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Calibration_QA')
    Calibration_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(Calibration_QA,/type),attrib8
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudTopPressure')
    CloudTopPressure = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressure,/type),attrib12
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'CloudTopPressureError')
    CloudTopPressureError = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressureError,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudVariability_QA')
    CloudVariability_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudVariability_QA,/type),attrib14
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DegreesOfFreedomForSignal')
    DegreesOfFreedomForSignal = h5d_read(dataid)
    get_data_attributes,dataid,size(DegreesOfFreedomForSignal,/type),attrib16
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DeviationVsRetrievalCovariance')
    DeviationVsRetrievalCovariance = h5d_read(dataid)
    get_data_attributes,dataid,size(DeviationVsRetrievalCovariance,/type),attrib17
    h5d_close,dataid

    dataid = h5d_open(fieldid,'H2O_HDO_Corr_QA')
    H2O_HDO_Corr_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(H2O_HDO_Corr_QA,/type),attrib51
    h5d_close,dataid

    dataid = h5d_open(fieldid,'InformationContent')
    InformationContent = h5d_read(dataid)
    get_data_attributes,dataid,size(InformationContent,/type),attrib18
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Initial')
    Initial = h5d_read(dataid)
    get_data_attributes,dataid,size(Initial,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(fieldid,'KDotDL_QA')
    KDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(KDotDL_QA,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(fieldid,'LDotDL_QA')
    LDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(LDotDL_QA,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(fieldid,'MaxNumIterations')
    MaxNumIterations = h5d_read(dataid)
    get_data_attributes,dataid,size(MaxNumIterations,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(fieldid,'NumberIterPerformed')
    NumberIterPerformed = h5d_read(dataid)
    get_data_attributes,dataid,size(NumberIterPerformed,/type),attrib24
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Pressure')
    Pressure = h5d_read(dataid)
    get_data_attributes,dataid,size(Pressure,/type),attrib26
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMax')
    RadianceResidualMax = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMax,/type),attrib27
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMean')
    RadianceResidualMean = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMean,/type),attrib28
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualRMS')
    RadianceResidualRMS = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualRMS,/type),attrib29
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalConverged')
    SpeciesRetrievalConverged = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalConverged,/type),attrib31
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalQuality')
    SpeciesRetrievalQuality = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalQuality,/type),attrib32
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensity')
    TotalColumnDensity = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib44
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityError')
    TotalColumnDensityError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityError,/type),attrib45
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityInitial')
    TotalColumnDensityInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityInitial,/type),attrib46
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalError')
    TotalError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalError,/type),attrib47
    h5d_close,dataid

    dataid = h5d_open(fieldid,'UTCTime')
    UTCTime = h5d_read(dataid)
    get_data_attributes,dataid,size(utctime,/type),attrib49,/utctime
    h5d_close,dataid

    h5g_close,fieldid

    species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AverageCloudEffOpticalDepthError : temporary(AverageCloudEffOpticalDepthError), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               UTCTime : temporary(UTCTime) $
                   }


;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressure = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id
;
; get geolocation data
;
    geoid = h5g_open(specid,'Geolocation Fields')
;
;
    dataid = h5d_open(geoid,'BoresightAzimuth')
    BoresightAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightAzimuth,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngle')
    BoresightNadirAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngle,/type),attrib1
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngleUnc')
    BoresightNadirAngleUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngleUnc,/type),attrib2
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightTangentHeight')
    BoresightTangentHeight = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightTangentHeight,/type),attrib3
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightTangentHeightUnc')
    BoresightTangentHeightUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightTangentHeightUnc,/type),attrib4
    h5d_close,dataid

    dataid = h5d_open(geoid,'DayNightFlag')
    DayNightFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(DayNightFlag,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'LocalSolarTime')
    LocalSolarTime = h5d_read(dataid)
    get_data_attributes,dataid,size(LocalSolarTime,/type),attrib12
    h5d_close,dataid

    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    get_data_attributes,dataid,size(Scan,/type),attrib18
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    get_data_attributes,dataid,size(Sequence,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(geoid,'SolarZenithAngle')
    SolarZenithAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(SolarZenithAngle,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceElevStandardDeviation')
    SurfaceElevStandardDeviation = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceElevStandardDeviation,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceTypeFootprint')
    SurfaceTypeFootprint = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTypeFootprint,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(geoid,'Tgt_SpacecraftAzimuth')
    Tgt_SpacecraftAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftAzimuth,/type),attrib23
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Tgt_SpacecraftZenith')
    Tgt_SpacecraftZenith = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftZenith,/type),attrib24
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Time')
    Time = h5d_read(dataid)
    get_data_attributes,dataid,size(Time,/type),attrib25
    h5d_close,dataid

    geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           BoresightTangentHeight : temporary(BoresightTangentHeight), $
           BoresightTangentHeightUnc : temporary(BoresightTangentHeightUnc), $
           DayNightFlag : temporary(DayNightFlag), $
           Latitude : temporary(Latitude), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           SurfaceTypeFootprint : temporary(SurfaceTypeFootprint), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) $
           }

    co_limb= { $
               species_data : temporary(species_data), $
               geo_data : temporary(geo_data), $
               CloudFrequency : temporary(CloudFrequency), $
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               Pressure : temporary(Pressure), $
               VerticalCoordinate : temporary(VerticalCoordinate) }

endif ; co limb swath


swath = 'H2ONadirSwath'
swpos = where(rswaths eq swath)
nspos = where(names eq swath)
if nspos eq -1 then begin
    status = 8
    print,'WARNING:  Requested swath '+swath+' not present in file.'
endif
if swpos ne -1 and nspos ne -1 then begin
    specid = h5g_open(swid,swath)
; data fields
    fieldid = h5g_open(specid,'Data Fields')

; READ DATA

    gas = 'H2O'
    dataid = h5d_open(fieldid,gas)
    species = h5d_read(dataid)
    get_data_attributes,dataid,size(species,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(fieldid,gas+'Precision')
    Precision = h5d_read(dataid)
    get_data_attributes,dataid,size(Precision,/type),attrib1
    h5d_close,dataid

; the rest, alphabetically

    dataid = h5d_open(fieldid,'Altitude')
    altitude = h5d_read(dataid)
    get_data_attributes,dataid,size(altitude,/type),attrib4
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepth')
    AverageCloudEffOpticalDepth = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepth,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepthError')
    AverageCloudEffOpticalDepthError = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepthError,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Calibration_QA')
    Calibration_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(Calibration_QA,/type),attrib8
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudTopPressure')
    CloudTopPressure = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressure,/type),attrib12
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudTopPressureError')
    CloudTopPressureError = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressureError,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudVariability_QA')
    CloudVariability_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudVariability_QA,/type),attrib14
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DegreesOfFreedomForSignal')
    DegreesOfFreedomForSignal = h5d_read(dataid)
    get_data_attributes,dataid,size(DegreesOfFreedomForSignal,/type),attrib16
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DeviationVsRetrievalCovariance')
    DeviationVsRetrievalCovariance = h5d_read(dataid)
    get_data_attributes,dataid,size(DeviationVsRetrievalCovariance,/type),attrib17
    h5d_close,dataid

    dataid = h5d_open(fieldid,'H2O_HDO_Corr_QA')
    H2O_HDO_Corr_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(H2O_HDO_Corr_QA,/type),attrib51
    h5d_close,dataid

    dataid = h5d_open(fieldid,'InformationContent')
    InformationContent = h5d_read(dataid)
    get_data_attributes,dataid,size(InformationContent,/type),attrib18
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Initial')
    Initial = h5d_read(dataid)
    get_data_attributes,dataid,size(Initial,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(fieldid,'KDotDL_QA')
    KDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(KDotDL_QA,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(fieldid,'LDotDL_QA')
    LDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(LDotDL_QA,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(fieldid,'MaxNumIterations')
    MaxNumIterations = h5d_read(dataid)
    get_data_attributes,dataid,size(MaxNumIterations,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(fieldid,'NumberIterPerformed')
    NumberIterPerformed = h5d_read(dataid)
    get_data_attributes,dataid,size(NumberIterPerformed,/type),attrib24
    h5d_close,dataid

    if version ge 2 then begin
        dataid = h5d_open(fieldid,'O3_Ccurve_QA')
        O3_Ccurve_QA = h5d_read(dataid)
        get_data_attributes,dataid,size(O3_Ccurve_QA,/type),attrib26a
        h5d_close,dataid
    endif

    dataid = h5d_open(fieldid,'Pressure')
    Pressure = h5d_read(dataid)
    get_data_attributes,dataid,size(Pressure,/type),attrib26
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMax')
    RadianceResidualMax = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMax,/type),attrib27
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMean')
    RadianceResidualMean = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMean,/type),attrib28
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualRMS')
    RadianceResidualRMS = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualRMS,/type),attrib29
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Scan_Averaged_Count')
    Scan_Averaged_Count = h5d_read(dataid)
    get_data_attributes,dataid,size(Scan_Averaged_Count,/type),attrib30
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalConverged')
    SpeciesRetrievalConverged = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalConverged,/type),attrib31
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalQuality')
    SpeciesRetrievalQuality = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalQuality,/type),attrib32
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceEmissMean_QA')
    SurfaceEmissMean_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceEmissMean_QA,/type),attrib33
    h5d_close,dataid

;XXXX
    dataid = h5d_open(fieldid,'SurfaceEmissionLayer_QA')
    SurfaceEmissionLayer_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceEmissionLayer_QA,/type),attrib50
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempDegreesOfFreedom')
    SurfaceTempDegreesOfFreedom = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempDegreesOfFreedom,/type),attrib35
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempError')
    SurfaceTempError = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempError,/type),attrib36
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempInitial')
    SurfaceTempInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempInitial,/type),attrib37
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempObservationError')
    SurfaceTempObservationError = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempObservationError,/type),attrib38
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempPrecision')
    SurfaceTempPrecision = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempPrecision,/type),attrib39
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempVsApriori_QA')
    SurfaceTempVsApriori_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempVsApriori_QA,/type),attrib40
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempVsAtmTemp_QA')
    SurfaceTempVsAtmTemp_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempVsAtmTemp_QA,/type),attrib41
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTemperature')
    SurfaceTemperature = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTemperature,/type),attrib42
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensity')
    TotalColumnDensity = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib44
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityError')
    TotalColumnDensityError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityError,/type),attrib45
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityInitial')
    TotalColumnDensityInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityInitial,/type),attrib46
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalError')
    TotalError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalError,/type),attrib47
    h5d_close,dataid

    dataid = h5d_open(fieldid,'UTCTime')
    UTCTime = h5d_read(dataid)
    get_data_attributes,dataid,size(utctime,/type),attrib49,/utctime
    h5d_close,dataid

    h5g_close,fieldid
    if version eq 1 then begin
        species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AverageCloudEffOpticalDepthError : temporary(AverageCloudEffOpticalDepthError), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               Scan_Averaged_Count : temporary(Scan_Averaged_Count), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               SurfaceEmissMean_QA : temporary(SurfaceEmissMean_QA), $    
;XXXX
               SurfaceEmissionLayer_QA : temporary(SurfaceEmissionLayer_QA), $    
               SurfaceTempDegreesOfFreedom : temporary(SurfaceTempDegreesOfFreedom), $    
               SurfaceTempError : temporary(SurfaceTempError), $
               SurfaceTempInitial : temporary(SurfaceTempInitial), $    
               SurfaceTempObservationError : temporary(SurfaceTempObservationError), $    
               SurfaceTempPrecision : temporary(SurfaceTempPrecision), $    
               SurfaceTempVsApriori_QA : temporary(SurfaceTempVsApriori_QA), $    
               SurfaceTempVsAtmTemp_QA : temporary(SurfaceTempVsAtmTemp_QA), $    
               SurfaceTemperature : temporary(SurfaceTemperature), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               UTCTime : temporary(UTCTime) $
                   }
    endif
    if version ge 2 then begin
        species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AverageCloudEffOpticalDepthError : temporary(AverageCloudEffOpticalDepthError), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               O3_Ccurve_QA : temporary(O3_Ccurve_QA), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               Scan_Averaged_Count : temporary(Scan_Averaged_Count), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               SurfaceEmissMean_QA : temporary(SurfaceEmissMean_QA), $    
;XXXX
               SurfaceEmissionLayer_QA : temporary(SurfaceEmissionLayer_QA), $    
               SurfaceTempDegreesOfFreedom : temporary(SurfaceTempDegreesOfFreedom), $    
               SurfaceTempError : temporary(SurfaceTempError), $
               SurfaceTempInitial : temporary(SurfaceTempInitial), $    
               SurfaceTempObservationError : temporary(SurfaceTempObservationError), $    
               SurfaceTempPrecision : temporary(SurfaceTempPrecision), $    
               SurfaceTempVsApriori_QA : temporary(SurfaceTempVsApriori_QA), $    
               SurfaceTempVsAtmTemp_QA : temporary(SurfaceTempVsAtmTemp_QA), $    
               SurfaceTemperature : temporary(SurfaceTemperature), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               UTCTime : temporary(UTCTime) $
                   }
    endif
;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressure = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id

;
; read in geolocation data
;
    geoid = h5g_open(specid,'Geolocation Fields')
;
;
    dataid = h5d_open(geoid,'BoresightAzimuth')
    BoresightAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightAzimuth,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngle')
    BoresightNadirAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngle,/type),attrib1
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngleUnc')
    BoresightNadirAngleUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngleUnc,/type),attrib2
    h5d_close,dataid

    dataid = h5d_open(geoid,'DayNightFlag')
    DayNightFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(DayNightFlag,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(geoid,'DominantSurfaceType')
    DominantSurfaceType = h5d_read(dataid)
    get_data_attributes,dataid,size(DominantSurfaceType,/type),attrib6
    h5d_close,dataid

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'LocalSolarTime')
    LocalSolarTime = h5d_read(dataid)
    get_data_attributes,dataid,size(LocalSolarTime,/type),attrib12
    h5d_close,dataid

    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    get_data_attributes,dataid,size(Scan,/type),attrib18
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    get_data_attributes,dataid,size(Sequence,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(geoid,'SolarZenithAngle')
    SolarZenithAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(SolarZenithAngle,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceElevStandardDeviation')
    SurfaceElevStandardDeviation = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceElevStandardDeviation,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceTypeFootprint')
    SurfaceTypeFootprint = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTypeFootprint,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(geoid,'Tgt_SpacecraftAzimuth')
    Tgt_SpacecraftAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftAzimuth,/type),attrib23
    h5d_close,dataid

    dataid = h5d_open(geoid,'Tgt_SpacecraftZenith')
    Tgt_SpacecraftZenith = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftZenith,/type),attrib24
    h5d_close,dataid

    dataid = h5d_open(geoid,'Time')
    Time = h5d_read(dataid)
    get_data_attributes,dataid,size(Time,/type),attrib25
    h5d_close,dataid

    geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           DayNightFlag : temporary(DayNightFlag), $
           DominantSurfaceType : temporary(DominantSurfaceType), $
           Latitude : temporary(Latitude), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           SurfaceTypeFootprint : temporary(SurfaceTypeFootprint), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) $
           }


    h2o_nadir= { $
                species_data : temporary(species_data), $
                geo_data : temporary(geo_data), $
                CloudFrequency : temporary(CloudFrequency), $
                EmissivityWavenumber : temporary(EmissivityWavenumber), $
                Pressure : temporary(Pressure), $
                VerticalCoordinate : temporary(VerticalCoordinate) }


endif ; h2o nadir


swath = 'H2OLimbSwath'
swpos = where(rswaths eq swath)
nspos = where(names eq swath)
if nspos eq -1 then begin
    status = 8
    print,'WARNING:  Requested swath '+swath+' not present in file.'
endif
if swpos ne -1 and nspos ne -1 then begin
    specid = h5g_open(swid,swath)
; data fields
    fieldid = h5g_open(specid,'Data Fields')

; READ DATA
    gas = 'H2O'
    dataid = h5d_open(fieldid,gas)
    species = h5d_read(dataid)
    get_data_attributes,dataid,size(species,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(fieldid,gas+'Precision')
    Precision = h5d_read(dataid)
    get_data_attributes,dataid,size(Precision,/type),attrib1
    h5d_close,dataid

; the rest, alphabetically

    dataid = h5d_open(fieldid,'Altitude')
    altitude = h5d_read(dataid)
    get_data_attributes,dataid,size(altitude,/type),attrib4
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepth')
    AverageCloudEffOpticalDepth = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepth,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepthError')
    AverageCloudEffOpticalDepthError = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepthError,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Calibration_QA')
    Calibration_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(Calibration_QA,/type),attrib8
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudTopPressure')
    CloudTopPressure = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressure,/type),attrib12
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'CloudTopPressureError')
    CloudTopPressureError = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressureError,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudVariability_QA')
    CloudVariability_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudVariability_QA,/type),attrib14
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DegreesOfFreedomForSignal')
    DegreesOfFreedomForSignal = h5d_read(dataid)
    get_data_attributes,dataid,size(DegreesOfFreedomForSignal,/type),attrib16
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DeviationVsRetrievalCovariance')
    DeviationVsRetrievalCovariance = h5d_read(dataid)
    get_data_attributes,dataid,size(DeviationVsRetrievalCovariance,/type),attrib17
    h5d_close,dataid

    dataid = h5d_open(fieldid,'H2O_HDO_Corr_QA')
    H2O_HDO_Corr_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(H2O_HDO_Corr_QA,/type),attrib51
    h5d_close,dataid

    dataid = h5d_open(fieldid,'InformationContent')
    InformationContent = h5d_read(dataid)
    get_data_attributes,dataid,size(InformationContent,/type),attrib18
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Initial')
    Initial = h5d_read(dataid)
    get_data_attributes,dataid,size(Initial,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(fieldid,'KDotDL_QA')
    KDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(KDotDL_QA,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(fieldid,'LDotDL_QA')
    LDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(LDotDL_QA,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(fieldid,'MaxNumIterations')
    MaxNumIterations = h5d_read(dataid)
    get_data_attributes,dataid,size(MaxNumIterations,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(fieldid,'NumberIterPerformed')
    NumberIterPerformed = h5d_read(dataid)
    get_data_attributes,dataid,size(NumberIterPerformed,/type),attrib24
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Pressure')
    Pressure = h5d_read(dataid)
    get_data_attributes,dataid,size(Pressure,/type),attrib26
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMax')
    RadianceResidualMax = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMax,/type),attrib27
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMean')
    RadianceResidualMean = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMean,/type),attrib28
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualRMS')
    RadianceResidualRMS = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualRMS,/type),attrib29
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalConverged')
    SpeciesRetrievalConverged = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalConverged,/type),attrib31
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalQuality')
    SpeciesRetrievalQuality = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalQuality,/type),attrib32
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensity')
    TotalColumnDensity = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib44
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityError')
    TotalColumnDensityError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityError,/type),attrib45
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityInitial')
    TotalColumnDensityInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityInitial,/type),attrib46
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalError')
    TotalError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalError,/type),attrib47
    h5d_close,dataid

    dataid = h5d_open(fieldid,'UTCTime')
    UTCTime = h5d_read(dataid)
    get_data_attributes,dataid,size(utctime,/type),attrib49,/utctime
    h5d_close,dataid

    h5g_close,fieldid

    species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AverageCloudEffOpticalDepthError : temporary(AverageCloudEffOpticalDepthError), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               UTCTime : temporary(UTCTime) $
                   }


;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressure = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id

;
; get geolocation data
;
    geoid = h5g_open(specid,'Geolocation Fields')
;
;
    dataid = h5d_open(geoid,'BoresightAzimuth')
    BoresightAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightAzimuth,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngle')
    BoresightNadirAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngle,/type),attrib1
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngleUnc')
    BoresightNadirAngleUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngleUnc,/type),attrib2
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightTangentHeight')
    BoresightTangentHeight = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightTangentHeight,/type),attrib3
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightTangentHeightUnc')
    BoresightTangentHeightUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightTangentHeightUnc,/type),attrib4
    h5d_close,dataid

    dataid = h5d_open(geoid,'DayNightFlag')
    DayNightFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(DayNightFlag,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'LocalSolarTime')
    LocalSolarTime = h5d_read(dataid)
    get_data_attributes,dataid,size(LocalSolarTime,/type),attrib12
    h5d_close,dataid

    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    get_data_attributes,dataid,size(Scan,/type),attrib18
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    get_data_attributes,dataid,size(Sequence,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(geoid,'SolarZenithAngle')
    SolarZenithAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(SolarZenithAngle,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceElevStandardDeviation')
    SurfaceElevStandardDeviation = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceElevStandardDeviation,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceTypeFootprint')
    SurfaceTypeFootprint = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTypeFootprint,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(geoid,'Tgt_SpacecraftAzimuth')
    Tgt_SpacecraftAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftAzimuth,/type),attrib23
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Tgt_SpacecraftZenith')
    Tgt_SpacecraftZenith = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftZenith,/type),attrib24
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Time')
    Time = h5d_read(dataid)
    get_data_attributes,dataid,size(Time,/type),attrib25
    h5d_close,dataid

    geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           BoresightTangentHeight : temporary(BoresightTangentHeight), $
           BoresightTangentHeightUnc : temporary(BoresightTangentHeightUnc), $
           DayNightFlag : temporary(DayNightFlag), $
           Latitude : temporary(Latitude), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           SurfaceTypeFootprint : temporary(SurfaceTypeFootprint), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) $
           }

    h2o_limb= { $
               species_data : temporary(species_data), $
               geo_data : temporary(geo_data), $
               CloudFrequency : temporary(CloudFrequency), $
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               Pressure : temporary(Pressure), $
               VerticalCoordinate : temporary(VerticalCoordinate) }

endif
                            ; h2o limb swath


swath = 'HDONadirSwath'
swpos = where(rswaths eq swath)
nspos = where(names eq swath)
if nspos eq -1 then begin
    status = 8
    print,'WARNING:  Requested swath '+swath+' not present in file.'
endif
if swpos ne -1 and nspos ne -1 then begin
    specid = h5g_open(swid,swath)
; data fields
    fieldid = h5g_open(specid,'Data Fields')

; READ DATA
    gas = 'HDO'
    dataid = h5d_open(fieldid,gas)
    species = h5d_read(dataid)
    get_data_attributes,dataid,size(species,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(fieldid,gas+'Precision')
    Precision = h5d_read(dataid)
    get_data_attributes,dataid,size(Precision,/type),attrib1
    h5d_close,dataid

; the rest, alphabetically

    dataid = h5d_open(fieldid,'Altitude')
    altitude = h5d_read(dataid)
    get_data_attributes,dataid,size(altitude,/type),attrib4
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepth')
    AverageCloudEffOpticalDepth = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepth,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepthError')
    AverageCloudEffOpticalDepthError = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepthError,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Calibration_QA')
    Calibration_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(Calibration_QA,/type),attrib8
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudTopPressure')
    CloudTopPressure = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressure,/type),attrib12
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudTopPressureError')
    CloudTopPressureError = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressureError,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudVariability_QA')
    CloudVariability_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudVariability_QA,/type),attrib14
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DegreesOfFreedomForSignal')
    DegreesOfFreedomForSignal = h5d_read(dataid)
    get_data_attributes,dataid,size(DegreesOfFreedomForSignal,/type),attrib16
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DeviationVsRetrievalCovariance')
    DeviationVsRetrievalCovariance = h5d_read(dataid)
    get_data_attributes,dataid,size(DeviationVsRetrievalCovariance,/type),attrib17
    h5d_close,dataid

    dataid = h5d_open(fieldid,'H2O_HDO_Corr_QA')
    H2O_HDO_Corr_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(H2O_HDO_Corr_QA,/type),attrib51
    h5d_close,dataid

    dataid = h5d_open(fieldid,'InformationContent')
    InformationContent = h5d_read(dataid)
    get_data_attributes,dataid,size(InformationContent,/type),attrib18
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Initial')
    Initial = h5d_read(dataid)
    get_data_attributes,dataid,size(Initial,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(fieldid,'KDotDL_QA')
    KDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(KDotDL_QA,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(fieldid,'LDotDL_QA')
    LDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(LDotDL_QA,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(fieldid,'MaxNumIterations')
    MaxNumIterations = h5d_read(dataid)
    get_data_attributes,dataid,size(MaxNumIterations,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(fieldid,'NumberIterPerformed')
    NumberIterPerformed = h5d_read(dataid)
    get_data_attributes,dataid,size(NumberIterPerformed,/type),attrib24
    h5d_close,dataid

    if version ge 2 then begin
        dataid = h5d_open(fieldid,'O3_Ccurve_QA')
        O3_Ccurve_QA = h5d_read(dataid)
        get_data_attributes,dataid,size(O3_Ccurve_QA,/type),attrib26a
        h5d_close,dataid
    endif

    dataid = h5d_open(fieldid,'Pressure')
    Pressure = h5d_read(dataid)
    get_data_attributes,dataid,size(Pressure,/type),attrib26
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMax')
    RadianceResidualMax = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMax,/type),attrib27
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMean')
    RadianceResidualMean = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMean,/type),attrib28
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualRMS')
    RadianceResidualRMS = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualRMS,/type),attrib29
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Scan_Averaged_Count')
    Scan_Averaged_Count = h5d_read(dataid)
    get_data_attributes,dataid,size(Scan_Averaged_Count,/type),attrib30
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalConverged')
    SpeciesRetrievalConverged = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalConverged,/type),attrib31
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalQuality')
    SpeciesRetrievalQuality = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalQuality,/type),attrib32
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceEmissMean_QA')
    SurfaceEmissMean_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceEmissMean_QA,/type),attrib33
    h5d_close,dataid

;XXXX
    dataid = h5d_open(fieldid,'SurfaceEmissionLayer_QA')
    SurfaceEmissionLayer_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceEmissionLayer_QA,/type),attrib50
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempDegreesOfFreedom')
    SurfaceTempDegreesOfFreedom = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempDegreesOfFreedom,/type),attrib35
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempError')
    SurfaceTempError = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempError,/type),attrib36
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempInitial')
    SurfaceTempInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempInitial,/type),attrib37
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempObservationError')
    SurfaceTempObservationError = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempObservationError,/type),attrib38
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempPrecision')
    SurfaceTempPrecision = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempPrecision,/type),attrib39
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempVsApriori_QA')
    SurfaceTempVsApriori_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempVsApriori_QA,/type),attrib40
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempVsAtmTemp_QA')
    SurfaceTempVsAtmTemp_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempVsAtmTemp_QA,/type),attrib41
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTemperature')
    SurfaceTemperature = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTemperature,/type),attrib42
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensity')
    TotalColumnDensity = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib44
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityError')
    TotalColumnDensityError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityError,/type),attrib45
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityInitial')
    TotalColumnDensityInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityInitial,/type),attrib46
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalError')
    TotalError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalError,/type),attrib47
    h5d_close,dataid

    dataid = h5d_open(fieldid,'UTCTime')
    UTCTime = h5d_read(dataid)
    get_data_attributes,dataid,size(utctime,/type),attrib49,/utctime
    h5d_close,dataid

    h5g_close,fieldid
    if version eq 1 then begin
        species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AverageCloudEffOpticalDepthError : temporary(AverageCloudEffOpticalDepthError), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               Scan_Averaged_Count : temporary(Scan_Averaged_Count), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               SurfaceEmissMean_QA : temporary(SurfaceEmissMean_QA), $    
;XXXX
               SurfaceEmissionLayer_QA : temporary(SurfaceEmissionLayer_QA), $    
               SurfaceTempDegreesOfFreedom : temporary(SurfaceTempDegreesOfFreedom), $    
               SurfaceTempError : temporary(SurfaceTempError), $
               SurfaceTempInitial : temporary(SurfaceTempInitial), $    
               SurfaceTempObservationError : temporary(SurfaceTempObservationError), $    
               SurfaceTempPrecision : temporary(SurfaceTempPrecision), $    
               SurfaceTempVsApriori_QA : temporary(SurfaceTempVsApriori_QA), $    
               SurfaceTempVsAtmTemp_QA : temporary(SurfaceTempVsAtmTemp_QA), $    
               SurfaceTemperature : temporary(SurfaceTemperature), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               UTCTime : temporary(UTCTime) $
                   }
    endif
    if version ge 2 then begin
        species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AverageCloudEffOpticalDepthError : temporary(AverageCloudEffOpticalDepthError), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               O3_Ccurve_QA : temporary(O3_Ccurve_QA), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               Scan_Averaged_Count : temporary(Scan_Averaged_Count), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               SurfaceEmissMean_QA : temporary(SurfaceEmissMean_QA), $    
;XXXX
               SurfaceEmissionLayer_QA : temporary(SurfaceEmissionLayer_QA), $    
               SurfaceTempDegreesOfFreedom : temporary(SurfaceTempDegreesOfFreedom), $    
               SurfaceTempError : temporary(SurfaceTempError), $
               SurfaceTempInitial : temporary(SurfaceTempInitial), $    
               SurfaceTempObservationError : temporary(SurfaceTempObservationError), $    
               SurfaceTempPrecision : temporary(SurfaceTempPrecision), $    
               SurfaceTempVsApriori_QA : temporary(SurfaceTempVsApriori_QA), $    
               SurfaceTempVsAtmTemp_QA : temporary(SurfaceTempVsAtmTemp_QA), $    
               SurfaceTemperature : temporary(SurfaceTemperature), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               UTCTime : temporary(UTCTime) $
                   }
    endif
;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressure = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id

;
; read in geolocation data
;
    geoid = h5g_open(specid,'Geolocation Fields')
;
;
    dataid = h5d_open(geoid,'BoresightAzimuth')
    BoresightAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightAzimuth,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngle')
    BoresightNadirAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngle,/type),attrib1
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngleUnc')
    BoresightNadirAngleUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngleUnc,/type),attrib2
    h5d_close,dataid

    dataid = h5d_open(geoid,'DayNightFlag')
    DayNightFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(DayNightFlag,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(geoid,'DominantSurfaceType')
    DominantSurfaceType = h5d_read(dataid)
    get_data_attributes,dataid,size(DominantSurfaceType,/type),attrib6
    h5d_close,dataid

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'LocalSolarTime')
    LocalSolarTime = h5d_read(dataid)
    get_data_attributes,dataid,size(LocalSolarTime,/type),attrib12
    h5d_close,dataid

    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    get_data_attributes,dataid,size(Scan,/type),attrib18
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    get_data_attributes,dataid,size(Sequence,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(geoid,'SolarZenithAngle')
    SolarZenithAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(SolarZenithAngle,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceElevStandardDeviation')
    SurfaceElevStandardDeviation = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceElevStandardDeviation,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceTypeFootprint')
    SurfaceTypeFootprint = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTypeFootprint,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(geoid,'Tgt_SpacecraftAzimuth')
    Tgt_SpacecraftAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftAzimuth,/type),attrib23
    h5d_close,dataid

    dataid = h5d_open(geoid,'Tgt_SpacecraftZenith')
    Tgt_SpacecraftZenith = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftZenith,/type),attrib24
    h5d_close,dataid

    dataid = h5d_open(geoid,'Time')
    Time = h5d_read(dataid)
    get_data_attributes,dataid,size(Time,/type),attrib25
    h5d_close,dataid

    geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           DayNightFlag : temporary(DayNightFlag), $
           DominantSurfaceType : temporary(DominantSurfaceType), $
           Latitude : temporary(Latitude), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           SurfaceTypeFootprint : temporary(SurfaceTypeFootprint), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) $
           }


    hdo_nadir= { $
                species_data : temporary(species_data), $
                geo_data : temporary(geo_data), $
                CloudFrequency : temporary(CloudFrequency), $
                EmissivityWavenumber : temporary(EmissivityWavenumber), $
                Pressure : temporary(Pressure), $
                VerticalCoordinate : temporary(VerticalCoordinate) }


endif ; hdo nadir swath


swath = 'HDOLimbSwath'
swpos = where(rswaths eq swath)
nspos = where(names eq swath)
if nspos eq -1 then begin
    status = 8
    print,'WARNING:  Requested swath '+swath+' not present in file.'
endif
if swpos ne -1 and nspos ne -1 then begin
    specid = h5g_open(swid,swath)
; data fields
    fieldid = h5g_open(specid,'Data Fields')

; READ DATA
    gas = 'HDO'
    dataid = h5d_open(fieldid,gas)
    species = h5d_read(dataid)
    get_data_attributes,dataid,size(species,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(fieldid,gas+'Precision')
    Precision = h5d_read(dataid)
    get_data_attributes,dataid,size(Precision,/type),attrib1
    h5d_close,dataid

; the rest, alphabetically

    dataid = h5d_open(fieldid,'Altitude')
    altitude = h5d_read(dataid)
    get_data_attributes,dataid,size(altitude,/type),attrib4
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepth')
    AverageCloudEffOpticalDepth = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepth,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepthError')
    AverageCloudEffOpticalDepthError = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepthError,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Calibration_QA')
    Calibration_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(Calibration_QA,/type),attrib8
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudTopPressure')
    CloudTopPressure = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressure,/type),attrib12
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'CloudTopPressureError')
    CloudTopPressureError = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressureError,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudVariability_QA')
    CloudVariability_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudVariability_QA,/type),attrib14
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DegreesOfFreedomForSignal')
    DegreesOfFreedomForSignal = h5d_read(dataid)
    get_data_attributes,dataid,size(DegreesOfFreedomForSignal,/type),attrib16
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DeviationVsRetrievalCovariance')
    DeviationVsRetrievalCovariance = h5d_read(dataid)
    get_data_attributes,dataid,size(DeviationVsRetrievalCovariance,/type),attrib17
    h5d_close,dataid

    dataid = h5d_open(fieldid,'H2O_HDO_Corr_QA')
    H2O_HDO_Corr_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(H2O_HDO_Corr_QA,/type),attrib51
    h5d_close,dataid

    dataid = h5d_open(fieldid,'InformationContent')
    InformationContent = h5d_read(dataid)
    get_data_attributes,dataid,size(InformationContent,/type),attrib18
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Initial')
    Initial = h5d_read(dataid)
    get_data_attributes,dataid,size(Initial,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(fieldid,'KDotDL_QA')
    KDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(KDotDL_QA,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(fieldid,'LDotDL_QA')
    LDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(LDotDL_QA,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(fieldid,'MaxNumIterations')
    MaxNumIterations = h5d_read(dataid)
    get_data_attributes,dataid,size(MaxNumIterations,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(fieldid,'NumberIterPerformed')
    NumberIterPerformed = h5d_read(dataid)
    get_data_attributes,dataid,size(NumberIterPerformed,/type),attrib24
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Pressure')
    Pressure = h5d_read(dataid)
    get_data_attributes,dataid,size(Pressure,/type),attrib26
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMax')
    RadianceResidualMax = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMax,/type),attrib27
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMean')
    RadianceResidualMean = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMean,/type),attrib28
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualRMS')
    RadianceResidualRMS = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualRMS,/type),attrib29
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalConverged')
    SpeciesRetrievalConverged = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalConverged,/type),attrib31
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalQuality')
    SpeciesRetrievalQuality = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalQuality,/type),attrib32
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensity')
    TotalColumnDensity = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib44
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityError')
    TotalColumnDensityError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityError,/type),attrib45
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityInitial')
    TotalColumnDensityInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityInitial,/type),attrib46
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalError')
    TotalError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalError,/type),attrib47
    h5d_close,dataid

    dataid = h5d_open(fieldid,'UTCTime')
    UTCTime = h5d_read(dataid)
    get_data_attributes,dataid,size(utctime,/type),attrib49,/utctime
    h5d_close,dataid

    h5g_close,fieldid

    species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AverageCloudEffOpticalDepthError : temporary(AverageCloudEffOpticalDepthError), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               UTCTime : temporary(UTCTime) $
                   }

;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressure = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id
;
; get geolocation data
;
    geoid = h5g_open(specid,'Geolocation Fields')
;
;
    dataid = h5d_open(geoid,'BoresightAzimuth')
    BoresightAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightAzimuth,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngle')
    BoresightNadirAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngle,/type),attrib1
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngleUnc')
    BoresightNadirAngleUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngleUnc,/type),attrib2
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightTangentHeight')
    BoresightTangentHeight = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightTangentHeight,/type),attrib3
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightTangentHeightUnc')
    BoresightTangentHeightUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightTangentHeightUnc,/type),attrib4
    h5d_close,dataid

    dataid = h5d_open(geoid,'DayNightFlag')
    DayNightFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(DayNightFlag,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'LocalSolarTime')
    LocalSolarTime = h5d_read(dataid)
    get_data_attributes,dataid,size(LocalSolarTime,/type),attrib12
    h5d_close,dataid

    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    get_data_attributes,dataid,size(Scan,/type),attrib18
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    get_data_attributes,dataid,size(Sequence,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(geoid,'SolarZenithAngle')
    SolarZenithAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(SolarZenithAngle,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceElevStandardDeviation')
    SurfaceElevStandardDeviation = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceElevStandardDeviation,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceTypeFootprint')
    SurfaceTypeFootprint = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTypeFootprint,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(geoid,'Tgt_SpacecraftAzimuth')
    Tgt_SpacecraftAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftAzimuth,/type),attrib23
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Tgt_SpacecraftZenith')
    Tgt_SpacecraftZenith = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftZenith,/type),attrib24
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Time')
    Time = h5d_read(dataid)
    get_data_attributes,dataid,size(Time,/type),attrib25
    h5d_close,dataid

    geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           BoresightTangentHeight : temporary(BoresightTangentHeight), $
           BoresightTangentHeightUnc : temporary(BoresightTangentHeightUnc), $
           DayNightFlag : temporary(DayNightFlag), $
           Latitude : temporary(Latitude), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           SurfaceTypeFootprint : temporary(SurfaceTypeFootprint), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) $
           }

    hdo_limb= { $
               species_data : temporary(species_data), $
               geo_data : temporary(geo_data), $
               CloudFrequency : temporary(CloudFrequency), $
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               Pressure : temporary(Pressure), $
               VerticalCoordinate : temporary(VerticalCoordinate) }


endif ; hdo limb swath
 

swath = 'CH4NadirSwath'
swpos = where(rswaths eq swath)
nspos = where(names eq swath)
if nspos eq -1 then begin
    status = 8
    print,'WARNING:  Requested swath '+swath+' not present in file.'
endif
if swpos ne -1 and nspos ne -1 then begin
    specid = h5g_open(swid,swath)
; data fields
    fieldid = h5g_open(specid,'Data Fields')

; READ DATA
    gas = 'CH4'
    dataid = h5d_open(fieldid,gas)
    species = h5d_read(dataid)
    get_data_attributes,dataid,size(species,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(fieldid,gas+'Precision')
    Precision = h5d_read(dataid)
    get_data_attributes,dataid,size(Precision,/type),attrib1
    h5d_close,dataid

; the rest, alphabetically

    dataid = h5d_open(fieldid,'Altitude')
    altitude = h5d_read(dataid)
    get_data_attributes,dataid,size(altitude,/type),attrib4
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepth')
    AverageCloudEffOpticalDepth = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepth,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepthError')
    AverageCloudEffOpticalDepthError = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepthError,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Calibration_QA')
    Calibration_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(Calibration_QA,/type),attrib8
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudTopPressure')
    CloudTopPressure = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressure,/type),attrib12
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudTopPressureError')
    CloudTopPressureError = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressureError,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudVariability_QA')
    CloudVariability_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudVariability_QA,/type),attrib14
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DegreesOfFreedomForSignal')
    DegreesOfFreedomForSignal = h5d_read(dataid)
    get_data_attributes,dataid,size(DegreesOfFreedomForSignal,/type),attrib16
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DeviationVsRetrievalCovariance')
    DeviationVsRetrievalCovariance = h5d_read(dataid)
    get_data_attributes,dataid,size(DeviationVsRetrievalCovariance,/type),attrib17
    h5d_close,dataid

    dataid = h5d_open(fieldid,'H2O_HDO_Corr_QA')
    H2O_HDO_Corr_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(H2O_HDO_Corr_QA,/type),attrib51
    h5d_close,dataid

    dataid = h5d_open(fieldid,'InformationContent')
    InformationContent = h5d_read(dataid)
    get_data_attributes,dataid,size(InformationContent,/type),attrib18
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Initial')
    Initial = h5d_read(dataid)
    get_data_attributes,dataid,size(Initial,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(fieldid,'KDotDL_QA')
    KDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(KDotDL_QA,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(fieldid,'LDotDL_QA')
    LDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(LDotDL_QA,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(fieldid,'MaxNumIterations')
    MaxNumIterations = h5d_read(dataid)
    get_data_attributes,dataid,size(MaxNumIterations,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(fieldid,'NumberIterPerformed')
    NumberIterPerformed = h5d_read(dataid)
    get_data_attributes,dataid,size(NumberIterPerformed,/type),attrib24
    h5d_close,dataid

    if version ge 2 then begin
        dataid = h5d_open(fieldid,'O3_Ccurve_QA')
        O3_Ccurve_QA = h5d_read(dataid)
        get_data_attributes,dataid,size(O3_Ccurve_QA,/type),attrib26a
        h5d_close,dataid
    endif

    dataid = h5d_open(fieldid,'Pressure')
    Pressure = h5d_read(dataid)
    get_data_attributes,dataid,size(Pressure,/type),attrib26
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMax')
    RadianceResidualMax = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMax,/type),attrib27
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMean')
    RadianceResidualMean = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMean,/type),attrib28
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualRMS')
    RadianceResidualRMS = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualRMS,/type),attrib29
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Scan_Averaged_Count')
    Scan_Averaged_Count = h5d_read(dataid)
    get_data_attributes,dataid,size(Scan_Averaged_Count,/type),attrib30
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalConverged')
    SpeciesRetrievalConverged = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalConverged,/type),attrib31
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalQuality')
    SpeciesRetrievalQuality = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalQuality,/type),attrib32
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceEmissMean_QA')
    SurfaceEmissMean_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceEmissMean_QA,/type),attrib33
    h5d_close,dataid
;XXXX
    dataid = h5d_open(fieldid,'SurfaceEmissionLayer_QA')
    SurfaceEmissionLayer_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceEmissionLayer_QA,/type),attrib50
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempDegreesOfFreedom')
    SurfaceTempDegreesOfFreedom = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempDegreesOfFreedom,/type),attrib35
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempError')
    SurfaceTempError = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempError,/type),attrib36
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempInitial')
    SurfaceTempInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempInitial,/type),attrib37
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempObservationError')
    SurfaceTempObservationError = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempObservationError,/type),attrib38
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempPrecision')
    SurfaceTempPrecision = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempPrecision,/type),attrib39
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempVsApriori_QA')
    SurfaceTempVsApriori_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempVsApriori_QA,/type),attrib40
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempVsAtmTemp_QA')
    SurfaceTempVsAtmTemp_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempVsAtmTemp_QA,/type),attrib41
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTemperature')
    SurfaceTemperature = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTemperature,/type),attrib42
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensity')
    TotalColumnDensity = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib44
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityError')
    TotalColumnDensityError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityError,/type),attrib45
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityInitial')
    TotalColumnDensityInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityInitial,/type),attrib46
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalError')
    TotalError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalError,/type),attrib47
    h5d_close,dataid

    dataid = h5d_open(fieldid,'UTCTime')
    UTCTime = h5d_read(dataid)
    get_data_attributes,dataid,size(utctime,/type),attrib49,/utctime
    h5d_close,dataid

    h5g_close,fieldid
    if version eq 1 then begin
        species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AverageCloudEffOpticalDepthError : temporary(AverageCloudEffOpticalDepthError), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               Scan_Averaged_Count : temporary(Scan_Averaged_Count), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               SurfaceEmissMean_QA : temporary(SurfaceEmissMean_QA), $    
;XXXX
               SurfaceEmissionLayer_QA : temporary(SurfaceEmissionLayer_QA), $    
               SurfaceTempDegreesOfFreedom : temporary(SurfaceTempDegreesOfFreedom), $    
               SurfaceTempError : temporary(SurfaceTempError), $
               SurfaceTempInitial : temporary(SurfaceTempInitial), $    
               SurfaceTempObservationError : temporary(SurfaceTempObservationError), $    
               SurfaceTempPrecision : temporary(SurfaceTempPrecision), $    
               SurfaceTempVsApriori_QA : temporary(SurfaceTempVsApriori_QA), $    
               SurfaceTempVsAtmTemp_QA : temporary(SurfaceTempVsAtmTemp_QA), $    
               SurfaceTemperature : temporary(SurfaceTemperature), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               UTCTime : temporary(UTCTime) $
                   }
    endif
    if version ge 2 then begin
        species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AverageCloudEffOpticalDepthError : temporary(AverageCloudEffOpticalDepthError), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               O3_Ccurve_QA : temporary(O3_Ccurve_QA), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               Scan_Averaged_Count : temporary(Scan_Averaged_Count), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               SurfaceEmissMean_QA : temporary(SurfaceEmissMean_QA), $    
;XXXX
               SurfaceEmissionLayer_QA : temporary(SurfaceEmissionLayer_QA), $    
               SurfaceTempDegreesOfFreedom : temporary(SurfaceTempDegreesOfFreedom), $    
               SurfaceTempError : temporary(SurfaceTempError), $
               SurfaceTempInitial : temporary(SurfaceTempInitial), $    
               SurfaceTempObservationError : temporary(SurfaceTempObservationError), $    
               SurfaceTempPrecision : temporary(SurfaceTempPrecision), $    
               SurfaceTempVsApriori_QA : temporary(SurfaceTempVsApriori_QA), $    
               SurfaceTempVsAtmTemp_QA : temporary(SurfaceTempVsAtmTemp_QA), $    
               SurfaceTemperature : temporary(SurfaceTemperature), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               UTCTime : temporary(UTCTime) $
                   }
    endif
;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressure = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id

;
; read in geolocation data
;
    geoid = h5g_open(specid,'Geolocation Fields')
;
;
    dataid = h5d_open(geoid,'BoresightAzimuth')
    BoresightAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightAzimuth,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngle')
    BoresightNadirAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngle,/type),attrib1
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngleUnc')
    BoresightNadirAngleUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngleUnc,/type),attrib2
    h5d_close,dataid

    dataid = h5d_open(geoid,'DayNightFlag')
    DayNightFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(DayNightFlag,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(geoid,'DominantSurfaceType')
    DominantSurfaceType = h5d_read(dataid)
    get_data_attributes,dataid,size(DominantSurfaceType,/type),attrib6
    h5d_close,dataid

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'LocalSolarTime')
    LocalSolarTime = h5d_read(dataid)
    get_data_attributes,dataid,size(LocalSolarTime,/type),attrib12
    h5d_close,dataid

    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    get_data_attributes,dataid,size(Scan,/type),attrib18
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    get_data_attributes,dataid,size(Sequence,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(geoid,'SolarZenithAngle')
    SolarZenithAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(SolarZenithAngle,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceElevStandardDeviation')
    SurfaceElevStandardDeviation = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceElevStandardDeviation,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceTypeFootprint')
    SurfaceTypeFootprint = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTypeFootprint,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(geoid,'Tgt_SpacecraftAzimuth')
    Tgt_SpacecraftAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftAzimuth,/type),attrib23
    h5d_close,dataid

    dataid = h5d_open(geoid,'Tgt_SpacecraftZenith')
    Tgt_SpacecraftZenith = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftZenith,/type),attrib24
    h5d_close,dataid

    dataid = h5d_open(geoid,'Time')
    Time = h5d_read(dataid)
    get_data_attributes,dataid,size(Time,/type),attrib25
    h5d_close,dataid

    geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           DayNightFlag : temporary(DayNightFlag), $
           DominantSurfaceType : temporary(DominantSurfaceType), $
           Latitude : temporary(Latitude), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           SurfaceTypeFootprint : temporary(SurfaceTypeFootprint), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) $
           }

    ch4_nadir= { $
                species_data : temporary(species_data), $
                geo_data : temporary(geo_data), $
                CloudFrequency : temporary(CloudFrequency), $
                EmissivityWavenumber : temporary(EmissivityWavenumber), $
                Pressure : temporary(Pressure), $
                VerticalCoordinate : temporary(VerticalCoordinate) }
endif ; ch4 nadir swath


swath = 'CH4LimbSwath'
swpos = where(rswaths eq swath)
nspos = where(names eq swath)
if nspos eq -1 then begin
    status = 8
    print,'WARNING:  Requested swath '+swath+' not present in file.'
endif
if swpos ne -1 and nspos ne -1 then begin
    specid = h5g_open(swid,swath)
; data fields
    fieldid = h5g_open(specid,'Data Fields')

; READ DATA
    gas = 'CH4'
    dataid = h5d_open(fieldid,gas)
    species = h5d_read(dataid)
    get_data_attributes,dataid,size(species,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(fieldid,gas+'Precision')
    Precision = h5d_read(dataid)
    get_data_attributes,dataid,size(Precision,/type),attrib1
    h5d_close,dataid

; the rest, alphabetically

    dataid = h5d_open(fieldid,'Altitude')
    altitude = h5d_read(dataid)
    get_data_attributes,dataid,size(altitude,/type),attrib4
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepth')
    AverageCloudEffOpticalDepth = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepth,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepthError')
    AverageCloudEffOpticalDepthError = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepthError,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Calibration_QA')
    Calibration_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(Calibration_QA,/type),attrib8
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudTopPressure')
    CloudTopPressure = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressure,/type),attrib12
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'CloudTopPressureError')
    CloudTopPressureError = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressureError,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudVariability_QA')
    CloudVariability_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudVariability_QA,/type),attrib14
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DegreesOfFreedomForSignal')
    DegreesOfFreedomForSignal = h5d_read(dataid)
    get_data_attributes,dataid,size(DegreesOfFreedomForSignal,/type),attrib16
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DeviationVsRetrievalCovariance')
    DeviationVsRetrievalCovariance = h5d_read(dataid)
    get_data_attributes,dataid,size(DeviationVsRetrievalCovariance,/type),attrib17
    h5d_close,dataid

    dataid = h5d_open(fieldid,'H2O_HDO_Corr_QA')
    H2O_HDO_Corr_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(H2O_HDO_Corr_QA,/type),attrib51
    h5d_close,dataid

    dataid = h5d_open(fieldid,'InformationContent')
    InformationContent = h5d_read(dataid)
    get_data_attributes,dataid,size(InformationContent,/type),attrib18
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Initial')
    Initial = h5d_read(dataid)
    get_data_attributes,dataid,size(Initial,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(fieldid,'KDotDL_QA')
    KDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(KDotDL_QA,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(fieldid,'LDotDL_QA')
    LDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(LDotDL_QA,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(fieldid,'MaxNumIterations')
    MaxNumIterations = h5d_read(dataid)
    get_data_attributes,dataid,size(MaxNumIterations,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(fieldid,'NumberIterPerformed')
    NumberIterPerformed = h5d_read(dataid)
    get_data_attributes,dataid,size(NumberIterPerformed,/type),attrib24
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Pressure')
    Pressure = h5d_read(dataid)
    get_data_attributes,dataid,size(Pressure,/type),attrib26
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMax')
    RadianceResidualMax = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMax,/type),attrib27
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMean')
    RadianceResidualMean = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMean,/type),attrib28
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualRMS')
    RadianceResidualRMS = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualRMS,/type),attrib29
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalConverged')
    SpeciesRetrievalConverged = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalConverged,/type),attrib31
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalQuality')
    SpeciesRetrievalQuality = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalQuality,/type),attrib32
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensity')
    TotalColumnDensity = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib44
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityError')
    TotalColumnDensityError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityError,/type),attrib45
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityInitial')
    TotalColumnDensityInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityInitial,/type),attrib46
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalError')
    TotalError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalError,/type),attrib47
    h5d_close,dataid

    dataid = h5d_open(fieldid,'UTCTime')
    UTCTime = h5d_read(dataid)
    get_data_attributes,dataid,size(utctime,/type),attrib49,/utctime
    h5d_close,dataid

    h5g_close,fieldid

    species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AverageCloudEffOpticalDepthError : temporary(AverageCloudEffOpticalDepthError), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               UTCTime : temporary(UTCTime) $
                   }
;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressure = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id
;
; get geolocation data
;
    geoid = h5g_open(specid,'Geolocation Fields')
;
;
    dataid = h5d_open(geoid,'BoresightAzimuth')
    BoresightAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightAzimuth,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngle')
    BoresightNadirAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngle,/type),attrib1
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngleUnc')
    BoresightNadirAngleUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngleUnc,/type),attrib2
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightTangentHeight')
    BoresightTangentHeight = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightTangentHeight,/type),attrib3
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightTangentHeightUnc')
    BoresightTangentHeightUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightTangentHeightUnc,/type),attrib4
    h5d_close,dataid

    dataid = h5d_open(geoid,'DayNightFlag')
    DayNightFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(DayNightFlag,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'LocalSolarTime')
    LocalSolarTime = h5d_read(dataid)
    get_data_attributes,dataid,size(LocalSolarTime,/type),attrib12
    h5d_close,dataid

    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    get_data_attributes,dataid,size(Scan,/type),attrib18
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    get_data_attributes,dataid,size(Sequence,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(geoid,'SolarZenithAngle')
    SolarZenithAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(SolarZenithAngle,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceElevStandardDeviation')
    SurfaceElevStandardDeviation = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceElevStandardDeviation,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceTypeFootprint')
    SurfaceTypeFootprint = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTypeFootprint,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(geoid,'Tgt_SpacecraftAzimuth')
    Tgt_SpacecraftAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftAzimuth,/type),attrib23
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Tgt_SpacecraftZenith')
    Tgt_SpacecraftZenith = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftZenith,/type),attrib24
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Time')
    Time = h5d_read(dataid)
    get_data_attributes,dataid,size(Time,/type),attrib25
    h5d_close,dataid

    geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           BoresightTangentHeight : temporary(BoresightTangentHeight), $
           BoresightTangentHeightUnc : temporary(BoresightTangentHeightUnc), $
           DayNightFlag : temporary(DayNightFlag), $
           Latitude : temporary(Latitude), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           SurfaceTypeFootprint : temporary(SurfaceTypeFootprint), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) $
           }

    ch4_limb= { $
               species_data : temporary(species_data), $
               geo_data : temporary(geo_data), $
               CloudFrequency : temporary(CloudFrequency), $
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               Pressure : temporary(Pressure), $
               VerticalCoordinate : temporary(VerticalCoordinate) }

endif ; ch4 limb swath

swath = 'HNO3LimbSwath'
swpos = where(rswaths eq swath)
nspos = where(names eq swath)
if nspos eq -1 then begin
    status = 8
    print,'WARNING:  Requested swath '+swath+' not present in file.'
endif
if swpos ne -1 and nspos ne -1 then begin
    specid = h5g_open(swid,swath)
; data fields
    fieldid = h5g_open(specid,'Data Fields')

; READ DATA
    gas = 'HNO3'
    dataid = h5d_open(fieldid,gas)
    species = h5d_read(dataid)
    get_data_attributes,dataid,size(species,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(fieldid,gas+'Precision')
    Precision = h5d_read(dataid)
    get_data_attributes,dataid,size(Precision,/type),attrib1
    h5d_close,dataid

; the rest, alphabetically

    dataid = h5d_open(fieldid,'Altitude')
    altitude = h5d_read(dataid)
    get_data_attributes,dataid,size(altitude,/type),attrib4
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepth')
    AverageCloudEffOpticalDepth = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepth,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepthError')
    AverageCloudEffOpticalDepthError = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepthError,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Calibration_QA')
    Calibration_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(Calibration_QA,/type),attrib8
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudTopPressure')
    CloudTopPressure = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressure,/type),attrib12
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'CloudTopPressureError')
    CloudTopPressureError = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressureError,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudVariability_QA')
    CloudVariability_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudVariability_QA,/type),attrib14
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DegreesOfFreedomForSignal')
    DegreesOfFreedomForSignal = h5d_read(dataid)
    get_data_attributes,dataid,size(DegreesOfFreedomForSignal,/type),attrib16
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DeviationVsRetrievalCovariance')
    DeviationVsRetrievalCovariance = h5d_read(dataid)
    get_data_attributes,dataid,size(DeviationVsRetrievalCovariance,/type),attrib17
    h5d_close,dataid

    dataid = h5d_open(fieldid,'H2O_HDO_Corr_QA')
    H2O_HDO_Corr_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(H2O_HDO_Corr_QA,/type),attrib51
    h5d_close,dataid

    dataid = h5d_open(fieldid,'InformationContent')
    InformationContent = h5d_read(dataid)
    get_data_attributes,dataid,size(InformationContent,/type),attrib18
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Initial')
    Initial = h5d_read(dataid)
    get_data_attributes,dataid,size(Initial,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(fieldid,'KDotDL_QA')
    KDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(KDotDL_QA,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(fieldid,'LDotDL_QA')
    LDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(LDotDL_QA,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(fieldid,'MaxNumIterations')
    MaxNumIterations = h5d_read(dataid)
    get_data_attributes,dataid,size(MaxNumIterations,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(fieldid,'NumberIterPerformed')
    NumberIterPerformed = h5d_read(dataid)
    get_data_attributes,dataid,size(NumberIterPerformed,/type),attrib24
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Pressure')
    Pressure = h5d_read(dataid)
    get_data_attributes,dataid,size(Pressure,/type),attrib26
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMax')
    RadianceResidualMax = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMax,/type),attrib27
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMean')
    RadianceResidualMean = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMean,/type),attrib28
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualRMS')
    RadianceResidualRMS = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualRMS,/type),attrib29
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalConverged')
    SpeciesRetrievalConverged = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalConverged,/type),attrib31
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalQuality')
    SpeciesRetrievalQuality = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalQuality,/type),attrib32
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensity')
    TotalColumnDensity = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib44
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityError')
    TotalColumnDensityError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityError,/type),attrib45
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityInitial')
    TotalColumnDensityInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityInitial,/type),attrib46
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalError')
    TotalError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalError,/type),attrib47
    h5d_close,dataid

    dataid = h5d_open(fieldid,'UTCTime')
    UTCTime = h5d_read(dataid)
    get_data_attributes,dataid,size(utctime,/type),attrib49,/utctime
    h5d_close,dataid

    h5g_close,fieldid

    species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AverageCloudEffOpticalDepthError : temporary(AverageCloudEffOpticalDepthError), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               UTCTime : temporary(UTCTime) $
                   }
;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressure = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id
;
; get geolocation data
;
    geoid = h5g_open(specid,'Geolocation Fields')
;
;
    dataid = h5d_open(geoid,'BoresightAzimuth')
    BoresightAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightAzimuth,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngle')
    BoresightNadirAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngle,/type),attrib1
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngleUnc')
    BoresightNadirAngleUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngleUnc,/type),attrib2
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightTangentHeight')
    BoresightTangentHeight = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightTangentHeight,/type),attrib3
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightTangentHeightUnc')
    BoresightTangentHeightUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightTangentHeightUnc,/type),attrib4
    h5d_close,dataid

    dataid = h5d_open(geoid,'DayNightFlag')
    DayNightFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(DayNightFlag,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'LocalSolarTime')
    LocalSolarTime = h5d_read(dataid)
    get_data_attributes,dataid,size(LocalSolarTime,/type),attrib12
    h5d_close,dataid

    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    get_data_attributes,dataid,size(Scan,/type),attrib18
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    get_data_attributes,dataid,size(Sequence,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(geoid,'SolarZenithAngle')
    SolarZenithAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(SolarZenithAngle,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceElevStandardDeviation')
    SurfaceElevStandardDeviation = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceElevStandardDeviation,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceTypeFootprint')
    SurfaceTypeFootprint = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTypeFootprint,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(geoid,'Tgt_SpacecraftAzimuth')
    Tgt_SpacecraftAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftAzimuth,/type),attrib23
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Tgt_SpacecraftZenith')
    Tgt_SpacecraftZenith = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftZenith,/type),attrib24
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Time')
    Time = h5d_read(dataid)
    get_data_attributes,dataid,size(Time,/type),attrib25
    h5d_close,dataid

    geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           BoresightTangentHeight : temporary(BoresightTangentHeight), $
           BoresightTangentHeightUnc : temporary(BoresightTangentHeightUnc), $
           DayNightFlag : temporary(DayNightFlag), $
           Latitude : temporary(Latitude), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           SurfaceTypeFootprint : temporary(SurfaceTypeFootprint), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) $
           }

    hno3_limb= { $
               species_data : temporary(species_data), $
               geo_data : temporary(geo_data), $
               CloudFrequency : temporary(CloudFrequency), $
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               Pressure : temporary(Pressure), $
               VerticalCoordinate : temporary(VerticalCoordinate) }

endif ; hno3 limb swath

swath = 'NO2LimbSwath'
swpos = where(rswaths eq swath)
nspos = where(names eq swath)
if nspos eq -1 then begin
    status = 8
    print,'WARNING:  Requested swath '+swath+' not present in file.'
endif
if swpos ne -1 and nspos ne -1 then begin
    specid = h5g_open(swid,swath)
; data fields
    fieldid = h5g_open(specid,'Data Fields')

; READ DATA
    gas = 'NO2'
    dataid = h5d_open(fieldid,gas)
    species = h5d_read(dataid)
    get_data_attributes,dataid,size(species,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(fieldid,gas+'Precision')
    Precision = h5d_read(dataid)
    get_data_attributes,dataid,size(Precision,/type),attrib1
    h5d_close,dataid

; the rest, alphabetically

    dataid = h5d_open(fieldid,'Altitude')
    altitude = h5d_read(dataid)
    get_data_attributes,dataid,size(altitude,/type),attrib4
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepth')
    AverageCloudEffOpticalDepth = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepth,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepthError')
    AverageCloudEffOpticalDepthError = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepthError,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Calibration_QA')
    Calibration_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(Calibration_QA,/type),attrib8
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudTopPressure')
    CloudTopPressure = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressure,/type),attrib12
    h5d_close,dataid
    
    dataid = h5d_open(fieldid,'CloudTopPressureError')
    CloudTopPressureError = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressureError,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudVariability_QA')
    CloudVariability_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudVariability_QA,/type),attrib14
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DegreesOfFreedomForSignal')
    DegreesOfFreedomForSignal = h5d_read(dataid)
    get_data_attributes,dataid,size(DegreesOfFreedomForSignal,/type),attrib16
    h5d_close,dataid

    dataid = h5d_open(fieldid,'DeviationVsRetrievalCovariance')
    DeviationVsRetrievalCovariance = h5d_read(dataid)
    get_data_attributes,dataid,size(DeviationVsRetrievalCovariance,/type),attrib17
    h5d_close,dataid

    dataid = h5d_open(fieldid,'H2O_HDO_Corr_QA')
    H2O_HDO_Corr_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(H2O_HDO_Corr_QA,/type),attrib51
    h5d_close,dataid

    dataid = h5d_open(fieldid,'InformationContent')
    InformationContent = h5d_read(dataid)
    get_data_attributes,dataid,size(InformationContent,/type),attrib18
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Initial')
    Initial = h5d_read(dataid)
    get_data_attributes,dataid,size(Initial,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(fieldid,'KDotDL_QA')
    KDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(KDotDL_QA,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(fieldid,'LDotDL_QA')
    LDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(LDotDL_QA,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(fieldid,'MaxNumIterations')
    MaxNumIterations = h5d_read(dataid)
    get_data_attributes,dataid,size(MaxNumIterations,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(fieldid,'NumberIterPerformed')
    NumberIterPerformed = h5d_read(dataid)
    get_data_attributes,dataid,size(NumberIterPerformed,/type),attrib24
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Pressure')
    Pressure = h5d_read(dataid)
    get_data_attributes,dataid,size(Pressure,/type),attrib26
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMax')
    RadianceResidualMax = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMax,/type),attrib27
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualMean')
    RadianceResidualMean = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualMean,/type),attrib28
    h5d_close,dataid

    dataid = h5d_open(fieldid,'RadianceResidualRMS')
    RadianceResidualRMS = h5d_read(dataid)
    get_data_attributes,dataid,size(RadianceResidualRMS,/type),attrib29
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalConverged')
    SpeciesRetrievalConverged = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalConverged,/type),attrib31
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SpeciesRetrievalQuality')
    SpeciesRetrievalQuality = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalQuality,/type),attrib32
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensity')
    TotalColumnDensity = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib44
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityError')
    TotalColumnDensityError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityError,/type),attrib45
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalColumnDensityInitial')
    TotalColumnDensityInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityInitial,/type),attrib46
    h5d_close,dataid

    dataid = h5d_open(fieldid,'TotalError')
    TotalError = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalError,/type),attrib47
    h5d_close,dataid

    dataid = h5d_open(fieldid,'UTCTime')
    UTCTime = h5d_read(dataid)
    get_data_attributes,dataid,size(utctime,/type),attrib49,/utctime
    h5d_close,dataid

    h5g_close,fieldid

    species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AverageCloudEffOpticalDepthError : temporary(AverageCloudEffOpticalDepthError), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               UTCTime : temporary(UTCTime) $
                   }

;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressure = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id
;
; get geolocation data
;
    geoid = h5g_open(specid,'Geolocation Fields')
;
;
    dataid = h5d_open(geoid,'BoresightAzimuth')
    BoresightAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightAzimuth,/type),attrib0
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngle')
    BoresightNadirAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngle,/type),attrib1
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightNadirAngleUnc')
    BoresightNadirAngleUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightNadirAngleUnc,/type),attrib2
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightTangentHeight')
    BoresightTangentHeight = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightTangentHeight,/type),attrib3
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightTangentHeightUnc')
    BoresightTangentHeightUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightTangentHeightUnc,/type),attrib4
    h5d_close,dataid

    dataid = h5d_open(geoid,'DayNightFlag')
    DayNightFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(DayNightFlag,/type),attrib5
    h5d_close,dataid

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Latitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'LocalSolarTime')
    LocalSolarTime = h5d_read(dataid)
    get_data_attributes,dataid,size(LocalSolarTime,/type),attrib12
    h5d_close,dataid

    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    get_data_attributes,dataid,size(Longitude,/type),attrib13
    h5d_close,dataid

    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    get_data_attributes,dataid,size(Scan,/type),attrib18
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    get_data_attributes,dataid,size(Sequence,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(geoid,'SolarZenithAngle')
    SolarZenithAngle = h5d_read(dataid)
    get_data_attributes,dataid,size(SolarZenithAngle,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceElevStandardDeviation')
    SurfaceElevStandardDeviation = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceElevStandardDeviation,/type),attrib21
    h5d_close,dataid

    dataid = h5d_open(geoid,'SurfaceTypeFootprint')
    SurfaceTypeFootprint = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTypeFootprint,/type),attrib22
    h5d_close,dataid

    dataid = h5d_open(geoid,'Tgt_SpacecraftAzimuth')
    Tgt_SpacecraftAzimuth = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftAzimuth,/type),attrib23
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Tgt_SpacecraftZenith')
    Tgt_SpacecraftZenith = h5d_read(dataid)
    get_data_attributes,dataid,size(Tgt_SpacecraftZenith,/type),attrib24
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Time')
    Time = h5d_read(dataid)
    get_data_attributes,dataid,size(Time,/type),attrib25
    h5d_close,dataid

    geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           BoresightTangentHeight : temporary(BoresightTangentHeight), $
           BoresightTangentHeightUnc : temporary(BoresightTangentHeightUnc), $
           DayNightFlag : temporary(DayNightFlag), $
           Latitude : temporary(Latitude), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           SurfaceTypeFootprint : temporary(SurfaceTypeFootprint), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) $
           }

    no2_limb= { $
               species_data : temporary(species_data), $
               geo_data : temporary(geo_data), $
               CloudFrequency : temporary(CloudFrequency), $
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               Pressure : temporary(Pressure), $
               VerticalCoordinate : temporary(VerticalCoordinate) }
endif ; no2_limb swath

;
; get 'Additional' information
;
aid = h5g_open(hdfid,'ADDITIONAL')
faaid = h5g_open(aid,'FILE_ATTRIBUTES')

if read_ph ne 0 then begin
    dataid = h5d_open(faaid,'ProductionHistory')
    ProductionHistory = h5d_read(dataid)
    h5d_close,dataid
endif else begin
    ProductionHistory = ''
endelse

id = h5a_open_name(faaid,'InstrumentName')
InstrumentName = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'ProcessLevel')
ProcessLevel = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'TAI93At0zOfGranule')
TAI93At0zOfGranule = h5a_read(id)
h5a_close,id
   
 
id = h5a_open_name(faaid,'GlobalSurveyNumber')
GlobalSurveyNumber = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'GranuleMonth')
GranuleMonth = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'GranuleDay')
GranuleDay = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'GranuleYear')
GranuleYear = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'SurveyMode')
SurveyMode = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'PGEVersion')
PGEVersion = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'Command_Seq_ID')
Command_Seq_ID = h5a_read(id)
h5a_close,id

; clean up
h5g_close,faaid
h5g_close,aid
h5g_close,hdfid

file_attributes = { $
                  ProductionHistory : temporary(ProductionHistory), $
                  InstrumentName : temporary(InstrumentName), $
                  ProcessLevel : temporary(ProcessLevel), $
                  TAI93At0zOfGranule : temporary(TAI93At0zOfGranule), $
                  GlobalSurveyNumber : temporary(GlobalSurveyNumber), $
                  GranuleMonth : temporary(GranuleMonth), $
                  GranuleDay : temporary(GranuleDay), $
                  GranuleYear : temporary(GranuleYear), $
                  SurveyMode : temporary(SurveyMode), $
                  PGEVersion : temporary(PGEVersion), $
                  Command_Seq_ID : temporary(Command_Seq_ID)    }

; last bit of file
hdfinfid = h5g_open(fid,'HDFEOS INFORMATION')

dataid = h5d_open(hdfinfid,'StructMetadata.0')
StructMetadata = h5d_read(dataid)
h5d_close,dataid

dataid = h5d_open(hdfinfid,'coremetadata')
coremetadata = h5d_read(dataid)
h5d_close,dataid

id = h5a_open_name(hdfinfid,'HDFEOSVersion')
HDFEOSVersion = h5a_read(id)
h5a_close,id

h5g_close,hdfinfid
h5f_close,fid

hdfeos_data = { $
              StructMetadata : temporary(StructMetadata), $
              coremetadata : temporary(coremetadata), $
              HDFEOSVersion : temporary(HDFEOSVersion) }

end
